@@ -3,12 +3,13 @@ package com.fasterxml.jackson.databind;
 import java.util.*;
 
 import com.fasterxml.jackson.core.*;
-
+import com.fasterxml.jackson.core.json.JsonReadFeature;
 import com.fasterxml.jackson.databind.cfg.*;
 import com.fasterxml.jackson.databind.deser.DeserializationProblemHandler;
 import com.fasterxml.jackson.databind.introspect.*;
 import com.fasterxml.jackson.databind.jsontype.*;
 import com.fasterxml.jackson.databind.node.JsonNodeFactory;
+import com.fasterxml.jackson.databind.type.LogicalType;
 import com.fasterxml.jackson.databind.util.LinkedNode;
 import com.fasterxml.jackson.databind.util.RootNameLookup;
 
@@ -28,6 +29,9 @@ public final class DeserializationConfig
     // since 2.9
     private static final long serialVersionUID = 2;
 
+    // since 2.10.1
+    private final static int DESER_FEATURE_DEFAULTS = collectFeatureDefaults(DeserializationFeature.class);
+
     /*
     /**********************************************************
     /* Configured helper objects
@@ -46,6 +50,16 @@ public final class DeserializationConfig
      */
     protected final JsonNodeFactory _nodeFactory;
 
+    /**
+     * @since 2.12
+     */
+    protected final CoercionConfigs _coercionConfigs;
+
+    /**
+     * @since 2.12
+     */
+    protected final ConstructorDetector _ctorDetector;
+
     /*
     /**********************************************************
     /* Deserialization features 
@@ -95,15 +109,20 @@ public final class DeserializationConfig
 
     /**
      * Constructor used by ObjectMapper to create default configuration object instance.
+     *
+     * @since 2.14
      */
     public DeserializationConfig(BaseSettings base,
             SubtypeResolver str, SimpleMixInResolver mixins, RootNameLookup rootNames,
-            ConfigOverrides configOverrides)
+            ConfigOverrides configOverrides, CoercionConfigs coercionConfigs,
+            DatatypeFeatures datatypeFeatures)
     {
-        super(base, str, mixins, rootNames, configOverrides);
-        _deserFeatures = collectFeatureDefaults(DeserializationFeature.class);
-        _nodeFactory = JsonNodeFactory.instance;
+        super(base, str, mixins, rootNames, configOverrides, datatypeFeatures);
+        _deserFeatures = DESER_FEATURE_DEFAULTS;
         _problemHandlers = null;
+        _nodeFactory = JsonNodeFactory.instance;
+        _ctorDetector = null;
+        _coercionConfigs = coercionConfigs;
         _parserFeatures = 0;
         _parserFeaturesToChange = 0;
         _formatReadFeatures = 0;
@@ -113,16 +132,19 @@ public final class DeserializationConfig
     /**
      * Copy-constructor used for making a copy used by new {@link ObjectMapper}.
      *
-     * @since 2.9
+     * @since 2.14
      */
     protected DeserializationConfig(DeserializationConfig src,
-            SimpleMixInResolver mixins, RootNameLookup rootNames,
-            ConfigOverrides configOverrides)
+            SubtypeResolver str, SimpleMixInResolver mixins, RootNameLookup rootNames,
+            ConfigOverrides configOverrides,
+            CoercionConfigs coercionConfigs)
     {
-        super(src, mixins, rootNames, configOverrides);
+        super(src, str, mixins, rootNames, configOverrides);
         _deserFeatures = src._deserFeatures;
         _problemHandlers = src._problemHandlers;
         _nodeFactory = src._nodeFactory;
+        _ctorDetector = src._ctorDetector;
+        _coercionConfigs = coercionConfigs;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
@@ -137,20 +159,22 @@ public final class DeserializationConfig
      */
 
     private DeserializationConfig(DeserializationConfig src,
-            int mapperFeatures, int deserFeatures,
+            long mapperFeatures, int deserFeatures,
             int parserFeatures, int parserFeatureMask,
             int formatFeatures, int formatFeatureMask)
     {
         super(src, mapperFeatures);
         _deserFeatures = deserFeatures;
-        _nodeFactory = src._nodeFactory;
         _problemHandlers = src._problemHandlers;
+        _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
         _parserFeatures = parserFeatures;
         _parserFeaturesToChange = parserFeatureMask;
         _formatReadFeatures = formatFeatures;
         _formatReadFeaturesToChange = formatFeatureMask;
     }
-    
+
     /**
      * Copy constructor used to create a non-shared instance with given mix-in
      * annotation definitions and subtype resolver.
@@ -159,8 +183,10 @@ public final class DeserializationConfig
     {
         super(src, str);
         _deserFeatures = src._deserFeatures;
-        _nodeFactory = src._nodeFactory;
         _problemHandlers = src._problemHandlers;
+        _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
@@ -171,20 +197,39 @@ public final class DeserializationConfig
     {
         super(src, base);
         _deserFeatures = src._deserFeatures;
-        _nodeFactory = src._nodeFactory;
         _problemHandlers = src._problemHandlers;
+        _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
         _formatReadFeaturesToChange = src._formatReadFeaturesToChange;
     }
-    
+
     private DeserializationConfig(DeserializationConfig src, JsonNodeFactory f)
     {
         super(src);
         _deserFeatures = src._deserFeatures;
         _problemHandlers = src._problemHandlers;
         _nodeFactory = f;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
+        _parserFeatures = src._parserFeatures;
+        _parserFeaturesToChange = src._parserFeaturesToChange;
+        _formatReadFeatures = src._formatReadFeatures;
+        _formatReadFeaturesToChange = src._formatReadFeaturesToChange;
+    }
+
+    // @since 2.12
+    private DeserializationConfig(DeserializationConfig src, ConstructorDetector ctorDetector)
+    {
+        super(src);
+        _deserFeatures = src._deserFeatures;
+        _problemHandlers = src._problemHandlers;
+        _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = ctorDetector;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
@@ -198,6 +243,8 @@ public final class DeserializationConfig
         _deserFeatures = src._deserFeatures;
         _problemHandlers = problemHandlers;
         _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
@@ -210,6 +257,8 @@ public final class DeserializationConfig
         _deserFeatures = src._deserFeatures;
         _problemHandlers = src._problemHandlers;
         _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
@@ -222,6 +271,8 @@ public final class DeserializationConfig
         _deserFeatures = src._deserFeatures;
         _problemHandlers = src._problemHandlers;
         _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
@@ -234,6 +285,8 @@ public final class DeserializationConfig
         _deserFeatures = src._deserFeatures;
         _problemHandlers = src._problemHandlers;
         _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
@@ -246,6 +299,26 @@ public final class DeserializationConfig
         _deserFeatures = src._deserFeatures;
         _problemHandlers = src._problemHandlers;
         _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
+        _parserFeatures = src._parserFeatures;
+        _parserFeaturesToChange = src._parserFeaturesToChange;
+        _formatReadFeatures = src._formatReadFeatures;
+        _formatReadFeaturesToChange = src._formatReadFeaturesToChange;
+    }
+
+    /**
+     * @since 2.14
+     */
+    protected DeserializationConfig(DeserializationConfig src,
+            DatatypeFeatures datatypeFeatures)
+    {
+        super(src, datatypeFeatures);
+        _deserFeatures = src._deserFeatures;
+        _problemHandlers = src._problemHandlers;
+        _nodeFactory = src._nodeFactory;
+        _coercionConfigs = src._coercionConfigs;
+        _ctorDetector = src._ctorDetector;
         _parserFeatures = src._parserFeatures;
         _parserFeaturesToChange = src._parserFeaturesToChange;
         _formatReadFeatures = src._formatReadFeatures;
@@ -261,16 +334,21 @@ public final class DeserializationConfig
     /**********************************************************
      */
 
-    @Override // since 2.9
+    @Override
     protected final DeserializationConfig _withBase(BaseSettings newBase) {
         return (_base == newBase) ? this : new DeserializationConfig(this, newBase);
     }
 
-    @Override // since 2.9
-    protected final DeserializationConfig _withMapperFeatures(int mapperFeatures) {
+    @Override
+    protected final DeserializationConfig _withMapperFeatures(long mapperFeatures) {
         return new DeserializationConfig(this, mapperFeatures, _deserFeatures,
-                        _parserFeatures, _parserFeaturesToChange,
-                        _formatReadFeatures, _formatReadFeaturesToChange);
+                _parserFeatures, _parserFeaturesToChange,
+                _formatReadFeatures, _formatReadFeaturesToChange);
+    }
+
+    @Override
+    protected final DeserializationConfig _with(DatatypeFeatures dtFeatures) {
+        return new DeserializationConfig(this, dtFeatures);
     }
 
     /*
@@ -498,6 +576,10 @@ public final class DeserializationConfig
      */
     public DeserializationConfig with(FormatFeature feature)
     {
+        // 08-Oct-2018, tatu: Alas, complexity due to newly (2.10) refactored json-features:
+        if (feature instanceof JsonReadFeature) {
+            return _withJsonReadFeatures(feature);
+        }
         int newSet = _formatReadFeatures | feature.getMask();
         int newMask = _formatReadFeaturesToChange | feature.getMask();
         return ((_formatReadFeatures == newSet) && (_formatReadFeaturesToChange == newMask)) ? this :
@@ -514,6 +596,10 @@ public final class DeserializationConfig
      */
     public DeserializationConfig withFeatures(FormatFeature... features)
     {
+        // 08-Oct-2018, tatu: Alas, complexity due to newly (2.10) refactored json-features:
+        if (features.length > 0 && (features[0] instanceof JsonReadFeature)) {
+            return _withJsonReadFeatures(features);
+        }
         int newSet = _formatReadFeatures;
         int newMask = _formatReadFeaturesToChange;
         for (FormatFeature f : features) {
@@ -535,6 +621,10 @@ public final class DeserializationConfig
      */
     public DeserializationConfig without(FormatFeature feature)
     {
+        // 08-Oct-2018, tatu: Alas, complexity due to newly (2.10) refactored json-features:
+        if (feature instanceof JsonReadFeature) {
+            return _withoutJsonReadFeatures(feature);
+        }
         int newSet = _formatReadFeatures & ~feature.getMask();
         int newMask = _formatReadFeaturesToChange | feature.getMask();
         return ((_formatReadFeatures == newSet) && (_formatReadFeaturesToChange == newMask)) ? this :
@@ -551,6 +641,10 @@ public final class DeserializationConfig
      */
     public DeserializationConfig withoutFeatures(FormatFeature... features)
     {
+        // 08-Oct-2018, tatu: Alas, complexity due to newly (2.10) refactored json-features:
+        if (features.length > 0 && (features[0] instanceof JsonReadFeature)) {
+            return _withoutJsonReadFeatures(features);
+        }
         int newSet = _formatReadFeatures;
         int newMask = _formatReadFeaturesToChange;
         for (FormatFeature f : features) {
@@ -562,7 +656,61 @@ public final class DeserializationConfig
             new DeserializationConfig(this,  _mapperFeatures, _deserFeatures,
                     _parserFeatures, _parserFeaturesToChange,
                     newSet, newMask);
-    }    
+    }
+
+    // temporary for 2.10
+    private DeserializationConfig _withJsonReadFeatures(FormatFeature... features) {
+        int parserSet = _parserFeatures;
+        int parserMask = _parserFeaturesToChange;
+        int newSet = _formatReadFeatures;
+        int newMask = _formatReadFeaturesToChange;
+        for (FormatFeature f : features) {
+            final int mask = f.getMask();
+            newSet |= mask;
+            newMask |= mask;
+
+            if (f instanceof JsonReadFeature) {
+                JsonParser.Feature oldF = ((JsonReadFeature) f).mappedFeature();
+                if (oldF != null) {
+                    final int pmask = oldF.getMask();
+                    parserSet |= pmask;
+                    parserMask |= pmask;
+                }
+            }
+        }
+        return ((_formatReadFeatures == newSet) && (_formatReadFeaturesToChange == newMask)
+                && (_parserFeatures == parserSet) && (_parserFeaturesToChange == parserMask)
+                ) ? this :
+            new DeserializationConfig(this,  _mapperFeatures, _deserFeatures,
+                    parserSet, parserMask, newSet, newMask);
+    }
+
+    // temporary for 2.10
+    private DeserializationConfig _withoutJsonReadFeatures(FormatFeature... features) {
+        int parserSet = _parserFeatures;
+        int parserMask = _parserFeaturesToChange;
+        int newSet = _formatReadFeatures;
+        int newMask = _formatReadFeaturesToChange;
+        for (FormatFeature f : features) {
+            final int mask = f.getMask();
+            newSet &= ~mask;
+            newMask |= mask;
+
+            if (f instanceof JsonReadFeature) {
+                JsonParser.Feature oldF = ((JsonReadFeature) f).mappedFeature();
+                if (oldF != null) {
+                    final int pmask = oldF.getMask();
+                    parserSet &= ~pmask;
+                    parserMask |= pmask;
+                }
+            }
+        }
+        return ((_formatReadFeatures == newSet) && (_formatReadFeaturesToChange == newMask)
+                && (_parserFeatures == parserSet) && (_parserFeaturesToChange == parserMask)
+                ) ? this :
+            new DeserializationConfig(this,  _mapperFeatures, _deserFeatures,
+                    parserSet, parserMask, newSet, newMask);
+    }
 
     /*
     /**********************************************************
@@ -581,6 +729,16 @@ public final class DeserializationConfig
         return new DeserializationConfig(this, f);
     }
 
+    /**
+     * @since 2.12
+     */
+    public DeserializationConfig with(ConstructorDetector ctorDetector) {
+        if (_ctorDetector == ctorDetector) {
+            return this;
+        }
+        return new DeserializationConfig(this, ctorDetector);
+    }
+
     /**
      * Method that can be used to add a handler that can (try to)
      * resolve non-fatal deserialization problems.
@@ -620,13 +778,30 @@ public final class DeserializationConfig
      * 
      * @since 2.5
      */
-    public void initialize(JsonParser p) {
+    public JsonParser initialize(JsonParser p) {
         if (_parserFeaturesToChange != 0) {
             p.overrideStdFeatures(_parserFeatures, _parserFeaturesToChange);
         }
         if (_formatReadFeaturesToChange != 0) {
             p.overrideFormatFeatures(_formatReadFeatures, _formatReadFeaturesToChange);
         }
+        return p;
+    }
+
+    /**
+     * @since 2.12
+     */
+    public JsonParser initialize(JsonParser p, FormatSchema schema) {
+        if (_parserFeaturesToChange != 0) {
+            p.overrideStdFeatures(_parserFeatures, _parserFeaturesToChange);
+        }
+        if (_formatReadFeaturesToChange != 0) {
+            p.overrideFormatFeatures(_formatReadFeatures, _formatReadFeaturesToChange);
+        }
+        if (schema != null) {
+            p.setSchema(schema);
+        }
+        return p;
     }
 
     /*
@@ -644,8 +819,16 @@ public final class DeserializationConfig
         return isEnabled(DeserializationFeature.UNWRAP_ROOT_VALUE);
     }
 
-    public final boolean isEnabled(DeserializationFeature f) {
-        return (_deserFeatures & f.getMask()) != 0;
+    /**
+     * Accessor for checking whether give {@link DeserializationFeature}
+     * is enabled or not.
+     *
+     * @param feature Feature to check
+     *
+     * @return True if feature is enabled; false otherwise
+     */
+    public final boolean isEnabled(DeserializationFeature feature) {
+        return (_deserFeatures & feature.getMask()) != 0;
     }
 
     public final boolean isEnabled(JsonParser.Feature f, JsonFactory factory) {
@@ -685,7 +868,7 @@ public final class DeserializationConfig
     }
 
     /**
-     * Convenience method equivalant to:
+     * Convenience method equivalent to:
      *<code>
      *   isEnabled(DeserializationFeature.FAIL_ON_TRAILING_TOKENS)
      *</code>
@@ -696,6 +879,20 @@ public final class DeserializationConfig
         return DeserializationFeature.FAIL_ON_TRAILING_TOKENS.enabledIn(_deserFeatures);
     }
 
+    /**
+     * Accessor for checking whether give {@link DatatypeFeature}
+     * is enabled or not.
+     *
+     * @param feature Feature to check
+     *
+     * @return True if feature is enabled; false otherwise
+     *
+     * @since 2.14
+     */
+    public final boolean isEnabled(DatatypeFeature feature) {
+        return _datatypeFeatures.isEnabled(feature);
+    }
+
     /*
     /**********************************************************
     /* Other configuration
@@ -714,6 +911,16 @@ public final class DeserializationConfig
         return _nodeFactory;
     }
 
+    /**
+     * @since 2.12
+     */
+    public ConstructorDetector getConstructorDetector() {
+        if (_ctorDetector == null) {
+            return ConstructorDetector.DEFAULT;
+        }
+        return _ctorDetector;
+    }
+
     /*
     /**********************************************************
     /* Introspection methods
@@ -726,26 +933,33 @@ public final class DeserializationConfig
      *
      * @param type Type of class to be introspected
      */
-    @SuppressWarnings("unchecked")
-    public <T extends BeanDescription> T introspect(JavaType type) {
-        return (T) getClassIntrospector().forDeserialization(this, type, this);
+    public BeanDescription introspect(JavaType type) {
+        return getClassIntrospector().forDeserialization(this, type, this);
     }
 
     /**
      * Method that will introspect subset of bean properties needed to
      * construct bean instance.
      */
-    @SuppressWarnings("unchecked")
-    public <T extends BeanDescription> T introspectForCreation(JavaType type) {
-        return (T) getClassIntrospector().forCreation(this, type, this);
+    public BeanDescription introspectForCreation(JavaType type) {
+        return getClassIntrospector().forCreation(this, type, this);
+    }
+
+    /**
+     * @since 2.12
+     */
+    public BeanDescription introspectForBuilder(JavaType builderType, BeanDescription valueTypeDesc) {
+        return getClassIntrospector().forDeserializationWithBuilder(this,
+                builderType, this, valueTypeDesc);
     }
 
     /**
      * @since 2.0
+     * @deprecated Since 2.12 - use variant that takes both builder and value type
      */
-    @SuppressWarnings("unchecked")
-    public <T extends BeanDescription> T introspectForBuilder(JavaType type) {
-        return (T) getClassIntrospector().forDeserializationWithBuilder(this, type, this);
+    @Deprecated
+    public BeanDescription introspectForBuilder(JavaType type) {
+        return getClassIntrospector().forDeserializationWithBuilder(this, type, this);
     }
 
     /*
@@ -782,4 +996,53 @@ public final class DeserializationConfig
         }
         return b.buildTypeDeserializer(this, baseType, subtypes);
     }
+
+    /*
+    /**********************************************************************
+    /* CoercionConfig access
+    /**********************************************************************
+     */
+
+    /**
+     * General-purpose accessor for finding what to do when specified coercion
+     * from shape that is now always allowed to be coerced from is requested.
+     *
+     * @param targetType Logical target type of coercion
+     * @param targetClass Physical target type of coercion
+     * @param inputShape Input shape to coerce from
+     *
+     * @return CoercionAction configured for specific coercion
+     *
+     * @since 2.12
+     */
+    public CoercionAction findCoercionAction(LogicalType targetType,
+            Class<?> targetClass, CoercionInputShape inputShape)
+    {
+        return _coercionConfigs.findCoercion(this,
+                targetType, targetClass, inputShape);
+    }
+
+    /**
+     * More specialized accessor called in case of input being a blank
+     * String (one consisting of only white space characters with length of at least one).
+     * Will basically first determine if "blank as empty" is allowed: if not,
+     * returns {@code actionIfBlankNotAllowed}, otherwise returns action for
+     * {@link CoercionInputShape#EmptyString}.
+     *
+     * @param targetType Logical target type of coercion
+     * @param targetClass Physical target type of coercion
+     * @param actionIfBlankNotAllowed Return value to use in case "blanks as empty"
+     *    is not allowed
+     *
+     * @return CoercionAction configured for specified coercion from blank string
+     *
+     * @since 2.12
+     */
+    public CoercionAction findCoercionFromBlankString(LogicalType targetType,
+            Class<?> targetClass,
+            CoercionAction actionIfBlankNotAllowed)
+    {
+        return _coercionConfigs.findCoercionFromBlankString(this,
+                targetType, targetClass, actionIfBlankNotAllowed);
+    }
 }
