@@ -1,6 +1,7 @@
 package com.fasterxml.jackson.databind;
 
 import com.fasterxml.jackson.databind.cfg.ConfigFeature;
+import com.fasterxml.jackson.databind.exc.InvalidNullException;
 
 /**
  * Enumeration that defines simple on/off features that affect
@@ -33,7 +34,13 @@ public enum DeserializationFeature implements ConfigFeature
      * or {@link java.util.Collection} context) is available.
      * If enabled such values will be deserialized as {@link java.math.BigDecimal}s;
      * if disabled, will be deserialized as {@link Double}s.
-     * <p>
+     *<p>
+     * NOTE: one aspect of {@link java.math.BigDecimal} handling that may need
+     * configuring is whether trailing zeroes are trimmed:
+     * {@link com.fasterxml.jackson.databind.node.JsonNodeFactory} has
+     * {@link com.fasterxml.jackson.databind.node.JsonNodeFactory#withExactBigDecimals} for
+     * changing default behavior (default is for trailing zeroes to be trimmed).
+     *<p>
      * Feature is disabled by default, meaning that "untyped" floating
      * point numbers will by default be deserialized as {@link Double}s
      * (choice is for performance reason -- BigDecimals are slower than
@@ -67,8 +74,6 @@ public enum DeserializationFeature implements ConfigFeature
      * If enabled, such values will be deserialized as {@link java.lang.Long};
      * if disabled, they will be deserialized as "smallest" available type,
      * {@link Integer}.
-     * In addition, if enabled, trying to bind values that do not fit in {@link java.lang.Long}
-     * will throw a {@link com.fasterxml.jackson.core.JsonProcessingException}.
      *<p>
      * Note: if {@link #USE_BIG_INTEGER_FOR_INTS} is enabled, it has precedence
      * over this setting, forcing use of {@link java.math.BigInteger} for all
@@ -84,9 +89,9 @@ public enum DeserializationFeature implements ConfigFeature
     
     /**
      * Feature that determines whether JSON Array is mapped to
-     * <code>Object[]</code> or <code>List&lt;Object></code> when binding
+     * <code>Object[]</code> or {@code List<Object>} when binding
      * "untyped" objects (ones with nominal type of <code>java.lang.Object</code>).
-     * If true, binds as <code>Object[]</code>; if false, as <code>List&lt;Object></code>.
+     * If true, binds as <code>Object[]</code>; if false, as {@code List<Object>}.
      *<p>
      * Feature is disabled by default, meaning that JSON arrays are bound as
      * {@link java.util.List}s.
@@ -118,7 +123,7 @@ public enum DeserializationFeature implements ConfigFeature
     /**
      * Feature that determines whether encountering of JSON null
      * is an error when deserializing into Java primitive types
-     * (like 'int' or 'double'). If it is, a JsonProcessingException
+     * (like 'int' or 'double'). If it is, a {@link InvalidNullException}
      * is thrown to indicate this; if not, default value is used
      * (0 for 'int', 0.0 for double, same defaulting as what JVM uses).
      *<p>
@@ -143,8 +148,8 @@ public enum DeserializationFeature implements ConfigFeature
     /**
      * Feature that determines what happens when type of a polymorphic
      * value (indicated for example by {@link com.fasterxml.jackson.annotation.JsonTypeInfo})
-     * cannot be found (missing) or resolved (invalid class name, unmappable id);
-     * if enabled, an exception ir thrown; if false, null value is used instead.
+     * cannot be found (missing) or resolved (invalid class name, non-mappable id);
+     * if enabled, an exception is thrown; if false, null value is used instead.
      *<p>
      * Feature is enabled by default so that exception is thrown for missing or invalid
      * type information.
@@ -278,6 +283,12 @@ public enum DeserializationFeature implements ConfigFeature
      * However, sometimes calling application may just want "raw"
      * unchecked exceptions passed as is.
      *<p>
+     * NOTE: most of the time exceptions that may or may not be wrapped are of
+     * type {@link RuntimeException}: as mentioned earlier, various
+     * {@link java.io.IOException}s (and in particular
+     * {@link com.fasterxml.jackson.core.JacksonException}s) will
+     * always be passed as-is.
+     *<p>
      * Feature is enabled by default.
      */
     WRAP_EXCEPTIONS(true),
@@ -306,8 +317,10 @@ public enum DeserializationFeature implements ConfigFeature
      * values to the corresponding value type.  This is basically the opposite of the {@link #ACCEPT_SINGLE_VALUE_AS_ARRAY}
      * feature.  If more than one value is found in the array, a JsonMappingException is thrown.
      * <p>
+     * NOTE: only <b>single</b> wrapper Array is allowed: if multiple attempted, exception
+     * will be thrown.
      * 
-     * Feature is disabled by default
+     * Feature is disabled by default.
      * @since 2.4
      */
     UNWRAP_SINGLE_VALUE_ARRAYS(false),
@@ -395,10 +408,10 @@ public enum DeserializationFeature implements ConfigFeature
      * Feature that allows unknown Enum values to be parsed as null values. 
      * If disabled, unknown Enum values will throw exceptions.
      *<p>
-     * Note that in some cases this will basically ignore unknown Enum values;
-     * this is the keys for keys of {@link java.util.EnumMap} and values
-     * of {@link java.util.EnumSet} (because nulls are not accepted in these
-     * cases).
+     * Note that in some cases this will in effect ignore unknown {@code Enum} values,
+     * e.g. when the unknown values are used as keys of {@link java.util.EnumMap} 
+     * or values of {@link java.util.EnumSet}: this because these data structures cannot
+     * store {@code null} values.
      *<p>
      * Feature is disabled by default.
      * 
@@ -453,7 +466,7 @@ public enum DeserializationFeature implements ConfigFeature
      *<p>
      *<p>
      * Taking above into account, this feature is supported only by extension modules for
-     * Joda and Java 8 date/tyime datatypes.
+     * Joda and Java 8 date/time datatypes.
      * 
      * @since 2.2
      */
