@@ -8,6 +8,7 @@ import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.deser.*;
 import com.fasterxml.jackson.databind.deser.impl.ObjectIdReader;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
+import com.fasterxml.jackson.databind.type.LogicalType;
 import com.fasterxml.jackson.databind.util.AccessPattern;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 
@@ -82,7 +83,7 @@ public abstract class JsonDeserializer<T>
      *  after the @class. Thus, if you want your method to work correctly
      *  both with and without polymorphism, you must begin your method with:
      *  <pre>
-     *       if (p.getCurrentToken() == JsonToken.START_OBJECT) {
+     *       if (p.currentToken() == JsonToken.START_OBJECT) {
      *         p.nextToken();
      *       }
      *  </pre>
@@ -104,7 +105,7 @@ public abstract class JsonDeserializer<T>
      * @return Deserialized value
      */
     public abstract T deserialize(JsonParser p, DeserializationContext ctxt)
-        throws IOException, JsonProcessingException;
+        throws IOException, JacksonException;
 
     /**
      * Alternate deserialization method (compared to the most commonly
@@ -123,13 +124,10 @@ public abstract class JsonDeserializer<T>
      * update-existing-value operation (esp. immutable types)
      */
     public T deserialize(JsonParser p, DeserializationContext ctxt, T intoValue)
-        throws IOException
+        throws IOException, JacksonException
     {
-        if (ctxt.isEnabled(MapperFeature.IGNORE_MERGE_FOR_UNMERGEABLE)) {
-            return deserialize(p, ctxt);
-        }
-        throw new UnsupportedOperationException("Cannot update object of type "
-                +intoValue.getClass().getName()+" (by deserializer of type "+getClass().getName()+")");
+        ctxt.handleBadMerge(this);
+        return deserialize(p, ctxt);
     }
 
     /**
@@ -148,12 +146,28 @@ public abstract class JsonDeserializer<T>
      */
     public Object deserializeWithType(JsonParser p, DeserializationContext ctxt,
             TypeDeserializer typeDeserializer)
-        throws IOException
+        throws IOException, JacksonException
     {
         // We could try calling 
         return typeDeserializer.deserializeTypedFromAny(p, ctxt);
     }
 
+    /**
+     * Method similar to {@link #deserializeWithType(JsonParser,DeserializationContext,TypeDeserializer)}
+     * but called when merging value. Considered "bad merge" by default implementation,
+     * but if {@link MapperFeature#IGNORE_MERGE_FOR_UNMERGEABLE} is enabled will simple delegate to
+     * {@link #deserializeWithType(JsonParser, DeserializationContext, TypeDeserializer)}.
+     *
+     * @since 2.10
+     */
+    public Object deserializeWithType(JsonParser p, DeserializationContext ctxt,
+            TypeDeserializer typeDeserializer, T intoValue)
+        throws IOException, JacksonException
+    {
+        ctxt.handleBadMerge(this);
+        return deserializeWithType(p, ctxt, typeDeserializer);
+    }
+
     /*
     /**********************************************************
     /* Fluent factory methods for constructing decorated versions
@@ -194,7 +208,7 @@ public abstract class JsonDeserializer<T>
      */
 
     /**
-     * Method for accessing type of values this deserializer produces.
+     * Method for accessing concrete physical type of values this deserializer produces.
      * Note that this information is not guaranteed to be exact -- it
      * may be a more generic (super-type) -- but it should not be
      * incorrect (return a non-related type).
@@ -202,11 +216,26 @@ public abstract class JsonDeserializer<T>
      * Default implementation will return null, which means almost same
      * same as returning <code>Object.class</code> would; that is, that
      * nothing is known about handled type.
-     *<p>
+     *
+     * @return Physical type of values this deserializer produces, if known;
+     *    {@code null} if not
+     *
      * @since 2.3
      */
     public Class<?> handledType() { return null; }
 
+    /**
+     * Method for accessing logical type of values this deserializer produces.
+     * Typically used for further configuring handling of values, for example,
+     * to find which coercions are legal.
+     *
+     * @return Logical type of values this deserializer produces, if known;
+     *    {@code null} if not
+     *
+     * @since 2.12
+     */
+    public LogicalType logicalType() { return null; }
+
     /**
      * Method called to see if deserializer instance is cachable and
      * usable for other properties of same type (type for which instance
@@ -267,9 +296,8 @@ public abstract class JsonDeserializer<T>
      * Java null, but for some types (especially primitives) it may be
      * necessary to use non-null values.
      *<p>
-     * Since version 2.6 (in which the context argument was added), call is
-     * expected to be made each and every time a null token needs to
-     * be handled.
+     * This method may be called once, or multiple times, depending on what
+     * {@link #getNullAccessPattern()} returns.
      *<p>
      * Default implementation simply returns null.
      * 
@@ -282,8 +310,14 @@ public abstract class JsonDeserializer<T>
     }
 
     /**
-     * Default implementation indicates that "null value" to use for input null
-     * is simply Java `null` for all deserializers, unless overridden by sub-classes.
+     *  This method may be called in conjunction with calls to
+     * {@link #getNullValue(DeserializationContext)}, to check whether it needs
+     * to be called just once (static values), or each time empty value is
+     * needed.
+     *<p>
+     * Default implementation indicates that the "null value" to use for input null
+     * does not vary across uses so that {@link #getNullValue(DeserializationContext)}
+     * need not be called more than once per deserializer instance.
      * This information may be used as optimization.
      */
     @Override
@@ -294,6 +328,54 @@ public abstract class JsonDeserializer<T>
         return AccessPattern.CONSTANT;
     }
 
+    /**
+     * Method called to determine placeholder value to be used for cases
+     * where no value was obtained from input but we must pass a value
+     * nonetheless: the common case is that of Creator methods requiring
+     * passing a value for every parameter.
+     * Usually this is same as {@link #getNullValue} (which in turn
+     * is usually simply Java {@code null}), but it can be overridden
+     * for specific types: most notable scalar types must use "default"
+     * values.
+     *<p>
+     * This method needs to be called every time a determination is made.
+     *<p>
+     * Default implementation simply calls {@link #getNullValue} and
+     * returns value.
+     *
+     * @since 2.13
+     */
+    @Override
+    public Object getAbsentValue(DeserializationContext ctxt) throws JsonMappingException {
+        return getNullValue(ctxt);
+    }
+
+    /*
+    /**********************************************************
+    /* Accessors for other replacement/placeholder values
+    /**********************************************************
+     */
+
+    /**
+     * Method called to determine value to be used for "empty" values
+     * (most commonly when deserializing from empty JSON Strings).
+     * Usually this is same as {@link #getNullValue} (which in turn
+     * is usually simply Java null), but it can be overridden
+     * for specific types. Or, if type should never be converted from empty
+     * String, method can also throw an exception.
+     *<p>
+     * This method may be called once, or multiple times, depending on what
+     * {@link #getEmptyAccessPattern()} returns.
+     *<p>
+     * Default implementation simply calls {@link #getNullValue} and
+     * returns value.
+     *
+     * @since 2.6 Added to replace earlier no-arguments variant
+     */
+    public Object getEmptyValue(DeserializationContext ctxt) throws JsonMappingException {
+        return getNullValue(ctxt);
+    }
+
     /**
      * This method may be called in conjunction with calls to
      * {@link #getEmptyValue(DeserializationContext)}, to check whether it needs
@@ -311,28 +393,6 @@ public abstract class JsonDeserializer<T>
     /* Other accessors
     /**********************************************************
      */
-    
-    /**
-     * Method called to determine value to be used for "empty" values
-     * (most commonly when deserializing from empty JSON Strings).
-     * Usually this is same as {@link #getNullValue} (which in turn
-     * is usually simply Java null), but it can be overridden
-     * for types. Or, if type should never be converted from empty
-     * String, method can also throw an exception.
-     *<p>
-     * Since version 2.6 (in which the context argument was added), call is
-     * expected to be made each and every time an empty value is needed.
-     *<p>
-     * Since version 2.9 does not require return of `T` any more.
-     *<p>
-     * Default implementation simply calls {@link #getNullValue} and
-     * returns value.
-     *
-     * @since 2.6 Added to replace earlier no-arguments variant
-     */
-    public Object getEmptyValue(DeserializationContext ctxt) throws JsonMappingException {
-        return getNullValue(ctxt);
-    }
 
     /**
      * Accessor that can be used to check whether this deserializer
@@ -375,7 +435,7 @@ public abstract class JsonDeserializer<T>
      *<p>
      * Information gathered is typically used to either prevent merging update for
      * property (either by skipping, if based on global defaults; or by exception during
-     * deserialization construction if explicit attempt made) if {@link Boolean#FALSE}
+     * deserializer construction if explicit attempt made) if {@link Boolean#FALSE}
      * returned, or inclusion if {@link Boolean#TRUE} is specified. If "unknown" case
      * (<code>null</code> returned) behavior is to exclude property if global defaults
      * used; or to allow if explicit per-type or property merging is defined.
