@@ -6,6 +6,7 @@ import java.util.*;
 import java.util.concurrent.ConcurrentHashMap;
 
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.exc.StreamReadException;
 import com.fasterxml.jackson.core.filter.FilteringParserDelegate;
 import com.fasterxml.jackson.core.filter.JsonPointerBasedFilter;
 import com.fasterxml.jackson.core.filter.TokenFilter;
@@ -14,6 +15,7 @@ import com.fasterxml.jackson.core.type.ResolvedType;
 import com.fasterxml.jackson.core.type.TypeReference;
 
 import com.fasterxml.jackson.databind.cfg.ContextAttributes;
+import com.fasterxml.jackson.databind.cfg.DatatypeFeature;
 import com.fasterxml.jackson.databind.deser.DataFormatReaders;
 import com.fasterxml.jackson.databind.deser.DefaultDeserializationContext;
 import com.fasterxml.jackson.databind.deser.DeserializationProblemHandler;
@@ -135,7 +137,7 @@ public class ObjectReader
      * {@link Reader} or {@link String} input <b>will fail with exception</b>,
      * because format-detection only works on byte-sources. Also, if format
      * cannot be detect reliably (as per detector settings),
-     * a {@link JsonParseException} will be thrown).
+     * a {@link StreamReadException} will be thrown).
      * 
      * @since 2.1
      */
@@ -434,6 +436,52 @@ public class ObjectReader
         return _with(_config.withoutFeatures(features));
     }    
 
+    /*
+    /**********************************************************************
+    /* Life-cycle, fluent factory methods for DatatypeFeatures (2.14+)
+    /**********************************************************************
+     */
+
+    /**
+     * Method for constructing a new reader instance that is configured
+     * with specified feature enabled.
+     *
+     * @since 2.14
+     */
+    public ObjectReader with(DatatypeFeature feature) {
+        return _with(_config.with(feature));
+    }
+
+    /**
+     * Method for constructing a new reader instance that is configured
+     * with specified features enabled.
+     *
+     * @since 2.14
+     */
+    public ObjectReader withFeatures(DatatypeFeature... features) {
+        return _with(_config.withFeatures(features));
+    }    
+
+    /**
+     * Method for constructing a new reader instance that is configured
+     * with specified feature disabled.
+     *
+     * @since 2.14
+     */
+    public ObjectReader without(DatatypeFeature feature) {
+        return _with(_config.without(feature)); 
+    }
+
+    /**
+     * Method for constructing a new reader instance that is configured
+     * with specified features disabled.
+     *
+     * @since 2.14
+     */
+    public ObjectReader withoutFeatures(DatatypeFeature... features) {
+        return _with(_config.withoutFeatures(features));
+    }    
+
     /*
     /**********************************************************
     /* Life-cycle, fluent factory methods for JsonParser.Features
@@ -952,6 +1000,13 @@ public class ObjectReader
         return _config.isEnabled(f);
     }
 
+    /**
+     * @since 2.14
+     */
+    public boolean isEnabled(DatatypeFeature f) {
+        return _config.isEnabled(f);
+    }
+
     public boolean isEnabled(JsonParser.Feature f) {
         return _config.isEnabled(f, _parserFactory);
     }
@@ -1972,7 +2027,24 @@ public class ObjectReader
         } catch (IOException e) { // should not occur, no real i/o...
             throw JsonMappingException.fromUnexpectedIOE(e);
         }
-    }    
+    }
+
+    /**
+     * Same as {@link #treeToValue(TreeNode, Class)} but with type-resolved target value type.
+     *
+     * @since 2.13
+     */
+    public <T> T treeToValue(TreeNode n, JavaType valueType) throws JsonProcessingException
+    {
+        _assertNotNull("n", n);
+        try {
+            return readValue(treeAsTokens(n), valueType);
+        } catch (JsonProcessingException e) {
+            throw e;
+        } catch (IOException e) { // should not occur, no real i/o...
+            throw JsonMappingException.fromUnexpectedIOE(e);
+        }
+    }
 
     @Override
     public void writeValue(JsonGenerator gen, Object value) throws IOException {
@@ -2032,6 +2104,7 @@ public class ObjectReader
             } else {
                 result = ctxt.readRootValue(p, _valueType, _findRootDeserializer(ctxt), _valueToUpdate);
             }
+            // No need to consume the token as parser gets closed anyway
             if (_config.isEnabled(DeserializationFeature.FAIL_ON_TRAILING_TOKENS)) {
                 _verifyNoTrailingTokens(p, ctxt, _valueType);
             }
@@ -2047,6 +2120,12 @@ public class ObjectReader
 
     protected final JsonNode _bindAsTree(JsonParser p) throws IOException
     {
+        // 16-Apr-2021, tatu: Should usually NOT be called this way but
+        //    as per [databind#3122] should still work
+        if (_valueToUpdate != null) {
+            return (JsonNode) _bind(p, _valueToUpdate);
+        }
+        
         // Need to inline `_initForReading()` due to tree reading handling end-of-input specially
         _config.initialize(p);
         if (_schema != null) {
@@ -2069,6 +2148,8 @@ public class ObjectReader
             // Will not be called for merge (need not pass _valueToUpdate)
             resultNode = (JsonNode) ctxt.readRootValue(p, _jsonNodeType(), _findTreeDeserializer(ctxt), null);
         }
+        // Need to consume the token too
+        p.clearCurrentToken();
         if (_config.isEnabled(DeserializationFeature.FAIL_ON_TRAILING_TOKENS)) {
             _verifyNoTrailingTokens(p, ctxt, _jsonNodeType());
         }
@@ -2081,6 +2162,13 @@ public class ObjectReader
      */
     protected final JsonNode _bindAsTreeOrNull(JsonParser p) throws IOException
     {
+        // 16-Apr-2021, tatu: Should usually NOT be called this way but
+        //    as per [databind#3122] should still work
+        if (_valueToUpdate != null) {
+            return (JsonNode) _bind(p, _valueToUpdate);
+        }
+
+        // Need to inline `_initForReading()` (as per above)
         _config.initialize(p);
         if (_schema != null) {
             p.setSchema(_schema);
@@ -2100,6 +2188,8 @@ public class ObjectReader
             // Will not be called for merge (need not pass _valueToUpdate)
             resultNode = (JsonNode) ctxt.readRootValue(p, _jsonNodeType(), _findTreeDeserializer(ctxt), null);
         }
+        // Need to consume the token too
+        p.clearCurrentToken();
         if (_config.isEnabled(DeserializationFeature.FAIL_ON_TRAILING_TOKENS)) {
             _verifyNoTrailingTokens(p, ctxt, _jsonNodeType());
         }
@@ -2210,7 +2300,7 @@ public class ObjectReader
      * of given input
      */
     protected void _reportUnkownFormat(DataFormatReaders detector, DataFormatReaders.Match match)
-        throws JsonProcessingException
+        throws IOException
     {
         // 17-Aug-2015, tatu: Unfortunately, no parser/generator available so:
         throw new JsonParseException(null, "Cannot detect format from input, does not look like any of detectable formats "
@@ -2258,7 +2348,7 @@ public class ObjectReader
         return new FileInputStream(f);
     }
 
-    protected void _reportUndetectableSource(Object src) throws JsonParseException
+    protected void _reportUndetectableSource(Object src) throws StreamReadException
     {
         // 17-Aug-2015, tatu: Unfortunately, no parser/generator available so:
         throw new JsonParseException(null, "Cannot use source of type "
@@ -2275,7 +2365,7 @@ public class ObjectReader
      * Method called to locate deserializer for the passed root-level value.
      */
     protected JsonDeserializer<Object> _findRootDeserializer(DeserializationContext ctxt)
-        throws JsonMappingException
+        throws DatabindException
     {
         if (_rootDeserializer != null) {
             return _rootDeserializer;
@@ -2305,7 +2395,7 @@ public class ObjectReader
      * @since 2.6
      */
     protected JsonDeserializer<Object> _findTreeDeserializer(DeserializationContext ctxt)
-        throws JsonMappingException
+        throws DatabindException
     {
         final JavaType nodeType = _jsonNodeType();
         JsonDeserializer<Object> deser = _rootDeserializers.get(nodeType);
@@ -2342,7 +2432,7 @@ public class ObjectReader
                     _rootDeserializers.put(valueType, deser);
                 }
                 return deser;
-            } catch (JsonProcessingException e) {
+            } catch (JacksonException e) {
                 // need to swallow?
             }
         }
