@@ -5,18 +5,25 @@ import java.util.Locale;
 import java.util.TimeZone;
 
 import com.fasterxml.jackson.annotation.*;
+
 import com.fasterxml.jackson.core.Base64Variant;
 import com.fasterxml.jackson.core.SerializableString;
 import com.fasterxml.jackson.core.io.SerializedString;
 import com.fasterxml.jackson.core.type.TypeReference;
+
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.introspect.AccessorNamingStrategy;
 import com.fasterxml.jackson.databind.introspect.Annotated;
 import com.fasterxml.jackson.databind.introspect.AnnotatedClass;
 import com.fasterxml.jackson.databind.introspect.ClassIntrospector;
+import com.fasterxml.jackson.databind.introspect.NopAnnotationIntrospector;
 import com.fasterxml.jackson.databind.introspect.VisibilityChecker;
+import com.fasterxml.jackson.databind.jsontype.DefaultBaseTypeLimitingValidator;
+import com.fasterxml.jackson.databind.jsontype.PolymorphicTypeValidator;
 import com.fasterxml.jackson.databind.jsontype.SubtypeResolver;
 import com.fasterxml.jackson.databind.jsontype.TypeIdResolver;
 import com.fasterxml.jackson.databind.jsontype.TypeResolverBuilder;
+import com.fasterxml.jackson.databind.jsontype.impl.LaissezFaireSubTypeValidator;
 import com.fasterxml.jackson.databind.type.TypeFactory;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 
@@ -36,7 +43,7 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     implements ClassIntrospector.MixInResolver,
         java.io.Serializable
 {
-    private static final long serialVersionUID = 1L; // since 2.5
+    private static final long serialVersionUID = 2L; // since 2.9
 
     /**
      * @since 2.7
@@ -50,27 +57,30 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
 
     /**
      * Set of shared mapper features enabled.
+     *<p>
+     * NOTE: changed from {@code int} (in Jackson 2.12 and prior} to {@code long}
+     * (2.13 and later)
      */
-    protected final int _mapperFeatures;
-    
+    protected final long _mapperFeatures;
+
     /**
      * Immutable container object for simple configuration settings.
      */
     protected final BaseSettings _base;
-    
+
     /*
     /**********************************************************
     /* Life-cycle: constructors
     /**********************************************************
      */
 
-    protected MapperConfig(BaseSettings base, int mapperFeatures)
+    protected MapperConfig(BaseSettings base, long mapperFeatures)
     {
         _base = base;
         _mapperFeatures = mapperFeatures;
     }
 
-    protected MapperConfig(MapperConfig<T> src, int mapperFeatures)
+    protected MapperConfig(MapperConfig<T> src, long mapperFeatures)
     {
         _base = src._base;
         _mapperFeatures = mapperFeatures;
@@ -87,7 +97,7 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
         _base = src._base;
         _mapperFeatures = src._mapperFeatures;
     }
-    
+
     /**
      * Method that calculates bit set (flags) of all features that
      * are enabled by default.
@@ -137,7 +147,7 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      * serialization, deserialization)
      */
     public final boolean isEnabled(MapperFeature f) {
-        return (_mapperFeatures & f.getMask()) != 0;
+        return f.enabledIn(_mapperFeatures);
     }
 
     /**
@@ -145,7 +155,10 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      * mask are enabled.
      * 
      * @since 2.3
+     *
+     * @deprecated Since 2.13 -- no replacement
      */
+    @Deprecated
     public final boolean hasMapperFeatures(int featureMask) {
         return (_mapperFeatures & featureMask) == featureMask;
     }
@@ -209,7 +222,7 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      */
     public SerializableString compileString(String src) {
         /* 20-Jan-2014, tatu: For now we will just construct it directly, but
-         *    for 2.4 need to allow overriding to support non-standard extensions
+         *    in future should allow overriding to support non-standard extensions
          *    to be used by extensions like Afterburner.
          */
         return new SerializedString(src);
@@ -232,30 +245,25 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      * Non-final since it is actually overridden by sub-classes (for now?)
      */
     public AnnotationIntrospector getAnnotationIntrospector() {
-        return _base.getAnnotationIntrospector();
+        if (isEnabled(MapperFeature.USE_ANNOTATIONS)) {
+            return _base.getAnnotationIntrospector();
+        }
+        return NopAnnotationIntrospector.instance;
     }
 
-    /**
-     * Accessor for object used for determining whether specific property elements
-     * (method, constructors, fields) can be auto-detected based on
-     * their visibility (access modifiers). Can be changed to allow
-     * different minimum visibility levels for auto-detection. Note
-     * that this is the global handler; individual types (classes)
-     * can further override active checker used (using
-     * {@link JsonAutoDetect} annotation)
-     */
-    public VisibilityChecker<?> getDefaultVisibilityChecker() {
-        return _base.getVisibilityChecker();
-    }
-    
     public final PropertyNamingStrategy getPropertyNamingStrategy() {
         return _base.getPropertyNamingStrategy();
     }
 
+    // @since 2.12
+    public final AccessorNamingStrategy.Provider getAccessorNaming() {
+        return _base.getAccessorNaming();
+    }
+
     public final HandlerInstantiator getHandlerInstantiator() {
         return _base.getHandlerInstantiator();
     }
-    
+
     /*
     /**********************************************************
     /* Configuration: type and subtype handling
@@ -274,6 +282,28 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     
     public abstract SubtypeResolver getSubtypeResolver();
 
+    /**
+     * Simple accessor for default {@link PolymorphicTypeValidator} to use for
+     * legacy Default Typing methods ({@code ObjectMapper.enableDefaultTyping()})
+     * and annotation based enabling.
+     *<p>
+     * Since 2.11 will also check {@link MapperFeature#BLOCK_UNSAFE_POLYMORPHIC_BASE_TYPES}
+     * to possibly override default to more restrictive implementation, see
+     * {@link com.fasterxml.jackson.databind.jsontype.DefaultBaseTypeLimitingValidator}).
+     *
+     * @since 2.10
+     */
+    public PolymorphicTypeValidator getPolymorphicTypeValidator() {
+        PolymorphicTypeValidator ptv = _base.getPolymorphicTypeValidator();
+        // [databind#2587]: allow stricter default settings:
+        if (ptv == LaissezFaireSubTypeValidator.instance) {
+            if (isEnabled(MapperFeature.BLOCK_UNSAFE_POLYMORPHIC_BASE_TYPES)) {
+                ptv = new DefaultBaseTypeLimitingValidator();
+            }
+        }
+        return ptv;
+    }
+
     public final TypeFactory getTypeFactory() {
         return _base.getTypeFactory();
     }
@@ -303,7 +333,8 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     }
 
     public JavaType constructSpecializedType(JavaType baseType, Class<?> subclass) {
-        return getTypeFactory().constructSpecializedType(baseType, subclass);
+        // note: since 2.11 specify "strict" resolution
+        return getTypeFactory().constructSpecializedType(baseType, subclass, true);
     }
 
     /*
@@ -319,12 +350,14 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     public BeanDescription introspectClassAnnotations(Class<?> cls) {
         return introspectClassAnnotations(constructType(cls));
     }
-    
+
     /**
      * Accessor for getting bean description that only contains class
      * annotations: useful if no getter/setter/creator information is needed.
      */
-    public abstract BeanDescription introspectClassAnnotations(JavaType type);
+    public BeanDescription introspectClassAnnotations(JavaType type) {
+        return getClassIntrospector().forClassAnnotations(this, type, this);
+    }
 
     /**
      * Accessor for getting bean description that only contains immediate class
@@ -334,12 +367,15 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     public BeanDescription introspectDirectClassAnnotations(Class<?> cls) {
         return introspectDirectClassAnnotations(constructType(cls));
     }
+
     /**
      * Accessor for getting bean description that only contains immediate class
      * annotations: ones from the class, and its direct mix-in, if any, but
      * not from super types.
      */
-    public abstract BeanDescription introspectDirectClassAnnotations(JavaType type);
+    public final BeanDescription introspectDirectClassAnnotations(JavaType type) {
+        return getClassIntrospector().forDirectClassAnnotations(this, type, this);
+    }
 
     /*
     /**********************************************************
@@ -347,6 +383,33 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
     /**********************************************************
      */
 
+    /**
+     * Accessor for finding {@link ConfigOverride} to use for
+     * properties of given type, if any exist; or return `null` if not.
+     *<p>
+     * Note that only directly associated override
+     * is found; no type hierarchy traversal is performed.
+     *
+     * @since 2.8
+     * 
+     * @return Override object to use for the type, if defined; null if none.
+     */
+    public abstract ConfigOverride findConfigOverride(Class<?> type);
+
+    /**
+     * Accessor for finding {@link ConfigOverride} to use for
+     * properties of given type, if any exist; or if none, return an immutable
+     * "empty" instance with no overrides.
+     *<p>
+     * Note that only directly associated override
+     * is found; no type hierarchy traversal is performed.
+     *
+     * @since 2.9
+     * 
+     * @return Override object to use for the type, never null (but may be empty)
+     */
+    public abstract ConfigOverride getConfigOverride(Class<?> type);
+
     /**
      * Accessor for default property inclusion to use for serialization,
      * used unless overridden by per-type or per-property overrides.
@@ -374,8 +437,52 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      * 
      * @since 2.8.2
      */
-    public abstract JsonInclude.Value getDefaultPropertyInclusion(Class<?> baseType,
-            JsonInclude.Value defaultIncl);
+    public JsonInclude.Value getDefaultPropertyInclusion(Class<?> baseType,
+            JsonInclude.Value defaultIncl)
+    {
+        JsonInclude.Value v = getConfigOverride(baseType).getInclude();
+        if (v != null) {
+            return v;
+        }
+        return defaultIncl;
+    }
+
+    /**
+     * Accessor for default property inclusion to use for serialization,
+     * considering possible per-type override for given base type and
+     * possible per-type override for given property type.<br>
+     * NOTE: if no override found, defaults to value returned by
+     * {@link #getDefaultPropertyInclusion()}.
+     *
+     * @param baseType Type of the instance containing the targeted property.
+     * @param propertyType Type of the property to look up inclusion setting for.
+     *
+     * @since 2.9
+     */
+    public abstract JsonInclude.Value getDefaultInclusion(Class<?> baseType,
+            Class<?> propertyType);
+
+    /**
+     * Accessor for default property inclusion to use for serialization,
+     * considering possible per-type override for given base type and
+     * possible per-type override for given property type; but
+     * if none found, returning given <code>defaultIncl</code>
+     *
+     * @param baseType Type of the instance containing the targeted property.
+     * @param propertyType Type of the property to look up inclusion setting for.
+     * @param defaultIncl Inclusion setting to return if no overrides found.
+     *
+     * @since 2.9
+     */
+    public JsonInclude.Value getDefaultInclusion(Class<?> baseType,
+            Class<?> propertyType, JsonInclude.Value defaultIncl)
+    {
+        JsonInclude.Value baseOverride = getConfigOverride(baseType).getInclude();
+        JsonInclude.Value propOverride = getConfigOverride(propertyType).getIncludeAsProperty();
+
+        JsonInclude.Value result = JsonInclude.Value.mergeAll(defaultIncl, baseOverride, propOverride);
+        return result;
+    }
 
     /**
      * Accessor for default format settings to use for serialization (and, to a degree
@@ -406,15 +513,71 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
             AnnotatedClass actualClass);
 
     /**
-     * Accessor for finding possible {@link ConfigOverride} to use for
-     * properties of given type. Note that only directly associate override
-     * is found; no type hierarchy traversal is performed.
+     * Helper method that may be called to see if there are property inclusion
+     * definitions from annotations (via {@link AnnotatedClass}).
      *
-     * @since 2.8
-     * 
-     * @return Override object if there is an override for specified type; `null` if not
+     * TODO: config override.
+     *
+     * @since 2.12
      */
-    public abstract ConfigOverride findConfigOverride(Class<?> type);
+    public abstract JsonIncludeProperties.Value getDefaultPropertyInclusions(Class<?> baseType,
+            AnnotatedClass actualClass);
+
+    /**
+     * Accessor for object used for determining whether specific property elements
+     * (method, constructors, fields) can be auto-detected based on
+     * their visibility (access modifiers). Can be changed to allow
+     * different minimum visibility levels for auto-detection. Note
+     * that this is the global handler; individual types (classes)
+     * can further override active checker used (using
+     * {@link JsonAutoDetect} annotation)
+     */
+    public abstract VisibilityChecker<?> getDefaultVisibilityChecker();
+
+    /**
+     * Accessor for object used for determining whether specific property elements
+     * (method, constructors, fields) can be auto-detected based on
+     * their visibility (access modifiers). This is based on global defaults
+     * (as would be returned by {@link #getDefaultVisibilityChecker()}, but
+     * then modified by possible class annotation (see {@link JsonAutoDetect})
+     * and/or per-type config override (see {@link ConfigOverride#getVisibility()}).
+     *
+     * @since 2.9
+     */
+    public abstract VisibilityChecker<?> getDefaultVisibilityChecker(Class<?> baseType,
+            AnnotatedClass actualClass);
+
+    /**
+     * Accessor for the baseline setter info used as the global baseline,
+     * not considering possible per-type overrides.
+     *
+     * @return Global base settings; never null
+     *
+     * @since 2.9
+     */
+    public abstract JsonSetter.Value getDefaultSetterInfo();
+
+    /**
+     * Accessor for the baseline merge info used as the global baseline,
+     * not considering possible per-type overrides.
+     *
+     * @return Global base settings, if any; `null` if none.
+     *
+     * @since 2.9
+     */
+    public abstract Boolean getDefaultMergeable();
+
+    /**
+     * Accessor for the baseline merge info used for given type, including global
+     * defaults if no type-specific overrides defined.
+     *
+     * @return Type-specific settings (if any); global defaults (same as
+     *    {@link #getDefaultMergeable()}) otherwise, if any defined; or `null`
+     *    if neither defined
+     *
+     * @since 2.9
+     */
+    public abstract Boolean getDefaultMergeable(Class<?> baseType);
 
     /*
     /**********************************************************
@@ -446,14 +609,29 @@ public abstract class MapperConfig<T extends MapperConfig<T>>
      * Initially set to {@link Locale#getDefault()}.
      */
     public final Locale getLocale() { return _base.getLocale(); }
-    
+
     /**
      * Method for accessing the default {@link java.util.TimeZone} to use
      * for formatting, unless overridden by local annotations.
      * Initially set to {@link TimeZone#getDefault()}.
      */
     public final TimeZone getTimeZone() { return _base.getTimeZone(); }
-    
+
+    /**
+     * Method for checking whether a {@link java.util.TimeZone} has been explicitly
+     * set for this configuring during construction of {@code ObjectMapper}
+     * or if it still has the default timezone/offset (zero-offset, "zulu").
+     *
+     * @return {@code true} if this configuration has explicitly specified
+     *    {@link java.util.TimeZone}, or {@code false} if it uses the default
+     *    time zone
+     *
+     * @since 2.12
+     */
+    public boolean hasExplicitTimeZone() {
+        return _base.hasExplicitTimeZone();
+    }
+
     /**
      * Accessor for finding currently active view, if any (null if none)
      */
