@@ -8,6 +8,7 @@ import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.cfg.CoercionAction;
 import com.fasterxml.jackson.databind.deser.impl.*;
 import com.fasterxml.jackson.databind.deser.impl.ReadableObjectId.Referring;
+import com.fasterxml.jackson.databind.util.ClassUtil;
 import com.fasterxml.jackson.databind.util.IgnorePropertiesUtil;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 import com.fasterxml.jackson.databind.util.TokenBuffer;
@@ -20,17 +21,6 @@ public class BeanDeserializer
     extends BeanDeserializerBase
     implements java.io.Serializable
 {
-    /* TODOs for future versions:
-     * 
-     * For 2.9?
-     *
-     * - New method in JsonDeserializer (deserializeNext()) to allow use of more
-     *   efficient 'nextXxx()' method `JsonParser` provides.
-     *
-     * Also: need to ensure efficient impl of those methods for Smile, CBOR
-     * at least (in addition to JSON)
-     */
-
     private static final long serialVersionUID = 1L;
 
     /**
@@ -506,9 +496,18 @@ public class BeanDeserializer
                 }
                 continue;
             }
+
+            // 29-Mar-2021, tatu: [databind#3082] May skip collection if we know
+            //    they'd just get ignored (note: any-setter handled above; unwrapped
+            //    properties also separately handled)
+            if (_ignoreAllUnknown) {
+                // 22-Aug-2021, tatu: [databind#3252] must ensure we do skip the whole value
+                p.skipChildren();
+                continue;
+            }
             // Ok then, let's collect the whole field; name and value
             if (unknown == null) {
-                unknown = new TokenBuffer(p, ctxt);
+                unknown = ctxt.bufferForInputBuffering(p);
             }
             unknown.writeFieldName(propName);
             unknown.copyCurrentStructure(p);
@@ -534,7 +533,7 @@ public class BeanDeserializer
         }
         if (unknown != null) {
             // polymorphic?
-            if (bean.getClass() != _beanType.getRawClass()) {
+            if (bean.getClass() != _beanType.getRawClass()) { // lgtm [java/dereferenced-value-may-be-null]
                 return handlePolymorphic(null, ctxt, bean, unknown);
             }
             // no, just some extra unknown properties
@@ -586,7 +585,7 @@ public class BeanDeserializer
         //   `VALUE_NULL` token.
         if (p.requiresCustomCodec()) { // not only XML module, but mostly it...
             @SuppressWarnings("resource")
-            TokenBuffer tb = new TokenBuffer(p, ctxt);
+            TokenBuffer tb = ctxt.bufferForInputBuffering(p);
             tb.writeEndObject();
             JsonParser p2 = tb.asParser(p);
             p2.nextToken(); // to point to END_OBJECT
@@ -630,6 +629,15 @@ public class BeanDeserializer
                 return ctxt.handleUnexpectedToken(getValueType(ctxt), JsonToken.START_ARRAY, p, null);
             }
             if (unwrap) {
+                // 23-Aug-2022, tatu: To prevent unbounded nested arrays, we better
+                //   check there is NOT another START_ARRAY lurking there..
+                if (p.nextToken() == JsonToken.START_ARRAY) {
+                    JavaType targetType = getValueType(ctxt);
+                    return ctxt.handleUnexpectedToken(targetType, JsonToken.START_ARRAY, p,
+"Cannot deserialize value of type %s from deeply-nested Array: only single wrapper allowed with `%s`",
+                            ClassUtil.getTypeDescription(targetType),
+                                    "DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS");
+                }
                 final Object value = deserialize(p, ctxt);
                 if (p.nextToken() != JsonToken.END_ARRAY) {
                     handleMissingEndArrayForSingle(p, ctxt);
@@ -694,7 +702,7 @@ public class BeanDeserializer
         if (_propertyBasedCreator != null) {
             return deserializeUsingPropertyBasedWithUnwrapped(p, ctxt);
         }
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
+        TokenBuffer tokens = ctxt.bufferForInputBuffering(p);
         tokens.writeStartObject();
         final Object bean = _valueInstantiator.createUsingDefault(ctxt);
 
@@ -738,7 +746,7 @@ public class BeanDeserializer
                 continue;
             }
             // Need to copy to a separate buffer first
-            TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
+            TokenBuffer b2 = ctxt.bufferAsCopyOfValue(p);
             tokens.writeFieldName(propName);
             tokens.append(b2);
             try {
@@ -761,7 +769,7 @@ public class BeanDeserializer
         if (t == JsonToken.START_OBJECT) {
             t = p.nextToken();
         }
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
+        TokenBuffer tokens = ctxt.bufferForInputBuffering(p);
         tokens.writeStartObject();
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
@@ -794,7 +802,7 @@ public class BeanDeserializer
                 tokens.copyCurrentStructure(p);
             } else {
                 // Need to copy to a separate buffer first
-                TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
+                TokenBuffer b2 = ctxt.bufferAsCopyOfValue(p);
                 tokens.writeFieldName(propName);
                 tokens.append(b2);
                 try {
@@ -821,7 +829,7 @@ public class BeanDeserializer
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(p, ctxt, _objectIdReader);
 
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
+        TokenBuffer tokens = ctxt.bufferForInputBuffering(p);
         tokens.writeStartObject();
 
         JsonToken t = p.currentToken();
@@ -893,7 +901,7 @@ public class BeanDeserializer
                 tokens.copyCurrentStructure(p);
             } else {
                 // Need to copy to a separate buffer first
-                TokenBuffer b2 = TokenBuffer.asCopyOfValue(p);
+                TokenBuffer b2 = ctxt.bufferAsCopyOfValue(p);
                 tokens.writeFieldName(propName);
                 tokens.append(b2);
                 try {
@@ -947,9 +955,15 @@ public class BeanDeserializer
             Object bean)
         throws IOException
     {
-        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
-        final ExternalTypeHandler ext = _externalTypeIdHandler.start();
+        return _deserializeWithExternalTypeId(p, ctxt, bean,
+                _externalTypeIdHandler.start());
+    }
 
+    protected Object _deserializeWithExternalTypeId(JsonParser p, DeserializationContext ctxt,
+            Object bean, ExternalTypeHandler ext)
+        throws IOException
+    {
+        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         for (JsonToken t = p.currentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
             String propName = p.currentName();
             t = p.nextToken();
@@ -996,20 +1010,19 @@ public class BeanDeserializer
     }
 
     @SuppressWarnings("resource")
-    protected Object deserializeUsingPropertyBasedWithExternalTypeId(JsonParser p, DeserializationContext ctxt)
+    protected Object deserializeUsingPropertyBasedWithExternalTypeId(JsonParser p,
+            DeserializationContext ctxt)
         throws IOException
     {
         final ExternalTypeHandler ext = _externalTypeIdHandler.start();
         final PropertyBasedCreator creator = _propertyBasedCreator;
         PropertyValueBuffer buffer = creator.startBuilding(p, ctxt, _objectIdReader);
-
-        TokenBuffer tokens = new TokenBuffer(p, ctxt);
-        tokens.writeStartObject();
+        final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
 
         JsonToken t = p.currentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
             String propName = p.currentName();
-            p.nextToken(); // to point to value
+            t = p.nextToken(); // to point to value
             // creator property?
             final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
             // Object Id property?
@@ -1024,7 +1037,8 @@ public class BeanDeserializer
                     ;
                 } else {
                     // Last creator property to set?
-                    if (buffer.assignParameter(creatorProp, _deserializeWithErrorWrapping(p, ctxt, creatorProp))) {
+                    if (buffer.assignParameter(creatorProp,
+                            _deserializeWithErrorWrapping(p, ctxt, creatorProp))) {
                         t = p.nextToken(); // to move to following FIELD_NAME/END_OBJECT
                         Object bean;
                         try {
@@ -1033,12 +1047,6 @@ public class BeanDeserializer
                             wrapAndThrow(e, _beanType.getRawClass(), propName, ctxt);
                             continue; // never gets here
                         }
-                        // if so, need to copy all remaining tokens into buffer
-                        while (t == JsonToken.FIELD_NAME) {
-                            p.nextToken(); // to skip name
-                            tokens.copyCurrentStructure(p);
-                            t = p.nextToken();
-                        }
                         if (bean.getClass() != _beanType.getRawClass()) {
                             // !!! 08-Jul-2011, tatu: Could theoretically support; but for now
                             //   it's too complicated, so bail out
@@ -1046,7 +1054,7 @@ public class BeanDeserializer
                                     "Cannot create polymorphic instances with external type ids (%s -> %s)",
                                     _beanType, bean.getClass()));
                         }
-                        return ext.complete(p, ctxt, bean);
+                        return _deserializeWithExternalTypeId(p, ctxt, bean, ext);
                     }
                 }
                 continue;
@@ -1054,7 +1062,16 @@ public class BeanDeserializer
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) {
-                buffer.bufferProperty(prop, prop.deserialize(p, ctxt));
+                // [databind#3045]: may have property AND be used as external type id:
+                if (t.isScalarValue()) {
+                    ext.handleTypePropertyValue(p, ctxt, propName, null);
+                }
+                // 19-Feb-2021, tatu: Should probably consider view too?
+                if (activeView != null && !prop.visibleInView(activeView)) {
+                    p.skipChildren();
+                } else {
+                    buffer.bufferProperty(prop, prop.deserialize(p, ctxt));
+                }
                 continue;
             }
             // external type id (or property that depends on it)?
@@ -1075,7 +1092,6 @@ public class BeanDeserializer
             // Unknown: let's call handler method
             handleUnknownProperty(p, ctxt, _valueClass, propName);
         }
-        tokens.writeEndObject();
 
         // We hit END_OBJECT; resolve the pieces:
         try {
