@@ -155,9 +155,7 @@ public class ExternalTypeHandler
                     _typeIds[it.next()] = typeId;
                 }
             } else {
-                @SuppressWarnings("resource")
-                TokenBuffer tokens = new TokenBuffer(p, ctxt);
-                tokens.copyCurrentStructure(p);
+                TokenBuffer tokens = ctxt.bufferAsCopyOfValue(p);
                 _tokens[index] = tokens;
                 while (it.hasNext()) {
                     _tokens[it.next()] = tokens;
@@ -172,13 +170,14 @@ public class ExternalTypeHandler
         ExtTypedProperty prop = _properties[index];
         boolean canDeserialize;
         if (prop.hasTypePropertyName(propName)) {
-            _typeIds[index] = p.getText();
+            // 19-Feb-2021, tatu: as per [databind#3008], don't use "getText()"
+            //    since that'll coerce null value into String "null"...
+            _typeIds[index] = p.getValueAsString();
             p.skipChildren();
             canDeserialize = (bean != null) && (_tokens[index] != null);
         } else {
             @SuppressWarnings("resource")
-            TokenBuffer tokens = new TokenBuffer(p, ctxt);
-            tokens.copyCurrentStructure(p);
+            TokenBuffer tokens = ctxt.bufferAsCopyOfValue(p);
             _tokens[index] = tokens;
             canDeserialize = (bean != null) && (_typeIds[index] != null);
         }
@@ -270,7 +269,12 @@ public class ExternalTypeHandler
             final ExtTypedProperty extProp = _properties[i];
             if (typeId == null) {
                 // let's allow missing both type and property (may already have been set, too)
-                if (_tokens[i] == null) {
+                TokenBuffer tb = _tokens[i];
+                if ((tb == null)
+                        // 19-Feb-2021, tatu: Both missing value and explicit `null`
+                        //    should be accepted...
+                        || (tb.firstToken() == JsonToken.VALUE_NULL)
+                    ) {
                     continue;
                 }
                 // but not just one
@@ -282,17 +286,20 @@ public class ExternalTypeHandler
                 } else {
                     typeId = extProp.getDefaultTypeId();
                 }
-            } else if (_tokens[i] == null) {
-                SettableBeanProperty prop = extProp.getProperty();
-                if (prop.isRequired() ||
-                        ctxt.isEnabled(DeserializationFeature.FAIL_ON_MISSING_EXTERNAL_TYPE_ID_PROPERTY)) {
+            }
+
+            if (_tokens[i] != null) {
+                values[i] = _deserialize(p, ctxt, i, typeId);
+            } else {
+                if (ctxt.isEnabled(DeserializationFeature.FAIL_ON_MISSING_EXTERNAL_TYPE_ID_PROPERTY)) {
+                    SettableBeanProperty prop = extProp.getProperty();
                     ctxt.reportPropertyInputMismatch(_beanType, prop.getName(),
                             "Missing property '%s' for external type id '%s'",
                             prop.getName(), _properties[i].getTypePropertyName());
                 }
-            }
-            if (_tokens[i] != null) {
-                values[i] = _deserialize(p, ctxt, i, typeId);
+                // 03-Aug-2022, tatu: [databind#3533] to handle absent value matching
+                //    present type id
+                values[i] = _deserializeMissingToken(p, ctxt, i, typeId);
             }
 
             final SettableBeanProperty prop = extProp.getProperty();
@@ -310,7 +317,7 @@ public class ExternalTypeHandler
                     if (typeProp.getType().hasRawClass(String.class)) {
                         v = typeId;
                     } else {
-                        TokenBuffer tb = new TokenBuffer(p, ctxt);
+                        TokenBuffer tb = ctxt.bufferForInputBuffering(p);
                         tb.writeString(typeId);
                         v = typeProp.getValueDeserializer().deserialize(tb.asParserOnFirstToken(), ctxt);
                         tb.close();
@@ -340,7 +347,7 @@ public class ExternalTypeHandler
         if (t == JsonToken.VALUE_NULL) {
             return null;
         }
-        TokenBuffer merged = new TokenBuffer(p, ctxt);
+        TokenBuffer merged = ctxt.bufferForInputBuffering(p);
         merged.writeStartArray();
         merged.writeString(typeId);
         merged.copyCurrentStructure(p2);
@@ -352,6 +359,22 @@ public class ExternalTypeHandler
         return _properties[index].getProperty().deserialize(mp, ctxt);
     }
 
+    // 03-Aug-2022, tatu: [databind#3533] to handle absent value matching:
+    @SuppressWarnings("resource")
+    protected final Object _deserializeMissingToken(JsonParser p, DeserializationContext ctxt,
+            int index, String typeId) throws IOException
+    {
+        TokenBuffer merged = ctxt.bufferForInputBuffering(p);
+        merged.writeStartArray();
+        merged.writeString(typeId);
+        merged.writeEndArray();
+
+        // needs to point to START_OBJECT (or whatever first token is)
+        JsonParser mp = merged.asParser(p);
+        mp.nextToken();
+        return _properties[index].getProperty().deserialize(mp, ctxt);
+    }
+
     @SuppressWarnings("resource")
     protected final void _deserializeAndSet(JsonParser p, DeserializationContext ctxt,
             Object bean, int index, String typeId) throws IOException
@@ -370,7 +393,7 @@ public class ExternalTypeHandler
             _properties[index].getProperty().set(bean, null);
             return;
         }
-        TokenBuffer merged = new TokenBuffer(p, ctxt);
+        TokenBuffer merged = ctxt.bufferForInputBuffering(p);
         merged.writeStartArray();
         merged.writeString(typeId);
 
