@@ -302,18 +302,30 @@ _containerType,
         // Start by verifying if we got empty/blank string since accessing
         // CoercionAction may be costlier than String value we'll almost certainly
         // need anyway
-        if (value.isEmpty()) { // ... in future may want to allow blank, too?
+        if (value.isEmpty()) {
             CoercionAction act = ctxt.findCoercionAction(logicalType(), rawTargetType,
                     CoercionInputShape.EmptyString);
-            act = _checkCoercionFail(ctxt, act, rawTargetType, value,
-                    "empty String (\"\")");
-            if (act != null) {
-                    // handleNonArray may successfully deserialize the result (if
-                    // ACCEPT_SINGLE_VALUE_AS_ARRAY is enabled, for example) otherwise it
-                    // is capable of failing just as well as _deserializeFromEmptyString.
+            // handleNonArray may successfully deserialize the result (if
+            // ACCEPT_SINGLE_VALUE_AS_ARRAY is enabled, for example) otherwise it
+            // is capable of failing just as well as _deserializeFromEmptyString.
+            if (act != null && act != CoercionAction.Fail) {
                 return (Collection<Object>) _deserializeFromEmptyString(
                         p, ctxt, act, rawTargetType, "empty String (\"\")");
             }
+            // note: `CoercionAction.Fail` falls through because we may need to allow
+            // `ACCEPT_SINGLE_VALUE_AS_ARRAY` handling later on
+        }
+        // 26-Mar-2021, tatu: Some day is today; as per [dataformat-xml#460],
+        //    we do need to support blank String too...
+        else if (_isBlank(value)) {
+            final CoercionAction act = ctxt.findCoercionFromBlankString(logicalType(), rawTargetType,
+                    CoercionAction.Fail);
+            if (act != CoercionAction.Fail) {
+                return (Collection<Object>) _deserializeFromEmptyString(
+                        p, ctxt, act, rawTargetType, "blank String (all whitespace)");
+            }
+            // note: `CoercionAction.Fail` falls through because we may need to allow
+            // `ACCEPT_SINGLE_VALUE_AS_ARRAY` handling later on
         }
         return handleNonArray(p, ctxt, createDefaultInstance(ctxt));
     }
