@@ -8,7 +8,7 @@ import com.fasterxml.jackson.annotation.Nulls;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.JsonParser.NumberType;
-import com.fasterxml.jackson.core.exc.InputCoercionException;
+import com.fasterxml.jackson.core.exc.StreamReadException;
 import com.fasterxml.jackson.core.io.NumberInput;
 
 import com.fasterxml.jackson.databind.*;
@@ -357,12 +357,8 @@ public abstract class StdDeserializer<T>
         // 23-Mar-2017, tatu: Let's specifically block recursive resolution to avoid
         //   either supporting nested arrays, or to cause infinite looping.
         if (p.hasToken(JsonToken.START_ARRAY)) {
-            String msg = String.format(
-"Cannot deserialize instance of %s out of %s token: nested Arrays not allowed with %s",
-                    ClassUtil.nameOf(_valueClass), JsonToken.START_ARRAY,
-                    "DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS");
             @SuppressWarnings("unchecked")
-            T result = (T) ctxt.handleUnexpectedToken(getValueType(ctxt), p.currentToken(), p, msg);
+            T result = (T) handleNestedArrayForSingle(p, ctxt);
             return result;
         }
         return (T) deserialize(p, ctxt);
@@ -413,7 +409,9 @@ public abstract class StdDeserializer<T>
         case JsonTokenId.ID_START_ARRAY:
             // 12-Jun-2020, tatu: For some reason calling `_deserializeFromArray()` won't work so:
             if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-                p.nextToken();
+                if (p.nextToken() == JsonToken.START_ARRAY) {
+                    return (boolean) handleNestedArrayForSingle(p, ctxt);
+                }
                 final boolean parsed = _parseBooleanPrimitive(p, ctxt);
                 _verifyEndArrayForSingle(p, ctxt);
                 return parsed;
@@ -582,7 +580,9 @@ public abstract class StdDeserializer<T>
         case JsonTokenId.ID_START_ARRAY: // unwrapping / from-empty-array coercion?
             // 12-Jun-2020, tatu: For some reason calling `_deserializeFromArray()` won't work so:
             if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-                p.nextToken();
+                if (p.nextToken() == JsonToken.START_ARRAY) {
+                    return (byte) handleNestedArrayForSingle(p, ctxt);
+                }
                 final byte parsed = _parseBytePrimitive(p, ctxt);
                 _verifyEndArrayForSingle(p, ctxt);
                 return parsed;
@@ -596,7 +596,9 @@ public abstract class StdDeserializer<T>
         CoercionAction act = _checkFromStringCoercion(ctxt, text,
                 LogicalType.Integer, Byte.TYPE);
         if (act == CoercionAction.AsNull) {
-            return (byte) 0; // no need to check as does not come from `null`, explicit coercion
+            // 03-May-2021, tatu: Might not be allowed (should we do "empty" check?)
+            _verifyNullForPrimitive(ctxt);
+            return (byte) 0;
         }
         if (act == CoercionAction.AsEmpty) {
             return (byte) 0;
@@ -650,7 +652,9 @@ public abstract class StdDeserializer<T>
         case JsonTokenId.ID_START_ARRAY:
             // 12-Jun-2020, tatu: For some reason calling `_deserializeFromArray()` won't work so:
             if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-                p.nextToken();
+                if (p.nextToken() == JsonToken.START_ARRAY) {
+                    return (short) handleNestedArrayForSingle(p, ctxt);
+                }
                 final short parsed = _parseShortPrimitive(p, ctxt);
                 _verifyEndArrayForSingle(p, ctxt);
                 return parsed;
@@ -663,7 +667,9 @@ public abstract class StdDeserializer<T>
         CoercionAction act = _checkFromStringCoercion(ctxt, text,
                 LogicalType.Integer, Short.TYPE);
         if (act == CoercionAction.AsNull) {
-            return (short) 0; // no need to check as does not come from `null`, explicit coercion
+            // 03-May-2021, tatu: Might not be allowed (should we do "empty" check?)
+            _verifyNullForPrimitive(ctxt);
+            return (short) 0;
         }
         if (act == CoercionAction.AsEmpty) {
             return (short) 0;
@@ -715,7 +721,9 @@ public abstract class StdDeserializer<T>
             break;
         case JsonTokenId.ID_START_ARRAY:
             if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-                p.nextToken();
+                if (p.nextToken() == JsonToken.START_ARRAY) {
+                    return (int) handleNestedArrayForSingle(p, ctxt);
+                }
                 final int parsed = _parseIntPrimitive(p, ctxt);
                 _verifyEndArrayForSingle(p, ctxt);
                 return parsed;
@@ -728,7 +736,9 @@ public abstract class StdDeserializer<T>
         final CoercionAction act = _checkFromStringCoercion(ctxt, text,
                 LogicalType.Integer, Integer.TYPE);
         if (act == CoercionAction.AsNull) {
-            return 0; // no need to check as does not come from `null`, explicit coercion
+            // 03-May-2021, tatu: Might not be allowed (should we do "empty" check?)
+            _verifyNullForPrimitive(ctxt);
+            return 0;
         }
         if (act == CoercionAction.AsEmpty) {
             return 0;
@@ -748,7 +758,7 @@ public abstract class StdDeserializer<T>
     {
         try {
             if (text.length() > 9) {
-                long l = Long.parseLong(text);
+                long l = NumberInput.parseLong(text);
                 if (_intOverflow(l)) {
                     Number v = (Number) ctxt.handleWeirdStringValue(Integer.TYPE, text,
                         "Overflow: numeric value (%s) out of range of int (%d -%d)",
@@ -811,7 +821,29 @@ public abstract class StdDeserializer<T>
         if (_checkTextualNull(ctxt, text)) {
             return (Integer) getNullValue(ctxt);
         }
-        return _parseIntPrimitive(ctxt, text);
+        return _parseInteger(ctxt, text);
+    }
+
+    /**
+     * @since 2.14
+     */
+    protected final Integer _parseInteger(DeserializationContext ctxt, String text) throws IOException
+    {
+        try {
+            if (text.length() > 9) {
+                long l = NumberInput.parseLong(text);
+                if (_intOverflow(l)) {
+                    return (Integer) ctxt.handleWeirdStringValue(Integer.class, text,
+                        "Overflow: numeric value (%s) out of range of `java.lang.Integer` (%d -%d)",
+                        text, Integer.MIN_VALUE, Integer.MAX_VALUE);
+                }
+                return Integer.valueOf((int) l);
+            }
+            return NumberInput.parseInt(text);
+        } catch (IllegalArgumentException iae) {
+            return(Integer) ctxt.handleWeirdStringValue(Integer.class, text,
+                    "not a valid `java.lang.Integer` value");
+        }
     }
 
     protected final long _parseLongPrimitive(JsonParser p, DeserializationContext ctxt)
@@ -842,7 +874,9 @@ public abstract class StdDeserializer<T>
             break;
         case JsonTokenId.ID_START_ARRAY:
             if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-                p.nextToken();
+                if (p.nextToken() == JsonToken.START_ARRAY) {
+                    return (long) handleNestedArrayForSingle(p, ctxt);
+                }
                 final long parsed = _parseLongPrimitive(p, ctxt);
                 _verifyEndArrayForSingle(p, ctxt);
                 return parsed;
@@ -855,7 +889,9 @@ public abstract class StdDeserializer<T>
         final CoercionAction act = _checkFromStringCoercion(ctxt, text,
                 LogicalType.Integer, Long.TYPE);
         if (act == CoercionAction.AsNull) {
-            return 0L; // no need to check as does not come from `null`, explicit coercion
+            // 03-May-2021, tatu: Might not be allowed (should we do "empty" check?)
+            _verifyNullForPrimitive(ctxt);
+            return 0L; 
         }
         if (act == CoercionAction.AsEmpty) {
             return 0L;
@@ -930,7 +966,19 @@ public abstract class StdDeserializer<T>
             return (Long) getNullValue(ctxt);
         }
         // let's allow Strings to be converted too
-        return _parseLongPrimitive(ctxt, text);
+        return _parseLong(ctxt, text);
+    }
+
+    /**
+     * @since 2.14
+     */
+    protected final Long _parseLong(DeserializationContext ctxt, String text) throws IOException
+    {
+        try {
+            return NumberInput.parseLong(text);
+        } catch (IllegalArgumentException iae) { }
+        return (Long) ctxt.handleWeirdStringValue(Long.class, text,
+                "not a valid `java.lang.Long` value");
     }
 
     protected final float _parseFloatPrimitive(JsonParser p, DeserializationContext ctxt)
@@ -942,6 +990,14 @@ public abstract class StdDeserializer<T>
             text = p.getText();
             break;
         case JsonTokenId.ID_NUMBER_INT:
+            final CoercionAction act = _checkIntToFloatCoercion(p, ctxt, Float.TYPE);
+            if (act == CoercionAction.AsNull) {
+                return 0.0f;
+            }
+            if (act == CoercionAction.AsEmpty) {
+                return 0.0f;
+            }
+            // fall through to coerce
         case JsonTokenId.ID_NUMBER_FLOAT:
             return p.getFloatValue();
         case JsonTokenId.ID_NULL:
@@ -953,7 +1009,9 @@ public abstract class StdDeserializer<T>
             break;
         case JsonTokenId.ID_START_ARRAY:
             if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-                p.nextToken();
+                if (p.nextToken() == JsonToken.START_ARRAY) {
+                    return (float) handleNestedArrayForSingle(p, ctxt);
+                }
                 final float parsed = _parseFloatPrimitive(p, ctxt);
                 _verifyEndArrayForSingle(p, ctxt);
                 return parsed;
@@ -976,7 +1034,9 @@ public abstract class StdDeserializer<T>
         final CoercionAction act = _checkFromStringCoercion(ctxt, text,
                 LogicalType.Integer, Float.TYPE);
         if (act == CoercionAction.AsNull) {
-            return  0.0f; // no need to check as does not come from `null`, explicit coercion
+            // 03-May-2021, tatu: Might not be allowed (should we do "empty" check?)
+            _verifyNullForPrimitive(ctxt);
+            return  0.0f;
         }
         if (act == CoercionAction.AsEmpty) {
             return  0.0f;
@@ -986,7 +1046,7 @@ public abstract class StdDeserializer<T>
             _verifyNullForPrimitiveCoercion(ctxt, text);
             return  0.0f;
         }
-        return _parseFloatPrimitive(ctxt, text);
+        return _parseFloatPrimitive(p, ctxt, text);
 }
 
     /**
@@ -996,7 +1056,21 @@ public abstract class StdDeserializer<T>
         throws IOException
     {
         try {
-            return Float.parseFloat(text);
+            return NumberInput.parseFloat(text);
+        } catch (IllegalArgumentException iae) { }
+        Number v = (Number) ctxt.handleWeirdStringValue(Float.TYPE, text,
+                "not a valid `float` value");
+        return _nonNullNumber(v).floatValue();
+    }
+
+    /**
+     * @since 2.14
+     */
+    protected final float _parseFloatPrimitive(JsonParser p, DeserializationContext ctxt, String text)
+            throws IOException
+    {
+        try {
+            return NumberInput.parseFloat(text, p.isEnabled(StreamReadFeature.USE_FAST_DOUBLE_PARSER));
         } catch (IllegalArgumentException iae) { }
         Number v = (Number) ctxt.handleWeirdStringValue(Float.TYPE, text,
                 "not a valid `float` value");
@@ -1047,6 +1121,14 @@ public abstract class StdDeserializer<T>
             text = p.getText();
             break;
         case JsonTokenId.ID_NUMBER_INT:
+            final CoercionAction act = _checkIntToFloatCoercion(p, ctxt, Double.TYPE);
+            if (act == CoercionAction.AsNull) {
+                return 0.0d;
+            }
+            if (act == CoercionAction.AsEmpty) {
+                return 0.0d;
+            }
+            // fall through to coerce
         case JsonTokenId.ID_NUMBER_FLOAT:
             return p.getDoubleValue();
         case JsonTokenId.ID_NULL:
@@ -1058,7 +1140,9 @@ public abstract class StdDeserializer<T>
             break;
         case JsonTokenId.ID_START_ARRAY:
             if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-                p.nextToken();
+                if (p.nextToken() == JsonToken.START_ARRAY) {
+                    return (double) handleNestedArrayForSingle(p, ctxt);
+                }
                 final double parsed = _parseDoublePrimitive(p, ctxt);
                 _verifyEndArrayForSingle(p, ctxt);
                 return parsed;
@@ -1081,7 +1165,9 @@ public abstract class StdDeserializer<T>
         final CoercionAction act = _checkFromStringCoercion(ctxt, text,
                 LogicalType.Integer, Double.TYPE);
         if (act == CoercionAction.AsNull) {
-            return  0.0; // no need to check as does not come from `null`, explicit coercion
+            // 03-May-2021, tatu: Might not be allowed (should we do "empty" check?)
+            _verifyNullForPrimitive(ctxt);
+            return  0.0;
         }
         if (act == CoercionAction.AsEmpty) {
             return  0.0;
@@ -1091,7 +1177,7 @@ public abstract class StdDeserializer<T>
             _verifyNullForPrimitiveCoercion(ctxt, text);
             return  0.0;
         }
-        return _parseDoublePrimitive(ctxt, text);
+        return _parseDoublePrimitive(p, ctxt, text);
     }
 
     /**
@@ -1108,17 +1194,41 @@ public abstract class StdDeserializer<T>
         return _nonNullNumber(v).doubleValue();
     }
 
+    /**
+     * @since 2.14
+     */
+    protected final double _parseDoublePrimitive(JsonParser p, DeserializationContext ctxt, String text)
+            throws IOException
+    {
+        try {
+            return _parseDouble(text, p.isEnabled(StreamReadFeature.USE_FAST_DOUBLE_PARSER));
+        } catch (IllegalArgumentException iae) { }
+        Number v = (Number) ctxt.handleWeirdStringValue(Double.TYPE, text,
+                "not a valid `double` value (as String to convert)");
+        return _nonNullNumber(v).doubleValue();
+    }
+
     /**
      * Helper method for encapsulating calls to low-level double value parsing; single place
      * just because we need a work-around that must be applied to all calls.
+     * Does not use the new <code>useFastParser</code> support.
+     *
+     * @see #_parseDouble(String, boolean)
      */
-    protected final static double _parseDouble(String numStr) throws NumberFormatException
+    protected final static double _parseDouble(final String numStr) throws NumberFormatException
     {
-        // avoid some nasty float representations... but should it be MIN_NORMAL or MIN_VALUE?
-        if (NumberInput.NASTY_SMALL_DOUBLE.equals(numStr)) {
-            return Double.MIN_NORMAL; // since 2.7; was MIN_VALUE prior
-        }
-        return Double.parseDouble(numStr);
+        return _parseDouble(numStr, false);
+    }
+
+    /**
+     * Helper method for encapsulating calls to low-level double value parsing; single place
+     * just because we need a work-around that must be applied to all calls.
+     *
+     * @since 2.14
+     */
+    protected final static double _parseDouble(final String numStr, final boolean useFastParser) throws NumberFormatException
+    {
+        return NumberInput.parseDouble(numStr, useFastParser);
     }
 
     /**
@@ -1171,9 +1281,8 @@ public abstract class StdDeserializer<T>
                 long ts;
                 try {
                     ts = p.getLongValue();
-                // 16-Jan-2019, tatu: 2.10 uses InputCoercionException, earlier JsonParseException
-                //     (but leave both until 3.0)
-                } catch (JsonParseException | InputCoercionException e) {
+                // 16-Jan-2019, tatu: 2.10 uses InputCoercionException, earlier StreamReadException
+                } catch (StreamReadException e) {
                     Number v = (Number) ctxt.handleWeirdNumberValue(_valueClass, p.getNumberValue(),
                             "not a valid 64-bit `long` for creating `java.util.Date`");
                     ts = v.longValue();
@@ -1214,6 +1323,9 @@ public abstract class StdDeserializer<T>
                 default:
                 }
             } else if (unwrap) {
+                if (t == JsonToken.START_ARRAY) {
+                    return (java.util.Date) handleNestedArrayForSingle(p, ctxt);
+                }
                 final Date parsed = _parseDate(p, ctxt);
                 _verifyEndArrayForSingle(p, ctxt);
                 return parsed;
@@ -1254,18 +1366,47 @@ public abstract class StdDeserializer<T>
     }
 
     /**
-     * Helper method used for accessing String value, if possible, doing
-     * necessary conversion or throwing exception as necessary.
-     *
      * @since 2.1
+     * @deprecated Since 2.14 (use the non-deprecated overload)
      */
-    protected final String _parseString(JsonParser p, DeserializationContext ctxt) throws IOException
+    @Deprecated
+    protected final String _parseString(JsonParser p, DeserializationContext ctxt)
+        throws IOException
     {
-        if (p.hasToken(JsonToken.VALUE_STRING)) {
+        return _parseString(p, ctxt,
+                // Note: could consider passing `this`, but since this methods is
+                // often called for values "inside" type we are handling (like
+                // String array or Collection of Strings) that would probably
+                // not match. So default to "nulls as nulls" handler.
+                NullsConstantProvider.nuller());
+    }
+
+    /**
+     * Helper method used for deserializing String value, if possible, doing
+     * necessary conversion or throwing exception as necessary.
+     *<p>
+     * Starting with 2.14 will also start checking {@code CoercionConfig}
+     * configuration to check whether coercion needed is allowed.
+     *
+     * @param p Currently active parser being iterated over
+     * @param ctxt Deserialization context
+     * @param nullProvider Entity we (only) need for case of secondary value
+     *    type being coerced into {@code null}: if so, provider is asked for
+     *    possible "null replacement" value.
+     *
+     * @since 2.14 (with this signature)
+     */
+    protected final String _parseString(JsonParser p, DeserializationContext ctxt,
+            NullValueProvider nullProvider)
+        throws IOException
+    {
+        CoercionAction act = CoercionAction.TryConvert;
+
+        switch (p.currentTokenId()) {
+        case JsonTokenId.ID_STRING:
             return p.getText();
-        }
         // 07-Nov-2019, tatu: [databind#2535] Need to support byte[]->Base64 same as `StringDeserializer`
-        if (p.hasToken(JsonToken.VALUE_EMBEDDED_OBJECT)) {
+        case JsonTokenId.ID_EMBEDDED_OBJECT:
             Object ob = p.getEmbeddedObject();
             if (ob instanceof byte[]) {
                 return ctxt.getBase64Variant().encode((byte[]) ob, false);
@@ -1275,17 +1416,38 @@ public abstract class StdDeserializer<T>
             }
             // otherwise, try conversion using toString()...
             return ob.toString();
-        }
         // 29-Jun-2020, tatu: New! "Scalar from Object" (mostly for XML)
-        if (p.hasToken(JsonToken.START_OBJECT)) {
+        case JsonTokenId.ID_START_OBJECT:
             return ctxt.extractScalarFromObject(p, this, _valueClass);
+        case JsonTokenId.ID_NUMBER_INT:
+            act = _checkIntToStringCoercion(p, ctxt, _valueClass);
+            break;
+        case JsonTokenId.ID_NUMBER_FLOAT:
+            act = _checkFloatToStringCoercion(p, ctxt, _valueClass);
+            break;
+        case JsonTokenId.ID_TRUE:
+        case JsonTokenId.ID_FALSE:
+            act = _checkBooleanToStringCoercion(p, ctxt, _valueClass);
+            break;
         }
 
-        String value = p.getValueAsString();
-        if (value != null) {
-            return value;
+        if (act == CoercionAction.AsNull) {
+            return (String) nullProvider.getNullValue(ctxt);
         }
-        return (String) ctxt.handleUnexpectedToken(String.class, p);
+        if (act == CoercionAction.AsEmpty) {
+            return "";
+        }
+
+        // allow coercions for other scalar types
+        // 17-Jan-2018, tatu: Related to [databind#1853] avoid FIELD_NAME by ensuring it's
+        //   "real" scalar
+        if (p.currentToken().isScalarValue()) {
+            String text = p.getValueAsString();
+            if (text != null) {
+                return text;
+            }
+        }
+        return (String) ctxt.handleUnexpectedToken(getValueType(ctxt), p);
     }
 
     /**
@@ -1391,6 +1553,68 @@ value, _coercedTypeDesc());
         return act;
     }
 
+    /**
+     * @since 2.14
+     */
+    protected CoercionAction _checkIntToStringCoercion(JsonParser p, DeserializationContext ctxt,
+            Class<?> rawTargetType)
+        throws IOException
+    {
+        return _checkToStringCoercion(p, ctxt, rawTargetType, p.getNumberValue(), CoercionInputShape.Integer);
+    }
+
+    /**
+     * @since 2.14
+     */
+    protected CoercionAction _checkFloatToStringCoercion(JsonParser p, DeserializationContext ctxt,
+            Class<?> rawTargetType)
+        throws IOException
+    {
+        return _checkToStringCoercion(p, ctxt, rawTargetType, p.getNumberValue(), CoercionInputShape.Float);
+    }
+
+    /**
+     * @since 2.14
+     */
+    protected CoercionAction _checkBooleanToStringCoercion(JsonParser p, DeserializationContext ctxt,
+            Class<?> rawTargetType)
+        throws IOException
+    {
+        return _checkToStringCoercion(p, ctxt, rawTargetType, p.getBooleanValue(), CoercionInputShape.Boolean);
+    }
+
+    /**
+     * @since 2.14
+     */
+    protected CoercionAction _checkToStringCoercion(JsonParser p, DeserializationContext ctxt,
+            Class<?> rawTargetType, Object inputValue, CoercionInputShape inputShape)
+        throws IOException
+    {
+        final CoercionAction act = ctxt.findCoercionAction(LogicalType.Textual,
+                rawTargetType, inputShape);
+        if (act == CoercionAction.Fail) {
+            return _checkCoercionFail(ctxt, act, rawTargetType, inputValue,
+                    inputShape.name() + " value (" + p.getText() + ")");
+        }
+        return act;
+    }
+
+    /**
+     * @since 2.14
+     */
+    protected CoercionAction _checkIntToFloatCoercion(JsonParser p, DeserializationContext ctxt,
+            Class<?> rawTargetType)
+        throws IOException
+    {
+        final CoercionAction act = ctxt.findCoercionAction(LogicalType.Float,
+                rawTargetType, CoercionInputShape.Integer);
+        if (act == CoercionAction.Fail) {
+            return _checkCoercionFail(ctxt, act, rawTargetType, p.getNumberValue(),
+                    "Integer value (" + p.getText() + ")");
+        }
+        return act;
+    }
+
     /**
      * @since 2.12
      */
@@ -1474,11 +1698,10 @@ inputDesc, _coercedTypeDesc());
      */
     protected Object _coerceIntegral(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        int feats = ctxt.getDeserializationFeatures();
-        if (DeserializationFeature.USE_BIG_INTEGER_FOR_INTS.enabledIn(feats)) {
+        if (ctxt.isEnabled(DeserializationFeature.USE_BIG_INTEGER_FOR_INTS)) {
             return p.getBigIntegerValue();
         }
-        if (DeserializationFeature.USE_LONG_FOR_INTS.enabledIn(feats)) {
+        if (ctxt.isEnabled(DeserializationFeature.USE_LONG_FOR_INTS)) {
             return p.getLongValue();
         }
         return p.getNumberValue(); // should be optimal, whatever it is
@@ -1875,7 +2098,10 @@ inputDesc, _coercedTypeDesc());
         if (prop != null) {
             return prop.getMetadata().getContentNulls();
         }
-        return null;
+        // 24-Aug-2021, tatu: As per [databind#3227] root values also need
+        //     to be checked for content nulls
+        //  NOTE: will this work wrt type-specific overrides? Probably not...
+        return ctxt.getConfig().getDefaultSetterInfo().getContentNulls();
     }
 
     // @since 2.9
@@ -1885,7 +2111,8 @@ inputDesc, _coercedTypeDesc());
     {
         if (nulls == Nulls.FAIL) {
             if (prop == null) {
-                return NullsFailProvider.constructForRootValue(ctxt.constructType(valueDeser.handledType()));
+                Class<?> rawType = (valueDeser == null) ? Object.class : valueDeser.handledType();
+                return NullsFailProvider.constructForRootValue(ctxt.constructType(rawType));
             }
             return NullsFailProvider.constructForProperty(prop);
         }
@@ -1900,10 +2127,11 @@ inputDesc, _coercedTypeDesc());
             // NOTE: although we could use `ValueInstantiator.Gettable` in general,
             // let's not since that would prevent being able to use custom impls:
             if (valueDeser instanceof BeanDeserializerBase) {
-                ValueInstantiator vi = ((BeanDeserializerBase) valueDeser).getValueInstantiator();
+                BeanDeserializerBase bd = (BeanDeserializerBase) valueDeser;
+                ValueInstantiator vi = bd.getValueInstantiator();
                 if (!vi.canCreateUsingDefault()) {
-                    final JavaType type = prop.getType();
-                    ctxt.reportBadDefinition(type,
+                    final JavaType type = (prop == null) ? bd.getValueType() : prop.getType();
+                    return ctxt.reportBadDefinition(type,
                             String.format("Cannot create empty instance of %s, no default Creator", type));
                 }
             }
@@ -1990,6 +2218,21 @@ handledType().getName());
         //     but for now just fall through
     }
 
+    /**
+     * Helper method called when detecting a deep(er) nesting of Arrays when trying
+     * to unwrap value for {@code DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS}.
+     *
+     * @since 2.14
+     */
+    protected Object handleNestedArrayForSingle(JsonParser p, DeserializationContext ctxt) throws IOException
+    {
+        String msg = String.format(
+"Cannot deserialize instance of %s out of %s token: nested Arrays not allowed with %s",
+                ClassUtil.nameOf(_valueClass), JsonToken.START_ARRAY,
+                "DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS");
+        return ctxt.handleUnexpectedToken(getValueType(ctxt), p.currentToken(), p, msg);
+    }
+
     protected void _verifyEndArrayForSingle(JsonParser p, DeserializationContext ctxt) throws IOException
     {
         JsonToken t = p.nextToken();
