@@ -219,10 +219,14 @@ final class AnnotatedCreatorCollector
         // 27-Oct-2020, tatu: SIGH. As per [databind#2894] there is widespread use of
         //   incorrect bindings in the wild -- not supported (no tests) but used
         //   nonetheless. So, for 2.11.x, put back "Bad Bindings"...
-//        final TypeResolutionContext typeResCtxt = _typeContext;
+//        
 
         // 03-Nov-2020, ckozak: Implement generic JsonCreator TypeVariable handling [databind#2895]
-        final TypeResolutionContext emptyTypeResCtxt = new TypeResolutionContext.Empty(typeFactory);
+//        final TypeResolutionContext emptyTypeResCtxt = new TypeResolutionContext.Empty(typeFactory);
+
+        // 23-Aug-2021, tatu: Double-d'oh! As per [databind#3220], we must revert to
+        //   the Incorrect Illogical But Used By Users Bindings... for 2.x at least.
+        final TypeResolutionContext initialTypeResCtxt = _typeContext;
 
         int factoryCount = candidates.size();
         List<AnnotatedMethod> result = new ArrayList<>(factoryCount);
@@ -247,7 +251,7 @@ final class AnnotatedCreatorCollector
                     if (key.equals(methodKeys[i])) {
                         result.set(i,
                                 constructFactoryCreator(candidates.get(i),
-                                        emptyTypeResCtxt, mixinFactory));
+                                        initialTypeResCtxt, mixinFactory));
                         break;
                     }
                 }
@@ -261,8 +265,9 @@ final class AnnotatedCreatorCollector
                 // 06-Nov-2020, tatu: Fix from [databind#2895] will try to resolve
                 //   nominal static method type bindings into expected target type
                 //   (if generic types involved)
+                // 23-Aug-2021, tatu: ... is this still needed, wrt un-fix in [databind#3220]?
                 TypeResolutionContext typeResCtxt = MethodGenericTypeResolver.narrowMethodTypeParameters(
-                        candidate, type, typeFactory, emptyTypeResCtxt);
+                        candidate, type, typeFactory, initialTypeResCtxt);
                 result.set(i,
                         constructFactoryCreator(candidate, typeResCtxt, null));
             }
@@ -347,7 +352,7 @@ ctor.getDeclaringClass().getName(), paramCount, paramAnns.length));
     protected AnnotatedMethod constructFactoryCreator(Method m,
             TypeResolutionContext typeResCtxt, Method mixin)
     {
-        final int paramCount = m.getParameterTypes().length;
+        final int paramCount = m.getParameterCount();
         if (_intr == null) { // when annotation processing is disabled
             return new AnnotatedMethod(typeResCtxt, m, _emptyAnnotationMap(),
                     _emptyAnnotationMaps(paramCount));
