@@ -184,8 +184,7 @@ public class AnnotatedMethodCollector
         }
         // also, for now we have no use for methods with more than 2 arguments:
         // (2 argument methods for "any setter", fwtw)
-        int pcount = m.getParameterTypes().length;
-        return (pcount <= 2);
+        return (m.getParameterCount() <= 2);
     }
 
     private final static class MethodBuilder {
