@@ -524,6 +524,20 @@ public class JacksonAnnotationIntrospector
         return (ann == null) ? null : ann.value();
     }
 
+    /**
+     * Specific implementation that will use following tie-breaker on
+     * given setter parameter types:
+     *<ol>
+     * <li>If either one is primitive type then either return {@code null}
+     *   (both primitives) or one that is primitive (when only primitive)
+     *  </li>
+     * <li>If only one is of type {@code String}, return that setter
+     *  </li>
+     * <li>Otherwise return {@code null}
+     *  </li>
+     * </ol>
+     * Returning {@code null} will indicate that resolution could not be done.
+     */
     @Override // since 2.7
     public AnnotatedMethod resolveSetterConflict(MapperConfig<?> config,
             AnnotatedMethod setter1, AnnotatedMethod setter2)
@@ -537,10 +551,12 @@ public class JacksonAnnotationIntrospector
             if (!cls2.isPrimitive()) {
                 return setter1;
             }
+            // 10-May-2021, tatu: if both primitives cannot decide
+            return null;
         } else if (cls2.isPrimitive()) {
             return setter2;
         }
-        
+
         if (cls1 == String.class) {
             if (cls2 != String.class) {
                 return setter1;
@@ -558,7 +574,7 @@ public class JacksonAnnotationIntrospector
         // Nothing to report, only used by modules. But define just as documentation
         return null;
     }
-    
+
     /*
     /**********************************************************
     /* Annotations for Polymorphic Type handling
@@ -605,14 +621,49 @@ public class JacksonAnnotationIntrospector
         JsonSubTypes t = _findAnnotation(a, JsonSubTypes.class);
         if (t == null) return null;
         JsonSubTypes.Type[] types = t.value();
+
+        // 02-Aug-2022, tatu: As per [databind#3500], may need to check uniqueness
+        //     of names
+        if (t.failOnRepeatedNames()) {
+            return findSubtypesCheckRepeatedNames(a.getName(), types);
+        } else {
+            ArrayList<NamedType> result = new ArrayList<NamedType>(types.length);
+            for (JsonSubTypes.Type type : types) {
+                result.add(new NamedType(type.value(), type.name()));
+                // [databind#2761]: alternative set of names to use
+                for (String name : type.names()) {
+                    result.add(new NamedType(type.value(), name));
+                }
+            }
+            return result;
+        }
+    }
+
+    // @since 2.14
+    private List<NamedType> findSubtypesCheckRepeatedNames(String annotatedTypeName, JsonSubTypes.Type[] types)
+    {
         ArrayList<NamedType> result = new ArrayList<NamedType>(types.length);
+        Set<String> seenNames = new HashSet<>();
         for (JsonSubTypes.Type type : types) {
-            result.add(new NamedType(type.value(), type.name()));
+            final String typeName = type.name();
+            if (!typeName.isEmpty() && seenNames.contains(typeName)) {
+                throw new IllegalArgumentException("Annotated type [" + annotatedTypeName + "] got repeated subtype name [" + typeName + "]");
+            } else {
+                seenNames.add(typeName);
+            }
+            result.add(new NamedType(type.value(), typeName));
+
             // [databind#2761]: alternative set of names to use
-            for (String name : type.names()) {
-                result.add(new NamedType(type.value(), name));
+            for (String altName : type.names()) {
+                if (!altName.isEmpty() && seenNames.contains(altName)) {
+                    throw new IllegalArgumentException("Annotated type [" + annotatedTypeName + "] got repeated subtype name [" + altName + "]");
+                } else {
+                    seenNames.add(altName);
+                }
+                result.add(new NamedType(type.value(), altName));
             }
         }
+
         return result;
     }
 
@@ -790,7 +841,8 @@ public class JacksonAnnotationIntrospector
     
     @Override
     public JavaType refineSerializationType(final MapperConfig<?> config,
-            final Annotated a, final JavaType baseType) throws JsonMappingException
+            final Annotated a, final JavaType baseType)
+        throws JsonMappingException
     {
         JavaType type = baseType;
         final TypeFactory tf = config.getTypeFactory();
@@ -818,15 +870,14 @@ public class JacksonAnnotationIntrospector
                         // 27-Apr-2017, tatu: [databind#1592] ignore primitive<->wrapper refinements
                         type = type.withStaticTyping();
                     } else {
-                        throw new JsonMappingException(null,
+                        throw _databindException(
                                 String.format("Cannot refine serialization type %s into %s; types not related",
                                         type, serClass.getName()));
                     }
                 } catch (IllegalArgumentException iae) {
-                    throw new JsonMappingException(null,
+                    throw _databindException(iae,
                             String.format("Failed to widen type %s with annotation (value %s), from '%s': %s",
-                                    type, serClass.getName(), a.getName(), iae.getMessage()),
-                                    iae);
+                                    type, serClass.getName(), a.getName(), iae.getMessage()));
                 }
             }
         }
@@ -853,15 +904,14 @@ public class JacksonAnnotationIntrospector
                             // 27-Apr-2017, tatu: [databind#1592] ignore primitive<->wrapper refinements
                             keyType = keyType.withStaticTyping();
                         } else {
-                            throw new JsonMappingException(null,
+                            throw _databindException(
                                     String.format("Cannot refine serialization key type %s into %s; types not related",
                                             keyType, keyClass.getName()));
                         }
                     } catch (IllegalArgumentException iae) {
-                        throw new JsonMappingException(null,
+                        throw _databindException(iae,
                                 String.format("Failed to widen key type of %s with concrete-type annotation (value %s), from '%s': %s",
-                                        type, keyClass.getName(), a.getName(), iae.getMessage()),
-                                        iae);
+                                        type, keyClass.getName(), a.getName(), iae.getMessage()));
                     }
                 }
                 type = ((MapLikeType) type).withKeyType(keyType);
@@ -889,15 +939,14 @@ public class JacksonAnnotationIntrospector
                            // 27-Apr-2017, tatu: [databind#1592] ignore primitive<->wrapper refinements
                            contentType = contentType.withStaticTyping();
                        } else {
-                           throw new JsonMappingException(null,
+                           throw _databindException(
                                    String.format("Cannot refine serialization content type %s into %s; types not related",
                                            contentType, contentClass.getName()));
                        }
                    } catch (IllegalArgumentException iae) { // shouldn't really happen
-                       throw new JsonMappingException(null,
+                       throw _databindException(iae,
                                String.format("Internal error: failed to refine value type of %s with concrete-type annotation (value %s), from '%s': %s",
-                                       type, contentClass.getName(), a.getName(), iae.getMessage()),
-                                       iae);
+                                       type, contentClass.getName(), a.getName(), iae.getMessage()));
                    }
                }
                type = type.withContentType(contentType);
@@ -1200,10 +1249,9 @@ public class JacksonAnnotationIntrospector
             try {
                 type = tf.constructSpecializedType(type, valueClass);
             } catch (IllegalArgumentException iae) {
-                throw new JsonMappingException(null,
+                throw _databindException(iae,
                         String.format("Failed to narrow type %s with annotation (value %s), from '%s': %s",
-                                type, valueClass.getName(), a.getName(), iae.getMessage()),
-                                iae);
+                                type, valueClass.getName(), a.getName(), iae.getMessage()));
             }
         }
         // Then further processing for container types
@@ -1218,10 +1266,9 @@ public class JacksonAnnotationIntrospector
                     keyType = tf.constructSpecializedType(keyType, keyClass);
                     type = ((MapLikeType) type).withKeyType(keyType);
                 } catch (IllegalArgumentException iae) {
-                    throw new JsonMappingException(null,
+                    throw _databindException(iae,
                             String.format("Failed to narrow key type of %s with concrete-type annotation (value %s), from '%s': %s",
-                                    type, keyClass.getName(), a.getName(), iae.getMessage()),
-                                    iae);
+                                    type, keyClass.getName(), a.getName(), iae.getMessage()));
                 }
             }
         }
@@ -1235,10 +1282,9 @@ public class JacksonAnnotationIntrospector
                     contentType = tf.constructSpecializedType(contentType, contentClass);
                     type = type.withContentType(contentType);
                 } catch (IllegalArgumentException iae) {
-                    throw new JsonMappingException(null,
+                    throw _databindException(iae,
                             String.format("Failed to narrow value type of %s with concrete-type annotation (value %s), from '%s': %s",
-                                    type, contentClass.getName(), a.getName(), iae.getMessage()),
-                            iae);
+                                    type, contentClass.getName(), a.getName(), iae.getMessage()));
                 }
             }
         }
@@ -1565,4 +1611,14 @@ public class JacksonAnnotationIntrospector
         }
         return false;
     }
+
+    // @since 2.12
+    private JsonMappingException _databindException(String msg) {
+        return new JsonMappingException(null, msg);
+    }
+
+    // @since 2.12
+    private JsonMappingException _databindException(Throwable t, String msg) {
+        return new JsonMappingException(null, msg, t);
+    }
 }
