@@ -5,13 +5,10 @@ import java.util.*;
 
 import com.fasterxml.jackson.annotation.JacksonInject;
 import com.fasterxml.jackson.annotation.JsonCreator;
-import com.fasterxml.jackson.annotation.JsonProperty;
 
 import com.fasterxml.jackson.databind.*;
-
 import com.fasterxml.jackson.databind.cfg.HandlerInstantiator;
 import com.fasterxml.jackson.databind.cfg.MapperConfig;
-import com.fasterxml.jackson.databind.util.BeanUtil;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 
 /**
@@ -31,17 +28,19 @@ public class POJOPropertiesCollector
      */
     protected final MapperConfig<?> _config;
 
+    /**
+     * Handler used for name-mangling of getter, mutator (setter/with) methods
+     *
+     * @since 2.12
+     */
+    protected final AccessorNamingStrategy _accessorNaming;
+
     /**
      * True if introspection is done for serialization (giving
      * precedence for serialization annotations), or not (false, deserialization)
      */
     protected final boolean _forSerialization;
 
-    /**
-     * @since 2.5
-     */
-    protected final boolean _stdBeanNaming;
-
     /**
      * Type of POJO for which properties are being collected.
      */
@@ -61,12 +60,6 @@ public class POJOPropertiesCollector
      */
     protected final boolean _useAnnotations;
 
-    /**
-     * Prefix used by auto-detected mutators ("setters"): usually "set",
-     * but differs for builder objects ("with" by default).
-     */
-    protected final String _mutatorPrefix;
-    
     /*
     /**********************************************************
     /* Collected property information
@@ -89,17 +82,45 @@ public class POJOPropertiesCollector
     protected LinkedHashMap<String, POJOPropertyBuilder> _properties;
 
     protected LinkedList<POJOPropertyBuilder> _creatorProperties;
+
+    /**
+     * A set of "field renamings" that have been discovered, indicating
+     * intended renaming of other accesors: key is the implicit original
+     * name and value intended name to use instead.
+     *<p>
+     * Note that these renamings are applied earlier than "regular" (explicit)
+     * renamings and affect implicit name: their effect may be changed by
+     * further renaming based on explicit indicators.
+     * The main use case is to effectively relink accessors based on fields
+     * discovered, and used to sort of correct otherwise missing linkage between
+     * fields and other accessors.
+     *
+     * @since 2.11
+     */
+    protected Map<PropertyName, PropertyName> _fieldRenameMappings;
     
     protected LinkedList<AnnotatedMember> _anyGetters;
 
+    /**
+     * @since 2.12
+     */
+    protected LinkedList<AnnotatedMember> _anyGetterField;
+
     protected LinkedList<AnnotatedMethod> _anySetters;
     
     protected LinkedList<AnnotatedMember> _anySetterField;
 
     /**
-     * Method(s) marked with 'JsonValue' annotation
-     *<p>
-     * NOTE: before 2.9, was `AnnotatedMethod`; with 2.9 allows fields too
+     * Accessors (field or "getter" method annotated with
+     * {@link com.fasterxml.jackson.annotation.JsonKey}
+     *
+     * @since 2.12
+     */
+    protected LinkedList<AnnotatedMember> _jsonKeyAccessors;
+
+    /**
+     * Accessors (field or "getter" method) annotated with
+     * {@link com.fasterxml.jackson.annotation.JsonValue}
      */
     protected LinkedList<AnnotatedMember> _jsonValueAccessors;
 
@@ -116,22 +137,38 @@ public class POJOPropertiesCollector
      * value injection.
      */
     protected LinkedHashMap<Object, AnnotatedMember> _injectables;
+
+    // // // Deprecated entries to remove from 3.0
     
+    /**
+     * @deprecated Since 2.12
+     */
+    @Deprecated
+    protected final boolean _stdBeanNaming;
+
+    /**
+     * @deprecated Since 2.12
+     */
+    @Deprecated
+    protected String _mutatorPrefix = "set";
+
     /*
     /**********************************************************
     /* Life-cycle
     /**********************************************************
      */
 
+    /**
+     * @since 2.12
+     */
     protected POJOPropertiesCollector(MapperConfig<?> config, boolean forSerialization,
-            JavaType type, AnnotatedClass classDef, String mutatorPrefix)
+            JavaType type, AnnotatedClass classDef,
+            AccessorNamingStrategy accessorNaming)
     {
         _config = config;
-        _stdBeanNaming = config.isEnabled(MapperFeature.USE_STD_BEAN_NAMING);
         _forSerialization = forSerialization;
         _type = type;
         _classDef = classDef;
-        _mutatorPrefix = (mutatorPrefix == null) ? "set" : mutatorPrefix;
         if (config.isAnnotationProcessingEnabled()) {
             _useAnnotations = true;
             _annotationIntrospector = _config.getAnnotationIntrospector();
@@ -141,6 +178,32 @@ public class POJOPropertiesCollector
         }
         _visibilityChecker = _config.getDefaultVisibilityChecker(type.getRawClass(),
                 classDef);
+        _accessorNaming = accessorNaming;
+
+        // for backwards-compatibility only
+        _stdBeanNaming = config.isEnabled(MapperFeature.USE_STD_BEAN_NAMING);
+    }
+
+    /**
+     * @deprecated Since 2.12
+     */
+    @Deprecated
+    protected POJOPropertiesCollector(MapperConfig<?> config, boolean forSerialization,
+            JavaType type, AnnotatedClass classDef,
+            String mutatorPrefix)
+    {
+        this(config, forSerialization, type, classDef,
+                _accessorNaming(config, classDef, mutatorPrefix));
+        _mutatorPrefix = mutatorPrefix;
+    }
+
+    private static AccessorNamingStrategy _accessorNaming(MapperConfig<?> config, AnnotatedClass classDef,
+            String mutatorPrefix) {
+        if (mutatorPrefix == null) {
+            mutatorPrefix = "set";
+        }
+        return new DefaultAccessorNamingStrategy.Provider()
+                .withSetterPrefix(mutatorPrefix).forPOJO(config, classDef);
     }
 
     /*
@@ -178,11 +241,22 @@ public class POJOPropertiesCollector
         return _injectables;
     }
 
-    @Deprecated // since 2.9
-    public AnnotatedMethod getJsonValueMethod() {
-        AnnotatedMember m = getJsonValueAccessor();
-        if (m instanceof AnnotatedMethod) {
-            return (AnnotatedMethod) m;
+    /**
+     * @since 2.12
+     */
+    public AnnotatedMember getJsonKeyAccessor() {
+        if (!_collected) {
+            collectAll();
+        }
+        // If @JsonKey defined, must have a single one
+        if (_jsonKeyAccessors != null) {
+            if (_jsonKeyAccessors.size() > 1) {
+                reportProblem("Multiple 'as-key' properties defined (%s vs %s)",
+                        _jsonKeyAccessors.get(0),
+                        _jsonKeyAccessors.get(1));
+            }
+            // otherwise we won't greatly care
+            return _jsonKeyAccessors.get(0);
         }
         return null;
     }
@@ -208,18 +282,50 @@ public class POJOPropertiesCollector
         return null;
     }
 
-    public AnnotatedMember getAnyGetter()
+    /**
+     * Alias for {@link #getAnyGetterMethod()}.
+     *
+     * @deprecated Since 2.12 use separate {@link #getAnyGetterMethod()} and
+     *     {@link #getAnyGetterField()}.
+     */
+    @Deprecated // since 2.12
+    public AnnotatedMember getAnyGetter() {
+        return getAnyGetterMethod();
+    }
+
+    /**
+     * @since 2.12 (before only had "getAnyGetter()")
+     */
+    public AnnotatedMember getAnyGetterField()
+    {
+        if (!_collected) {
+            collectAll();
+        }
+        if (_anyGetterField != null) {
+            if (_anyGetterField.size() > 1) {
+                reportProblem("Multiple 'any-getter' fields defined (%s vs %s)",
+                        _anyGetterField.get(0), _anyGetterField.get(1));
+            }
+            return _anyGetterField.getFirst();
+        }
+        return null;
+    }
+
+    /**
+     * @since 2.12 (before only had "getAnyGetter()")
+     */
+    public AnnotatedMember getAnyGetterMethod()
     {
         if (!_collected) {
             collectAll();
         }
         if (_anyGetters != null) {
             if (_anyGetters.size() > 1) {
-                reportProblem("Multiple 'any-getters' defined (%s vs %s)",
+                reportProblem("Multiple 'any-getter' methods defined (%s vs %s)",
                         _anyGetters.get(0), _anyGetters.get(1));
             }
             return _anyGetters.getFirst();
-        }        
+        }
         return null;
     }
 
@@ -274,13 +380,6 @@ public class POJOPropertiesCollector
         return info;
     }
 
-    /**
-     * Method for finding Class to use as POJO builder, if any.
-     */
-    public Class<?> findPOJOBuilderClass() {
-        return _annotationIntrospector.findPOJOBuilder(_classDef);
-    }
-    
     // for unit tests:
     protected Map<String, POJOPropertyBuilder> getPropertyMap() {
         if (!_collected) {
@@ -289,6 +388,20 @@ public class POJOPropertiesCollector
         return _properties;
     }
 
+    @Deprecated // since 2.9
+    public AnnotatedMethod getJsonValueMethod() {
+        AnnotatedMember m = getJsonValueAccessor();
+        if (m instanceof AnnotatedMethod) {
+            return (AnnotatedMethod) m;
+        }
+        return null;
+    }
+
+    @Deprecated // since 2.11 (not used by anything at this point)
+    public Class<?> findPOJOBuilderClass() {
+        return _annotationIntrospector.findPOJOBuilder(_classDef);
+    }
+
     /*
     /**********************************************************
     /* Public API: main-level collection
@@ -305,14 +418,13 @@ public class POJOPropertiesCollector
         LinkedHashMap<String, POJOPropertyBuilder> props = new LinkedHashMap<String, POJOPropertyBuilder>();
 
         // First: gather basic data
-        _addFields(props);
+        _addFields(props); // note: populates _fieldRenameMappings
         _addMethods(props);
         // 25-Jan-2016, tatu: Avoid introspecting (constructor-)creators for non-static
         //    inner classes, see [databind#1502]
         if (!_classDef.isNonStaticInnerClass()) {
             _addCreators(props);
         }
-        _addInjectables(props);
 
         // Remove ignored properties, first; this MUST precede annotation merging
         // since logic relies on knowing exactly which accessor has which annotation
@@ -323,6 +435,10 @@ public class POJOPropertiesCollector
         // Rename remaining properties
         _renameProperties(props);
 
+        // and now add injectables, but taking care to avoid overlapping ones
+        // via creator and regular properties
+        _addInjectables(props);
+
         // then merge annotations, to simplify further processing
         // 26-Sep-2017, tatu: Before 2.9.2 was done earlier but that prevented some of
         //   annotations from getting properly merged
@@ -331,22 +447,21 @@ public class POJOPropertiesCollector
         }
 
         // And use custom naming strategy, if applicable...
+        // 18-Jan-2021, tatu: To be done before trimming, to resolve
+        //   [databind#3368]
         PropertyNamingStrategy naming = _findNamingStrategy();
         if (naming != null) {
             _renameUsing(props, naming);
         }
 
-        /* Sort by visibility (explicit over implicit); drop all but first
-         * of member type (getter, setter etc) if there is visibility
-         * difference
-         */
+        // Sort by visibility (explicit over implicit); drop all but first of member
+        // type (getter, setter etc) if there is visibility difference
         for (POJOPropertyBuilder property : props.values()) {
             property.trimByVisibility();
         }
 
-        /* and, if required, apply wrapper name: note, MUST be done after
-         * annotations are merged.
-         */
+        // and, if required, apply wrapper name: note, MUST be done after
+        // annotations are merged.
         if (_config.isEnabled(MapperFeature.USE_WRAPPER_NAME_AS_PROPERTY_NAME)) {
             _renameWithWrappers(props);
         }
@@ -375,9 +490,15 @@ public class POJOPropertiesCollector
          */
         final boolean pruneFinalFields = !_forSerialization && !_config.isEnabled(MapperFeature.ALLOW_FINAL_FIELDS_AS_MUTATORS);
         final boolean transientAsIgnoral = _config.isEnabled(MapperFeature.PROPAGATE_TRANSIENT_MARKER);
-        
+
         for (AnnotatedField f : _classDef.fields()) {
-            String implName = ai.findImplicitPropertyName(f);
+            // @JsonKey?
+            if (Boolean.TRUE.equals(ai.hasAsKey(_config, f))) {
+                if (_jsonKeyAccessors == null) {
+                    _jsonKeyAccessors = new LinkedList<>();
+                }
+                _jsonKeyAccessors.add(f);
+            }
             // @JsonValue?
             if (Boolean.TRUE.equals(ai.hasAsValue(f))) {
                 if (_jsonValueAccessors == null) {
@@ -386,25 +507,57 @@ public class POJOPropertiesCollector
                 _jsonValueAccessors.add(f);
                 continue;
             }
-            // @JsonAnySetter?
-            if (Boolean.TRUE.equals(ai.hasAnySetter(f))) {
-                if (_anySetterField == null) {
-                    _anySetterField = new LinkedList<AnnotatedMember>();
+            // 12-October-2020, dominikrebhan: [databind#1458] Support @JsonAnyGetter on
+            //   fields and allow @JsonAnySetter to be declared as well.
+            boolean anyGetter = Boolean.TRUE.equals(ai.hasAnyGetter(f));
+            boolean anySetter = Boolean.TRUE.equals(ai.hasAnySetter(f));
+            if (anyGetter || anySetter) {
+                // @JsonAnyGetter?
+                if (anyGetter) {
+                    if (_anyGetterField == null) {
+                        _anyGetterField = new LinkedList<>();
+                    }
+                    _anyGetterField.add(f);
+                }
+                // @JsonAnySetter?
+                if (anySetter) {
+                    if (_anySetterField == null) {
+                        _anySetterField = new LinkedList<>();
+                    }
+                    _anySetterField.add(f);
                 }
-                _anySetterField.add(f);
                 continue;
             }
+            String implName = ai.findImplicitPropertyName(f);
             if (implName == null) {
                 implName = f.getName();
             }
+            // 27-Aug-2020, tatu: [databind#2800] apply naming strategy for
+            //   fields too, to allow use of naming conventions.
+            implName = _accessorNaming.modifyFieldName(f, implName);
+            if (implName == null) {
+                continue;
+            }
+
+            final PropertyName implNameP = _propNameFromSimple(implName);
+            // [databind#2527: Field-based renaming can be applied early (here),
+            // or at a later point, but probably must be done before pruning
+            // final fields. So let's do it early here
+            final PropertyName rename = ai.findRenameByField(_config, f, implNameP);
+            if ((rename != null) && !rename.equals(implNameP)) {
+                if (_fieldRenameMappings == null) {
+                    _fieldRenameMappings = new HashMap<>();
+                }
+                _fieldRenameMappings.put(rename, implNameP);
+            }
+
             PropertyName pn;
 
             if (_forSerialization) {
-                /* 18-Aug-2011, tatu: As per existing unit tests, we should only
-                 *   use serialization annotation (@JsonSerialize) when serializing
-                 *   fields, and similarly for deserialize-only annotations... so
-                 *   no fallbacks in this particular case.
-                 */
+                // 18-Aug-2011, tatu: As per existing unit tests, we should only
+                //   use serialization annotation (@JsonSerialize) when serializing
+                //   fields, and similarly for deserialize-only annotations... so
+                //   no fallbacks in this particular case.
                 pn = ai.findNameForSerialization(f);
             } else {
                 pn = ai.findNameForDeserialization(f);
@@ -437,7 +590,7 @@ public class POJOPropertiesCollector
             }
             /* [databind#190]: this is the place to prune final fields, if they are not
              *  to be used as mutators. Must verify they are not explicitly included.
-             *  Also: if 'ignored' is set, need to included until a later point, to
+             *  Also: if 'ignored' is set, need to include until a later point, to
              *  avoid losing ignoral information.
              */
             if (pruneFinalFields && (pn == null) && !ignored
@@ -503,9 +656,12 @@ public class POJOPropertiesCollector
             pn = PropertyName.construct(impl);
         }
 
+        // 27-Dec-2019, tatu: [databind#2527] may need to rename according to field
+        impl = _checkRenameByField(impl);
+
         // shouldn't need to worry about @JsonIgnore, since creators only added
         // if so annotated
-
+        
         /* 13-May-2015, tatu: We should try to start with implicit name, similar to how
          *   fields and methods work; but unlike those, we don't necessarily have
          *   implicit name to use (pre-Java8 at least). So:
@@ -521,26 +677,23 @@ public class POJOPropertiesCollector
      */
     protected void _addMethods(Map<String, POJOPropertyBuilder> props)
     {
-        final AnnotationIntrospector ai = _annotationIntrospector;
         for (AnnotatedMethod m : _classDef.memberMethods()) {
-            /* For methods, handling differs between getters and setters; and
-             * we will also only consider entries that either follow the bean
-             * naming convention or are explicitly marked: just being visible
-             * is not enough (unlike with fields)
-             */
+            // For methods, handling differs between getters and setters; and
+            // we will also only consider entries that either follow the bean
+            // naming convention or are explicitly marked: just being visible
+            // is not enough (unlike with fields)
+
             int argCount = m.getParameterCount();
             if (argCount == 0) { // getters (including 'any getter')
-            	_addGetterMethod(props, m, ai);
+                _addGetterMethod(props, m, _annotationIntrospector);
             } else if (argCount == 1) { // setters
-            	_addSetterMethod(props, m, ai);
+                _addSetterMethod(props, m, _annotationIntrospector);
             } else if (argCount == 2) { // any getter?
-                if (ai != null) {
-                    if (Boolean.TRUE.equals(ai.hasAnySetter(m))) {
-                        if (_anySetters == null) {
-                            _anySetters = new LinkedList<AnnotatedMethod>();
-                        }
-                        _anySetters.add(m);
+                if (Boolean.TRUE.equals(_annotationIntrospector.hasAnySetter(m))) {
+                    if (_anySetters == null) {
+                        _anySetters = new LinkedList<AnnotatedMethod>();
                     }
+                    _anySetters.add(m);
                 }
             }
         }
@@ -550,10 +703,15 @@ public class POJOPropertiesCollector
             AnnotatedMethod m, AnnotationIntrospector ai)
     {
         // Very first thing: skip if not returning any value
-        if (!m.hasReturnType()) {
-            return;
+        // 06-May-2020, tatu: [databind#2675] changes handling slightly...
+        {
+            final Class<?> rt = m.getRawReturnType();
+            if ((rt == Void.TYPE) ||
+                    ((rt == Void.class) && !_config.isEnabled(MapperFeature.ALLOW_VOID_VALUED_PROPERTIES))) {
+                return;
+            }
         }
-        
+
         // any getter?
         // @JsonAnyGetter?
         if (Boolean.TRUE.equals(ai.hasAnyGetter(m))) {
@@ -563,6 +721,14 @@ public class POJOPropertiesCollector
             _anyGetters.add(m);
             return;
         }
+        // @JsonKey?
+        if (Boolean.TRUE.equals(ai.hasAsKey(_config, m))) {
+            if (_jsonKeyAccessors == null) {
+                _jsonKeyAccessors = new LinkedList<>();
+            }
+            _jsonKeyAccessors.add(m);
+            return;
+        }
         // @JsonValue?
         if (Boolean.TRUE.equals(ai.hasAsValue(m))) {
             if (_jsonValueAccessors == null) {
@@ -580,10 +746,10 @@ public class POJOPropertiesCollector
         if (!nameExplicit) { // no explicit name; must consider implicit
             implName = ai.findImplicitPropertyName(m);
             if (implName == null) {
-                implName = BeanUtil.okNameForRegularGetter(m, m.getName(), _stdBeanNaming);
+                implName = _accessorNaming.findNameForRegularGetter(m, m.getName());
             }
             if (implName == null) { // if not, must skip
-                implName = BeanUtil.okNameForIsGetter(m, m.getName(), _stdBeanNaming);
+                implName = _accessorNaming.findNameForIsGetter(m, m.getName());
                 if (implName == null) {
                     return;
                 }
@@ -595,7 +761,10 @@ public class POJOPropertiesCollector
             // we still need implicit name to link with other pieces
             implName = ai.findImplicitPropertyName(m);
             if (implName == null) {
-                implName = BeanUtil.okNameForGetter(m, _stdBeanNaming);
+                implName = _accessorNaming.findNameForRegularGetter(m, m.getName());
+                if (implName == null) {
+                    implName = _accessorNaming.findNameForIsGetter(m, m.getName());
+                }
             }
             // if not regular getter name, use method name as is
             if (implName == null) {
@@ -608,6 +777,8 @@ public class POJOPropertiesCollector
             }
             visible = true;
         }
+        // 27-Dec-2019, tatu: [databind#2527] may need to rename according to field
+        implName = _checkRenameByField(implName);
         boolean ignore = ai.hasIgnoreMarker(m);
         _property(props, implName).addGetter(m, pn, nameExplicit, visible, ignore);
     }
@@ -617,12 +788,12 @@ public class POJOPropertiesCollector
     {
         String implName; // from naming convention
         boolean visible;
-        PropertyName pn = (ai == null) ? null : ai.findNameForDeserialization(m);
+        PropertyName pn = ai.findNameForDeserialization(m);
         boolean nameExplicit = (pn != null);
         if (!nameExplicit) { // no explicit name; must follow naming convention
-            implName = (ai == null) ? null : ai.findImplicitPropertyName(m);
+            implName = ai.findImplicitPropertyName(m);
             if (implName == null) {
-                implName = BeanUtil.okNameForMutator(m, _mutatorPrefix, _stdBeanNaming);
+                implName = _accessorNaming.findNameForMutator(m, m.getName());
             }
             if (implName == null) { // if not, must skip
             	return;
@@ -630,9 +801,9 @@ public class POJOPropertiesCollector
             visible = _visibilityChecker.isSetterVisible(m);
         } else { // explicit indication of inclusion, but may be empty
             // we still need implicit name to link with other pieces
-            implName = (ai == null) ? null : ai.findImplicitPropertyName(m);
+            implName = ai.findImplicitPropertyName(m);
             if (implName == null) {
-                implName = BeanUtil.okNameForMutator(m, _mutatorPrefix, _stdBeanNaming);
+                implName = _accessorNaming.findNameForMutator(m, m.getName());
             }
             // if not regular getter name, use method name as is
             if (implName == null) {
@@ -645,16 +816,18 @@ public class POJOPropertiesCollector
             }
             visible = true;
         }
-        boolean ignore = (ai == null) ? false : ai.hasIgnoreMarker(m);
-        _property(props, implName).addSetter(m, pn, nameExplicit, visible, ignore);
+        // 27-Dec-2019, tatu: [databind#2527] may need to rename according to field
+        implName = _checkRenameByField(implName);
+        final boolean ignore = ai.hasIgnoreMarker(m);
+        _property(props, implName)
+            .addSetter(m, pn, nameExplicit, visible, ignore);
     }
 
     protected void _addInjectables(Map<String, POJOPropertyBuilder> props)
     {
-        final AnnotationIntrospector ai = _annotationIntrospector;
         // first fields, then methods, to allow overriding
         for (AnnotatedField f : _classDef.fields()) {
-            _doAddInjectable(ai.findInjectableValue(f), f);
+            _doAddInjectable(_annotationIntrospector.findInjectableValue(f), f);
         }
         
         for (AnnotatedMethod m : _classDef.memberMethods()) {
@@ -662,7 +835,7 @@ public class POJOPropertiesCollector
             if (m.getParameterCount() != 1) {
                 continue;
             }
-            _doAddInjectable(ai.findInjectableValue(m), m);
+            _doAddInjectable(_annotationIntrospector.findInjectableValue(m), m);
         }
     }
 
@@ -681,7 +854,7 @@ public class POJOPropertiesCollector
             if (prev.getClass() == m.getClass()) {
                 String type = id.getClass().getName();
                 throw new IllegalArgumentException("Duplicate injectable value with id '"
-                        +String.valueOf(id)+"' (of type "+type+")");
+                        + id +"' (of type "+type+")");
             }
         }
     }
@@ -689,7 +862,20 @@ public class POJOPropertiesCollector
     private PropertyName _propNameFromSimple(String simpleName) {
         return PropertyName.construct(simpleName, null);
     }
-    
+
+    // @since 2.11
+    private String _checkRenameByField(String implName) {
+        if (_fieldRenameMappings != null) {
+            PropertyName p = _fieldRenameMappings.get(_propNameFromSimple(implName));
+            if (p != null) {
+                implName = p.getSimpleName();
+                return implName;
+
+            }
+        }
+        return implName;
+    }
+
     /*
     /**********************************************************
     /* Internal methods; removing ignored properties
@@ -714,7 +900,9 @@ public class POJOPropertiesCollector
             // Otherwise, check ignorals
             if (prop.anyIgnorals()) {
                 // first: if one or more ignorals, and no explicit markers, remove the whole thing
-                if (!prop.isExplicitlyIncluded()) {
+                // 16-May-2022, tatu: NOTE! As per [databind#3357] need to consider
+                //    only explicit inclusion by accessors OTHER than ones with ignoral marker
+                if (!prop.anyExplicitsWithoutIgnoral()) {
                     it.remove();
                     _collectIgnorals(prop.getName());
                     continue;
@@ -741,10 +929,9 @@ public class POJOPropertiesCollector
         while (it.hasNext()) {
             POJOPropertyBuilder prop = it.next();
             // 26-Jan-2017, tatu: [databind#935]: need to denote removal of
-            JsonProperty.Access acc = prop.removeNonVisible(inferMutators);
-            if (acc == JsonProperty.Access.READ_ONLY) {
-                _collectIgnorals(prop.getName());
-            }
+            // 16-May-2020, tatu: [databind#2719]: need to pass `this` to allow
+            //    addition of ignorals wrt explicit name
+            prop.removeNonVisible(inferMutators, _forSerialization ? null : this);
         }
     }
 
@@ -753,9 +940,9 @@ public class POJOPropertiesCollector
      * of known ignored properties; this helps in proper reporting of
      * errors.
      */
-    private void _collectIgnorals(String name)
+    protected void _collectIgnorals(String name)
     {
-        if (!_forSerialization) {
+        if (!_forSerialization && (name != null)) {
             if (_ignoredPropertyNames == null) {
                 _ignoredPropertyNames = new HashSet<String>();
             }
@@ -821,12 +1008,20 @@ public class POJOPropertiesCollector
                     old.addAll(prop);
                 }
                 // replace the creatorProperty too, if there is one
-                _updateCreatorProperty(prop, _creatorProperties);
-                // [databind#2001]: New name of property was ignored previously? Remove from ignored
-                // 01-May-2018, tatu: I have a feeling this will need to be revisited at some point,
-                //   to avoid removing some types of removals, possibly. But will do for now.
-                if (_ignoredPropertyNames != null) {
-                    _ignoredPropertyNames.remove(name);
+                if (_replaceCreatorProperty(prop, _creatorProperties)) {
+                    // [databind#2001]: New name of property was ignored previously? Remove from ignored
+                    // 01-May-2018, tatu: I have a feeling this will need to be revisited at some point,
+                    //   to avoid removing some types of removals, possibly. But will do for now.
+
+                    // 16-May-2020, tatu: ... and so the day came, [databind#2118] failed
+                    //    when explicit rename added to ignorals (for READ_ONLY) was suddenly
+                    //    removed from ignoral list. So, added a guard statement above so that
+                    //    ignoral is ONLY removed if there was matching creator property.
+                    //
+                    //    Chances are this is not the last tweak we need but... that bridge then etc
+                    if (_ignoredPropertyNames != null) {
+                        _ignoredPropertyNames.remove(name);
+                    }
                 }
             }
         }
@@ -851,16 +1046,15 @@ public class POJOPropertiesCollector
                     }
                 } else {
                     if (prop.hasSetter()) {
-                        rename = naming.nameForSetterMethod(_config, prop.getSetter(), fullName.getSimpleName());
+                        rename = naming.nameForSetterMethod(_config, prop.getSetterUnchecked(), fullName.getSimpleName());
                     } else if (prop.hasConstructorParameter()) {
                         rename = naming.nameForConstructorParameter(_config, prop.getConstructorParameter(), fullName.getSimpleName());
                     } else if (prop.hasField()) {
-                        rename = naming.nameForField(_config, prop.getField(), fullName.getSimpleName());
+                        rename = naming.nameForField(_config, prop.getFieldUnchecked(), fullName.getSimpleName());
                     } else if (prop.hasGetter()) {
-                        /* Plus, when getter-as-setter is used, need to convert that too..
-                         * (should we verify that's enabled? For now, assume it's ok always)
-                         */
-                        rename = naming.nameForGetterMethod(_config, prop.getGetter(), fullName.getSimpleName());
+                        // Plus, when getter-as-setter is used, need to convert that too..
+                        // (should we verify that's enabled? For now, assume it's ok always)
+                        rename = naming.nameForGetterMethod(_config, prop.getGetterUnchecked(), fullName.getSimpleName());
                     }
                 }
             }
@@ -880,7 +1074,7 @@ public class POJOPropertiesCollector
             }
 
             // replace the creatorProperty too, if there is one
-            _updateCreatorProperty(prop, _creatorProperties);
+            _replaceCreatorProperty(prop, _creatorProperties);
         }
     }
 
@@ -933,31 +1127,30 @@ public class POJOPropertiesCollector
     /**********************************************************
      */
     
-    /* First, order by [JACKSON-90] (explicit ordering and/or alphabetic)
-     * and then for [JACKSON-170] (implicitly order creator properties before others)
-     */
+    // First, order by(explicit ordering and/or alphabetic),
+    // then by (optional) index (if any)
+    // and then implicitly order creator properties before others)
+
     protected void _sortProperties(Map<String, POJOPropertyBuilder> props)
     {
         // Then how about explicit ordering?
-        AnnotationIntrospector intr = _annotationIntrospector;
-        Boolean alpha = intr.findSerializationSortAlphabetically((Annotated) _classDef);
-        boolean sort;
-        
-        if (alpha == null) {
-            sort = _config.shouldSortPropertiesAlphabetically();
-        } else {
-            sort = alpha.booleanValue();
-        }
+        final AnnotationIntrospector intr = _annotationIntrospector;
+        Boolean alpha = intr.findSerializationSortAlphabetically(_classDef);
+        final boolean sortAlpha = (alpha == null)
+                ? _config.shouldSortPropertiesAlphabetically()
+                : alpha.booleanValue();
+        final boolean indexed = _anyIndexed(props.values());
+
         String[] propertyOrder = intr.findSerializationPropertyOrder(_classDef);
         
         // no sorting? no need to shuffle, then
-        if (!sort && (_creatorProperties == null) && (propertyOrder == null)) {
+        if (!sortAlpha && !indexed && (_creatorProperties == null) && (propertyOrder == null)) {
             return;
         }
         int size = props.size();
         Map<String, POJOPropertyBuilder> all;
         // Need to (re)sort alphabetically?
-        if (sort) {
+        if (sortAlpha) {
             all = new TreeMap<String,POJOPropertyBuilder>();
         } else {
             all = new LinkedHashMap<String,POJOPropertyBuilder>(size+size);
@@ -966,11 +1159,11 @@ public class POJOPropertiesCollector
         for (POJOPropertyBuilder prop : props.values()) {
             all.put(prop.getName(), prop);
         }
-        Map<String,POJOPropertyBuilder> ordered = new LinkedHashMap<String,POJOPropertyBuilder>(size+size);
+        Map<String,POJOPropertyBuilder> ordered = new LinkedHashMap<>(size+size);
         // Ok: primarily by explicit order
         if (propertyOrder != null) {
             for (String name : propertyOrder) {
-                POJOPropertyBuilder w = all.get(name);
+                POJOPropertyBuilder w = all.remove(name);
                 if (w == null) { // will also allow use of "implicit" names for sorting
                     for (POJOPropertyBuilder prop : props.values()) {
                         if (name.equals(prop.getInternalName())) {
@@ -986,15 +1179,36 @@ public class POJOPropertiesCollector
                 }
             }
         }
-        // And secondly by sorting Creator properties before other unordered properties
-        if (_creatorProperties != null) {
+
+        // Second (starting with 2.11): index, if any:
+        if (indexed) {
+            Map<Integer,POJOPropertyBuilder> byIndex = new TreeMap<>();
+            Iterator<Map.Entry<String,POJOPropertyBuilder>> it = all.entrySet().iterator();
+            while (it.hasNext()) {
+                Map.Entry<String,POJOPropertyBuilder> entry = it.next();
+                POJOPropertyBuilder prop = entry.getValue();
+                Integer index = prop.getMetadata().getIndex();
+                if (index != null) {
+                    byIndex.put(index, prop);
+                    it.remove();
+                }
+            }
+            for (POJOPropertyBuilder prop : byIndex.values()) {
+                ordered.put(prop.getName(), prop);
+            }
+        }
+
+        // Third by sorting Creator properties before other unordered properties
+        // (unless strict ordering is requested)
+        if ((_creatorProperties != null)
+                && (!sortAlpha || _config.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST))) {
             /* As per [databind#311], this is bit delicate; but if alphabetic ordering
              * is mandated, at least ensure creator properties are in alphabetic
              * order. Related question of creator vs non-creator is punted for now,
              * so creator properties still fully predate non-creator ones.
              */
             Collection<POJOPropertyBuilder> cr;
-            if (sort) {
+            if (sortAlpha) {
                 TreeMap<String, POJOPropertyBuilder> sorted =
                         new TreeMap<String,POJOPropertyBuilder>();
                 for (POJOPropertyBuilder prop : _creatorProperties) {
@@ -1008,6 +1222,8 @@ public class POJOPropertiesCollector
                 // 16-Jan-2016, tatu: Related to [databind#1317], make sure not to accidentally
                 //    add back pruned creator properties!
                 String name = prop.getName();
+                // 27-Nov-2019, tatu: Not sure why, but we should NOT remove it from `all` tho:
+//                if (all.remove(name) != null) {
                 if (all.containsKey(name)) {
                     ordered.put(name, prop);
                 }
@@ -1017,7 +1233,16 @@ public class POJOPropertiesCollector
         ordered.putAll(all);
         props.clear();
         props.putAll(ordered);
-    }        
+    }
+
+    private boolean _anyIndexed(Collection<POJOPropertyBuilder> props) {
+        for (POJOPropertyBuilder prop : props) {
+            if (prop.getMetadata().hasIndex()) {
+                return true;
+            }
+        }
+        return false;
+    }
 
     /*
     /**********************************************************
@@ -1094,16 +1319,21 @@ public class POJOPropertiesCollector
                     _config.canOverrideAccessModifiers());
     }
 
+    @Deprecated // since 2.12.1 (temporarily missing from 2.12.0)
     protected void _updateCreatorProperty(POJOPropertyBuilder prop, List<POJOPropertyBuilder> creatorProperties) {
+        _replaceCreatorProperty(prop, creatorProperties);
+    }
 
+    protected boolean _replaceCreatorProperty(POJOPropertyBuilder prop, List<POJOPropertyBuilder> creatorProperties) {
         if (creatorProperties != null) {
             final String intName = prop.getInternalName();
             for (int i = 0, len = creatorProperties.size(); i < len; ++i) {
                 if (creatorProperties.get(i).getInternalName().equals(intName)) {
                     creatorProperties.set(i, prop);
-                    break;
+                    return true;
                 }
             }
         }
+        return false;
     }
 }
