@@ -1,11 +1,13 @@
 package com.fasterxml.jackson.databind.introspect;
 
 import java.util.*;
+import java.util.stream.Collectors;
 
 import com.fasterxml.jackson.annotation.JsonInclude;
 import com.fasterxml.jackson.annotation.JsonProperty;
 import com.fasterxml.jackson.annotation.JsonSetter;
 import com.fasterxml.jackson.annotation.Nulls;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.cfg.ConfigOverride;
 import com.fasterxml.jackson.databind.cfg.MapperConfig;
@@ -215,20 +217,30 @@ public class POJOPropertyBuilder
      */
 
     @Override
-    public PropertyMetadata getMetadata() {
+    public PropertyMetadata getMetadata()
+    {
         if (_metadata == null) {
-            final Boolean b = _findRequired();
-            final String desc = _findDescription();
-            final Integer idx = _findIndex();
-            final String def = _findDefaultValue();
-            if (b == null && idx == null && def == null) {
-                _metadata = (desc == null) ? PropertyMetadata.STD_REQUIRED_OR_OPTIONAL
-                        : PropertyMetadata.STD_REQUIRED_OR_OPTIONAL.withDescription(desc);
+            // 20-Jun-2020, tatu: Unfortunately strict checks lead to [databind#2757]
+            //   so we will need to try to avoid them at this point
+            final AnnotatedMember prim = getPrimaryMemberUnchecked();
+
+            if (prim == null) {
+                _metadata = PropertyMetadata.STD_REQUIRED_OR_OPTIONAL;
             } else {
-                _metadata = PropertyMetadata.construct(b, desc, idx, def);
-            }
-            if (!_forSerialization) {
-                _metadata = _getSetterInfo(_metadata);
+                final Boolean b = _annotationIntrospector.hasRequiredMarker(prim);
+                final String desc = _annotationIntrospector.findPropertyDescription(prim);
+                final Integer idx = _annotationIntrospector.findPropertyIndex(prim);
+                final String def = _annotationIntrospector.findPropertyDefaultValue(prim);
+
+                if (b == null && idx == null && def == null) {
+                    _metadata = (desc == null) ? PropertyMetadata.STD_REQUIRED_OR_OPTIONAL
+                            : PropertyMetadata.STD_REQUIRED_OR_OPTIONAL.withDescription(desc);
+                } else {
+                    _metadata = PropertyMetadata.construct(b, desc, idx, def);
+                }
+                if (!_forSerialization) {
+                    _metadata = _getSetterInfo(_metadata, prim);
+                }
             }
         }
         return _metadata;
@@ -238,23 +250,25 @@ public class POJOPropertyBuilder
      * Helper method that contains logic for accessing and merging all setter
      * information that we needed, regarding things like possible merging
      * of property value, and handling of incoming nulls.
+     * Only called for deserialization purposes.
      */
-    protected PropertyMetadata _getSetterInfo(PropertyMetadata metadata)
+    protected PropertyMetadata _getSetterInfo(PropertyMetadata metadata,
+            AnnotatedMember primary)
     {
         boolean needMerge = true;
         Nulls valueNulls = null;
         Nulls contentNulls = null;
-        
+
         // Slightly confusing: first, annotations should be accessed via primary member
-        // (mutator); but accessor is needed for actual merge operation. So:
-        AnnotatedMember prim = getPrimaryMember();
+        // (mutator); but accessor is needed for actual merge operation. So
+
         AnnotatedMember acc = getAccessor();
 
-        if (prim != null) {
+        if (primary != null) {
             // Ok, first: does property itself have something to say?
             if (_annotationIntrospector != null) {
                 if (acc != null) {
-                    Boolean b = _annotationIntrospector.findMergeInfo(prim);
+                    Boolean b = _annotationIntrospector.findMergeInfo(primary);
                     if (b != null) {
                         needMerge = false;
                         if (b.booleanValue()) {
@@ -262,7 +276,7 @@ public class POJOPropertyBuilder
                         }
                     }
                 }
-                JsonSetter.Value setterInfo = _annotationIntrospector.findSetterInfo(prim);
+                JsonSetter.Value setterInfo = _annotationIntrospector.findSetterInfo(primary);
                 if (setterInfo != null) {
                     valueNulls = setterInfo.nonDefaultValueNulls();
                     contentNulls = setterInfo.nonDefaultContentNulls();
@@ -271,7 +285,10 @@ public class POJOPropertyBuilder
             // If not, config override?
             // 25-Oct-2016, tatu: Either this, or type of accessor...
             if (needMerge || (valueNulls == null) || (contentNulls == null)) {
-                Class<?> rawType = getRawPrimaryType();
+                // 20-Jun-2020, tatu: Related to [databind#2757], need to find type
+                //   but keeping mind that type for setters is trickier; and that
+                //   generic typing gets tricky as well.
+                Class<?> rawType = _rawTypeOf(primary);
                 ConfigOverride co = _config.getConfigOverride(rawType);
                 JsonSetter.Value setterInfo = co.getSetterInfo();
                 if (setterInfo != null) {
@@ -329,7 +346,6 @@ public class POJOPropertyBuilder
                     // 09-Feb-2017, tatu: Not sure if this or `null` but...
                     return TypeFactory.unknownType();
                 }
-                return m.getType();
             }
             return m.getType();
         }
@@ -436,11 +452,24 @@ public class POJOPropertyBuilder
         _getters = curr.withoutNext();
         return curr.value;
     }
-    
+
+    /**
+     * Variant of {@link #getGetter} that does NOT trigger pruning of
+     * getter candidates.
+     */
+    protected AnnotatedMethod getGetterUnchecked()
+    {
+        Linked<AnnotatedMethod> curr = _getters;
+        if (curr == null) {
+            return null;
+        }
+        return curr.value;
+    }
+
     @Override
     public AnnotatedMethod getSetter()
     {
-        // Easy with zero or one getters...
+        // Easy with zero or one setters...
         Linked<AnnotatedMethod> curr = _setters;
         if (curr == null) {
             return null;
@@ -451,58 +480,127 @@ public class POJOPropertyBuilder
         }
         // But if multiple, verify that they do not conflict...
         for (; next != null; next = next.next) {
-            // Allow masking, i.e. do not fail if one is in super-class from the other
-            Class<?> currClass = curr.value.getDeclaringClass();
-            Class<?> nextClass = next.value.getDeclaringClass();
-            if (currClass != nextClass) {
-                if (currClass.isAssignableFrom(nextClass)) { // next is more specific
-                    curr = next;
-                    continue;
-                }
-                if (nextClass.isAssignableFrom(currClass)) { // current more specific
-                    continue;
-                }
-            }
-            AnnotatedMethod nextM = next.value;
-            AnnotatedMethod currM = curr.value;
-
-            /* 30-May-2014, tatu: Two levels of precedence:
-             * 
-             * 1. Regular setters ("setX(...)")
-             * 2. Implicit, possible setters ("x(...)")
-             */
-            int priNext = _setterPriority(nextM);
-            int priCurr = _setterPriority(currM);
-
-            if (priNext != priCurr) {
-                if (priNext < priCurr) {
-                    curr = next;
-                }
+            AnnotatedMethod selected = _selectSetter(curr.value, next.value);
+            if (selected == curr.value) {
                 continue;
             }
-            // 11-Dec-2015, tatu: As per [databind#1033] allow pluggable conflict resolution
-            if (_annotationIntrospector != null) {
-                AnnotatedMethod pref = _annotationIntrospector.resolveSetterConflict(_config,
-                        currM, nextM);
-                
-                // note: should be one of nextM/currM; but no need to check
-                if (pref == currM) {
-                    continue;
-                }
-                if (pref == nextM) {
-                    curr = next;
-                    continue;
-                }
+            if (selected == next.value) {
+                curr = next;
+                continue;
             }
-            throw new IllegalArgumentException(String.format(
- "Conflicting setter definitions for property \"%s\": %s vs %s",
- getName(), curr.value.getFullName(), next.value.getFullName()));
+            // 10-May-2021, tatu: unbreakable tie, for now; offline handling
+            return _selectSetterFromMultiple(curr, next);
         }
+
         // One more thing; to avoid having to do it again...
         _setters = curr.withoutNext();
         return curr.value;
     }
 
+    /**
+     * Variant of {@link #getSetter} that does NOT trigger pruning of
+     * setter candidates.
+     */
+    protected AnnotatedMethod getSetterUnchecked()
+    {
+        Linked<AnnotatedMethod> curr = _setters;
+        if (curr == null) {
+            return null;
+        }
+        return curr.value;
+    }
+
+    /**
+     * Helper method called in cases where we have encountered two setter methods
+     * that have same precedence and cannot be resolved. This does not yet necessarily
+     * mean a failure since it is possible something with a higher precedence could
+     * still be found; handling is just separated into separate method for convenience.
+     *
+     * @param curr
+     * @param next
+     *
+     * @return Chosen setter method, if any
+     *
+     * @throws IllegalArgumentException If conflict could not be resolved
+     *
+     * @since 2.13
+     */
+    protected AnnotatedMethod _selectSetterFromMultiple(Linked<AnnotatedMethod> curr,
+            Linked<AnnotatedMethod> next)
+    {
+        // First: store reference to the initial possible conflict
+        List<AnnotatedMethod> conflicts = new ArrayList<>();
+        conflicts.add(curr.value);
+        conflicts.add(next.value);
+
+        next = next.next;
+        for (; next != null; next = next.next) {
+            AnnotatedMethod selected = _selectSetter(curr.value, next.value);
+            if (selected == curr.value) {
+                // No change, next was lower-precedence
+                continue;
+            }
+            if (selected == next.value) {
+                // Hooray! Found a higher-priority one; clear conflict list
+                conflicts.clear();
+                curr = next;
+                continue;
+            }
+            // Tie means one more non-resolved, add
+            conflicts.add(next.value);
+        }
+
+        // It is possible we resolved it; if so:
+        if (conflicts.isEmpty()) {
+            _setters = curr.withoutNext();
+            return curr.value;
+        }
+        // Otherwise
+        String desc = conflicts.stream().map(AnnotatedMethod::getFullName)
+                .collect(Collectors.joining(" vs "));
+        throw new IllegalArgumentException(String.format(
+                "Conflicting setter definitions for property \"%s\": %s",
+                getName(), desc));
+    }
+
+    // @since 2.13
+    protected AnnotatedMethod _selectSetter(AnnotatedMethod currM, AnnotatedMethod nextM)
+    {
+        // Allow masking, i.e. do not fail if one is in super-class from the other
+        final Class<?> currClass = currM.getDeclaringClass();
+        final Class<?> nextClass = nextM.getDeclaringClass();
+        if (currClass != nextClass) {
+            if (currClass.isAssignableFrom(nextClass)) { // next is more specific
+                return nextM;
+            }
+            if (nextClass.isAssignableFrom(currClass)) { // current more specific
+                return currM;
+            }
+        }
+
+        /* 30-May-2014, tatu: Two levels of precedence:
+         * 
+         * 1. Regular setters ("setX(...)")
+         * 2. Implicit, possible setters ("x(...)")
+         */
+        // 25-Apr-2021, tatu: This is probably wrong, should not rely on
+        //    hard-coded "set" prefix here.
+        int priNext = _setterPriority(nextM);
+        int priCurr = _setterPriority(currM);
+
+        if (priNext != priCurr) {
+            // Smaller value, higher; so, if next has higher precedence:
+            if (priNext < priCurr) {
+                return nextM;
+            }
+            // otherwise current one has, proceed
+            return currM;
+        }
+        // 11-Dec-2015, tatu: As per [databind#1033] allow pluggable conflict resolution
+        return (_annotationIntrospector == null) ? null
+                : _annotationIntrospector.resolveSetterConflict(_config, currM, nextM);
+    }
+
     @Override
     public AnnotatedField getField()
     {
@@ -531,6 +629,19 @@ public class POJOPropertyBuilder
         return field;
     }
 
+    /**
+     * Variant of {@link #getField} that does NOT trigger pruning of
+     * Field candidates.
+     */
+    protected AnnotatedField getFieldUnchecked()
+    {
+        Linked<AnnotatedField> curr = _fields;
+        if (curr == null) {
+            return null;
+        }
+        return curr.value;
+    }
+
     @Override
     public AnnotatedParameter getConstructorParameter()
     {
@@ -576,6 +687,45 @@ public class POJOPropertyBuilder
         return m;
     }
 
+    // Sometimes we need to actually by-pass failures related to conflicting
+    // getters or setters (see [databind#2757] for specific example); if so,
+    // this method is to be used instead of `getPrimaryMember()`
+    // @since 2.11.1
+    protected AnnotatedMember getPrimaryMemberUnchecked() {
+        if (_forSerialization) { // Inlined `getAccessor()` logic:
+            // Inlined `getGetter()`:
+            if (_getters != null) {
+                return _getters.value;
+            }
+            // Inlined `getField()`:
+            if (_fields != null) {
+                return _fields.value;
+            }
+            return null;
+        }
+
+        // Otherwise, inlined `getMutator()` logic:
+
+        // Inlined `getConstructorParameter()`:
+        if (_ctorParameters != null) {
+            return _ctorParameters.value;
+        }
+        // Inlined `getSetter()`:
+        if (_setters != null) {
+            return _setters.value;
+        }
+        // Inlined `getField()`:
+        if (_fields != null) {
+            return _fields.value;
+        }
+        // but to support setterless-properties, also include part of
+        // `getAccessor()` not yet covered, `getGetter()`:
+        if (_getters != null) {
+            return _getters.value;
+        }
+        return null;
+    }
+
     protected int _getterPriority(AnnotatedMethod m)
     {
         final String name = m.getName();
@@ -648,42 +798,6 @@ public class POJOPropertyBuilder
         return (b != null) && b.booleanValue();
     }
 
-    protected Boolean _findRequired() {
-       return fromMemberAnnotations(new WithMember<Boolean>() {
-            @Override
-            public Boolean withMember(AnnotatedMember member) {
-                return _annotationIntrospector.hasRequiredMarker(member);
-            }
-        });
-    }
-    
-    protected String _findDescription() {
-        return fromMemberAnnotations(new WithMember<String>() {
-            @Override
-            public String withMember(AnnotatedMember member) {
-                return _annotationIntrospector.findPropertyDescription(member);
-            }
-        });
-    }
-
-    protected Integer _findIndex() {
-        return fromMemberAnnotations(new WithMember<Integer>() {
-            @Override
-            public Integer withMember(AnnotatedMember member) {
-                return _annotationIntrospector.findPropertyIndex(member);
-            }
-        });
-    }
-
-    protected String _findDefaultValue() {
-        return fromMemberAnnotations(new WithMember<String>() {
-            @Override
-            public String withMember(AnnotatedMember member) {
-                return _annotationIntrospector.findPropertyDefaultValue(member);
-            }
-        });
-    }
-    
     @Override
     public ObjectIdInfo findObjectIdInfo() {
         return fromMemberAnnotations(new WithMember<ObjectIdInfo>() {
@@ -782,11 +896,19 @@ public class POJOPropertyBuilder
         _ctorParameters = _removeIgnored(_ctorParameters);
     }
 
+    @Deprecated // since 2.12
+    public JsonProperty.Access removeNonVisible(boolean inferMutators) {
+        return removeNonVisible(inferMutators, null);
+    }
+    
     /**
      * @param inferMutators Whether mutators can be "pulled in" by visible
      *    accessors or not. 
+     *
+     * @since 2.12 (earlier had different signature)
      */
-    public JsonProperty.Access removeNonVisible(boolean inferMutators)
+    public JsonProperty.Access removeNonVisible(boolean inferMutators,
+            POJOPropertiesCollector parent)
     {
         /* 07-Jun-2015, tatu: With 2.6, we will allow optional definition
          *  of explicit access type for property; if not "AUTO", it will
@@ -798,6 +920,15 @@ public class POJOPropertyBuilder
         }
         switch (acc) {
         case READ_ONLY:
+            // [databind#2719]: Need to add ignorals, first, keeping in mind
+            // we have not yet resolved explicit names, so include implicit
+            // and possible explicit names
+            if (parent != null) {
+                parent._collectIgnorals(getName());
+                for (PropertyName pn : findExplicitNames()) {
+                    parent._collectIgnorals(pn.getSimpleName());
+                }
+            }
             // Remove setters, creators for sure, but fields too if deserializing
             _setters = null;
             _ctorParameters = null;
@@ -989,7 +1120,7 @@ public class POJOPropertyBuilder
         }
         return false;
     }
-    
+
     public boolean anyIgnorals() {
         return _anyIgnorals(_fields)
             || _anyIgnorals(_getters)
@@ -1008,6 +1139,37 @@ public class POJOPropertyBuilder
         return false;
     }
 
+    // @since 2.14
+    public boolean anyExplicitsWithoutIgnoral() {
+        return _anyExplicitsWithoutIgnoral(_fields)
+                || _anyExplicitsWithoutIgnoral(_getters)
+                || _anyExplicitsWithoutIgnoral(_setters)
+                // as per [databind#1317], constructor names are special...
+                || _anyExplicitNamesWithoutIgnoral(_ctorParameters);
+    }
+
+    // For accessors other than constructor parameters
+    private <T> boolean _anyExplicitsWithoutIgnoral(Linked<T> n) {
+        for (; n != null; n = n.next) {
+            if (!n.isMarkedIgnored
+                && (n.name != null && n.name.hasSimpleName())) {
+                    return true;
+            }
+        }
+        return false;
+    }
+
+    // For constructor parameters
+    private <T> boolean _anyExplicitNamesWithoutIgnoral(Linked<T> n) {
+        for (; n != null; n = n.next) {
+            if (!n.isMarkedIgnored
+                && (n.name != null && n.isNameExplicit)) {
+                    return true;
+            }
+        }
+        return false;
+    }
+
     /**
      * Method called to find out set of explicit names for accessors
      * bound together due to implicit name.
@@ -1059,8 +1221,8 @@ public class POJOPropertyBuilder
                     continue;
                 }
                 
-                throw new IllegalStateException("Conflicting/ambiguous property name definitions (implicit name '"
-                        +_name+"'): found multiple explicit names: "
+                throw new IllegalStateException("Conflicting/ambiguous property name definitions (implicit name "
+                        +ClassUtil.name(_name)+"): found multiple explicit names: "
                         +newNames+", but also implicit accessor: "+node);
             }
             POJOPropertyBuilder prop = props.get(name);
@@ -1087,7 +1249,7 @@ public class POJOPropertyBuilder
             }
         }
     }
-    
+
     private Set<PropertyName> _findExplicitNames(Linked<? extends AnnotatedMember> node,
             Set<PropertyName> renamed)
     {
@@ -1108,7 +1270,7 @@ public class POJOPropertyBuilder
         }
         return renamed;
     }
-    
+
     // For trouble-shooting
     @Override
     public String toString()
@@ -1123,7 +1285,7 @@ public class POJOPropertyBuilder
         sb.append("]");
         return sb.toString();
     }
-    
+
     /*
     /**********************************************************
     /* Helper methods
@@ -1219,6 +1381,26 @@ public class POJOPropertyBuilder
         return null;
     }
 
+    // Helper method needed to work around oddity in type access for
+    // `AnnotatedMethod`.
+    //
+    // @since 2.11.1
+    protected Class<?> _rawTypeOf(AnnotatedMember m) {
+        // AnnotatedMethod always returns return type, but for setters we
+        // actually need argument type
+        if (m instanceof AnnotatedMethod) {
+            AnnotatedMethod meh = (AnnotatedMethod) m;
+            if (meh.getParameterCount() > 0) {
+                // note: get raw type FROM full type since only that resolves
+                // generic types
+                return meh.getParameterType(0).getRawClass();
+            }
+        }
+        // same as above, must get fully resolved type to handled generic typing
+        // of fields etc.
+        return m.getType().getRawClass();
+    }
+
     /*
     /**********************************************************
     /* Helper classes
