@@ -1,20 +1,17 @@
 package com.fasterxml.jackson.databind.node;
 
-import com.fasterxml.jackson.core.*;
-import com.fasterxml.jackson.core.type.WritableTypeId;
-import com.fasterxml.jackson.databind.JsonNode;
-import com.fasterxml.jackson.databind.SerializerProvider;
-import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
-import com.fasterxml.jackson.databind.util.RawValue;
-
 import java.io.IOException;
 import java.math.BigDecimal;
 import java.math.BigInteger;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Comparator;
-import java.util.Iterator;
-import java.util.List;
+import java.util.*;
+
+import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.type.WritableTypeId;
+
+import com.fasterxml.jackson.databind.JsonNode;
+import com.fasterxml.jackson.databind.SerializerProvider;
+import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
+import com.fasterxml.jackson.databind.util.RawValue;
 
 /**
  * Node class that represents Arrays mapped from JSON content.
@@ -23,7 +20,10 @@ import java.util.List;
  */
 public class ArrayNode
     extends ContainerNode<ArrayNode>
+    implements java.io.Serializable // since 2.10
 {
+    private static final long serialVersionUID = 1L;
+
     private final List<JsonNode> _children;
 
     public ArrayNode(JsonNodeFactory nf) {
@@ -65,6 +65,150 @@ public class ArrayNode
         return ret;
     }
 
+    /*
+    /**********************************************************
+    /* Support for withArray()/withObject()
+    /**********************************************************
+     */
+
+    @SuppressWarnings("unchecked")
+    @Deprecated
+    @Override
+    public ObjectNode with(String exprOrProperty) {
+        JsonPointer ptr = _jsonPointerIfValid(exprOrProperty);
+        if (ptr != null) {
+            return withObject(ptr);
+        }
+        return super.with(exprOrProperty); // to give failure
+    }
+
+    @SuppressWarnings("unchecked")
+    @Override
+    public ArrayNode withArray(String exprOrProperty)
+    {
+        JsonPointer ptr = _jsonPointerIfValid(exprOrProperty);
+        if (ptr != null) {
+            return withArray(ptr);
+        }
+        return super.withArray(exprOrProperty); // to give failure
+    }
+
+    @Override
+    protected ObjectNode _withObject(JsonPointer origPtr,
+            JsonPointer currentPtr,
+            OverwriteMode overwriteMode, boolean preferIndex)
+    {
+        if (currentPtr.matches()) {
+            // Cannot return, not an ObjectNode so:
+            return null;
+        }
+        JsonNode n = _at(currentPtr);
+        // If there's a path, follow it
+        if ((n != null) && (n instanceof BaseJsonNode)) {
+            ObjectNode found = ((BaseJsonNode) n)._withObject(origPtr, currentPtr.tail(),
+                    overwriteMode, preferIndex);
+            if (found != null) {
+                return found;
+            }
+            // Ok no; must replace if allowed to
+            _withXxxVerifyReplace(origPtr, currentPtr, overwriteMode, preferIndex, n);
+        }
+        // Either way; must replace or add a new property
+        return _withObjectAddTailElement(currentPtr, preferIndex);
+    }
+
+    @Override
+    protected ArrayNode _withArray(JsonPointer origPtr,
+            JsonPointer currentPtr,
+            OverwriteMode overwriteMode, boolean preferIndex)
+    {
+        if (currentPtr.matches()) {
+            return this;
+        }
+        JsonNode n = _at(currentPtr);
+        // If there's a path, follow it
+        if ((n != null) && (n instanceof BaseJsonNode)) {
+            ArrayNode found = ((BaseJsonNode) n)._withArray(origPtr, currentPtr.tail(),
+                    overwriteMode, preferIndex);
+            if (found != null) {
+                return found;
+            }
+            // Ok no; must replace if allowed to
+            _withXxxVerifyReplace(origPtr, currentPtr, overwriteMode, preferIndex, n);
+        }
+        // Either way; must replace or add a new property
+        return _withArrayAddTailElement(currentPtr, preferIndex);
+    }
+
+    protected ObjectNode _withObjectAddTailElement(JsonPointer tail, boolean preferIndex)
+    {
+        final int index = tail.getMatchingIndex();
+        if (index < 0) {
+            return null;
+        }
+
+        tail = tail.tail();
+
+        // First: did we complete traversal? If so, easy, we got our result
+        if (tail.matches()) {
+            ObjectNode result = this.objectNode();
+            _withXxxSetArrayElement(index, result);
+            return result;
+        }
+
+        // Otherwise, do we want Array or Object
+        if (preferIndex && tail.mayMatchElement()) { // array!
+            ArrayNode next = this.arrayNode();
+            _withXxxSetArrayElement(index, next);
+            return next._withObjectAddTailElement(tail, preferIndex);
+        }
+        ObjectNode next = this.objectNode();
+        _withXxxSetArrayElement(index, next);
+        return next._withObjectAddTailProperty(tail, preferIndex);
+    }
+
+    protected ArrayNode _withArrayAddTailElement(JsonPointer tail, boolean preferIndex)
+    {
+        final int index = tail.getMatchingIndex();
+        if (index < 0) {
+            return null;
+        }
+        tail = tail.tail();
+
+        // First: did we complete traversal? If so, easy, we got our result
+        if (tail.matches()) {
+            ArrayNode result = this.arrayNode();
+            _withXxxSetArrayElement(index, result);
+            return result;
+        }
+
+        // Otherwise, do we want Array or Object
+        if (preferIndex && tail.mayMatchElement()) { // array!
+            ArrayNode next = this.arrayNode();
+            _withXxxSetArrayElement(index, next);
+            return next._withArrayAddTailElement(tail, preferIndex);
+        }
+        ArrayNode next = this.arrayNode();
+        _withXxxSetArrayElement(index, next);
+        return next._withArrayAddTailElement(tail, preferIndex);
+    }
+
+    protected void _withXxxSetArrayElement(int index, JsonNode value) {
+        // 27-Jul-2022, tatu: Let's make it less likely anyone OOMs by
+        //    humongous index...
+        if (index >= size()) {
+            final int max = _nodeFactory.getMaxElementIndexForInsert();
+            if (index > max) {
+                _reportWrongNodeOperation("Too big Array index (%d; max %d) to use for insert with `JsonPointer`",
+                        index, max);
+            }
+            while (index >= this.size()) {
+                addNull();
+            }
+        }
+        set(index, value);
+    }
+
     /*
     /**********************************************************
     /* Overrides for JsonSerializable.Base
@@ -99,6 +243,9 @@ public class ArrayNode
         return _children.size();
     }
 
+    @Override // since 2.10
+    public boolean isEmpty() { return _children.isEmpty(); }
+
     @Override
     public Iterator<JsonNode> elements() {
         return _children.iterator();
@@ -106,7 +253,7 @@ public class ArrayNode
 
     @Override
     public JsonNode get(int index) {
-        if (index >= 0 && index < _children.size()) {
+        if ((index >= 0) && (index < _children.size())) {
             return _children.get(index);
         }
         return null;
@@ -126,6 +273,15 @@ public class ArrayNode
         return MissingNode.getInstance();
     }
 
+    @Override
+    public JsonNode required(int index) {
+        if ((index >= 0) && (index < _children.size())) {
+            return _children.get(index);
+        }
+        return _reportRequiredViolation("No value at index #%d [0, %d) of `ArrayNode`",
+                index, _children.size());
+    }
+
     @Override
     public boolean equals(Comparator<JsonNode> comparator, JsonNode o)
     {
@@ -154,17 +310,16 @@ public class ArrayNode
      */
 
     @Override
-    public void serialize(JsonGenerator f, SerializerProvider provider) throws IOException
+    public void serialize(JsonGenerator g, SerializerProvider provider) throws IOException
     {
         final List<JsonNode> c = _children;
         final int size = c.size();
-        f.writeStartArray(size);
+        g.writeStartArray(this, size);
         for (int i = 0; i < size; ++i) { // we'll typically have array list
-            // For now, assuming it's either BaseJsonNode, JsonSerializable
-            JsonNode n = c.get(i);
-            ((BaseJsonNode) n).serialize(f, provider);
+            JsonNode value = c.get(i);
+            value.serialize(g, provider);
         }
-        f.writeEndArray();
+        g.writeEndArray();
     }
 
     @Override
@@ -301,14 +456,16 @@ public class ArrayNode
      */
     public ArrayNode addAll(Collection<? extends JsonNode> nodes)
     {
-        _children.addAll(nodes);
+        for (JsonNode node : nodes) {
+            add(node);
+        }
         return this;
     }
 
     /**
      * Method for inserting specified child node as an element
      * of this Array. If index is 0 or less, it will be inserted as
-     * the first element; if >= size(), appended at the end, and otherwise
+     * the first element; if {@code >= size()}, appended at the end, and otherwise
      * inserted before existing element in specified index.
      * No exceptions are thrown for any index.
      *
@@ -353,7 +510,7 @@ public class ArrayNode
 
     /*
     /**********************************************************
-    /* Extended ObjectNode API, mutators, generic; addXxx()/insertXxx()
+    /* Extended ObjectNode API, mutators, generic; addXxx()/insertXxx()/setXxx()
     /**********************************************************
      */
 
@@ -361,7 +518,7 @@ public class ArrayNode
      * Method that will construct an ArrayNode and add it at the end
      * of this array node.
      *
-     * @return Newly constructed ArrayNode
+     * @return Newly constructed ArrayNode (NOTE: NOT `this` ArrayNode)
      */
     public ArrayNode addArray()
     {
@@ -374,7 +531,7 @@ public class ArrayNode
      * Method that will construct an ObjectNode and add it at the end
      * of this array node.
      *
-     * @return Newly constructed ObjectNode
+     * @return Newly constructed ObjectNode (NOTE: NOT `this` ArrayNode)
      */
     public ObjectNode addObject()
     {
@@ -389,14 +546,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode addPOJO(Object value)
-    {
-        if (value == null) {
-            addNull();
-        } else {
-            _add(pojoNode(value));
-        }
-        return this;
+    public ArrayNode addPOJO(Object pojo) {
+        return _add((pojo == null) ? nullNode() : pojoNode(pojo));
     }
 
     /**
@@ -405,12 +556,7 @@ public class ArrayNode
      * @since 2.6
      */
     public ArrayNode addRawValue(RawValue raw) {
-        if (raw == null) {
-            addNull();
-        } else {
-            _add(rawValueNode(raw));
-        }
-        return this;
+        return _add((raw == null) ? nullNode() : rawValueNode(raw));
     }
 
     /**
@@ -418,10 +564,31 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode addNull()
-    {
-        _add(nullNode());
-        return this;
+    public ArrayNode addNull() {
+        return _add(nullNode());
+    }
+
+    /**
+     * Method for adding specified number at the end of this array.
+     *
+     * @return This array node, to allow chaining
+     *
+     * @since 2.13
+     */
+    public ArrayNode add(short v) {
+        return _add(numberNode(v));
+    }
+
+    /**
+     * Alternative method that we need to avoid bumping into NPE issues
+     * with auto-unboxing.
+     *
+     * @return This array node, to allow chaining
+     *
+     * @since 2.13
+     */
+    public ArrayNode add(Short v) {
+        return _add((v == null) ? nullNode() : numberNode(v.shortValue()));
     }
 
     /**
@@ -430,8 +597,7 @@ public class ArrayNode
      * @return This array node, to allow chaining
      */
     public ArrayNode add(int v) {
-        _add(numberNode(v));
-        return this;
+        return _add(numberNode(v));
     }
 
     /**
@@ -440,11 +606,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode add(Integer value) {
-        if (value == null) {
-            return addNull();
-        }
-        return _add(numberNode(value.intValue()));
+    public ArrayNode add(Integer v) {
+        return _add((v == null) ? nullNode() : numberNode(v.intValue()));
     }
 
     /**
@@ -460,11 +623,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode add(Long value) {
-        if (value == null) {
-            return addNull();
-        }
-        return _add(numberNode(value.longValue()));
+    public ArrayNode add(Long v) {
+        return _add((v == null) ? nullNode() : numberNode(v.longValue()));
     }
 
     /**
@@ -482,11 +642,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode add(Float value) {
-        if (value == null) {
-            return addNull();
-        }
-        return _add(numberNode(value.floatValue()));
+    public ArrayNode add(Float v) {
+        return _add((v == null) ? nullNode() : numberNode(v.floatValue()));
     }
 
     /**
@@ -504,11 +661,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode add(Double value) {
-        if (value == null) {
-            return addNull();
-        }
-        return _add(numberNode(value.doubleValue()));
+    public ArrayNode add(Double v) {
+        return _add((v == null) ? nullNode() : numberNode(v.doubleValue()));
     }
 
     /**
@@ -517,10 +671,7 @@ public class ArrayNode
      * @return This array node, to allow chaining
      */
     public ArrayNode add(BigDecimal v) {
-        if (v == null) {
-            return addNull();
-        }
-        return _add(numberNode(v));
+        return _add((v == null) ? nullNode() : numberNode(v));
     }
 
     /**
@@ -531,22 +682,16 @@ public class ArrayNode
      * @since 2.9
      */
     public ArrayNode add(BigInteger v) {
-        if (v == null) {
-            return addNull();
-        }
-        return _add(numberNode(v));
+        return _add((v == null) ? nullNode() : numberNode(v));
     }
-    
+
     /**
      * Method for adding specified String value at the end of this array.
      *
      * @return This array node, to allow chaining
      */
     public ArrayNode add(String v) {
-        if (v == null) {
-            return addNull();
-        }
-        return _add(textNode(v));
+        return _add((v == null) ? nullNode() : textNode(v));
     }
 
     /**
@@ -564,11 +709,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode add(Boolean value) {
-        if (value == null) {
-            return addNull();
-        }
-        return _add(booleanNode(value.booleanValue()));
+    public ArrayNode add(Boolean v) {
+        return _add((v == null) ? nullNode() : booleanNode(v.booleanValue()));
     }
 
     /**
@@ -578,10 +720,7 @@ public class ArrayNode
      * @return This array node, to allow chaining
      */
     public ArrayNode add(byte[] v) {
-        if (v == null) {
-            return addNull();
-        }
-        return _add(binaryNode(v));
+        return _add((v == null) ? nullNode() : binaryNode(v));
     }
 
     /**
@@ -589,6 +728,8 @@ public class ArrayNode
      * specified point in the array,
      * and returning the <b>newly created array</b>
      * (note: NOT 'this' array)
+     *
+     * @return Newly constructed {@code ArrayNode} (note! NOT `this` ArrayNode)
      */
     public ArrayNode insertArray(int index)
     {
@@ -602,7 +743,7 @@ public class ArrayNode
      * of this array, and returning the <b>newly created node</b>
      * (note: NOT 'this' array)
      *
-     * @return Newly constructed ObjectNode
+     * @return Newly constructed {@code ObjectNode} (note! NOT `this` ArrayNode)
      */
     public ObjectNode insertObject(int index)
     {
@@ -611,30 +752,57 @@ public class ArrayNode
         return n;
     }
 
-    /**
-     * Method that will construct a POJONode and
-     * insert it at specified position in this array.
-     *
-     * @return This array node, to allow chaining
-     */
-    public ArrayNode insertPOJO(int index, Object value)
-    {
-        if (value == null) {
-            return insertNull(index);
-        }
-        return _insert(index, pojoNode(value));
-    }
-
     /**
      * Method that will insert a null value
      * at specified position in this array.
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode insertNull(int index)
-    {
-        _insert(index, nullNode());
-        return this;
+    public ArrayNode insertNull(int index) {
+        return _insert(index, nullNode());
+    }
+
+    /**
+     * Method that will construct a POJONode and
+     * insert it at specified position in this array.
+     *
+     * @return This array node, to allow chaining
+     */
+    public ArrayNode insertPOJO(int index, Object pojo) {
+        return _insert(index, (pojo == null) ? nullNode() : pojoNode(pojo));
+    }
+
+    /**
+     * @return This array node, to allow chaining
+     *
+     * @since 2.13
+     */
+    public ArrayNode insertRawValue(int index, RawValue raw) {
+        return _insert(index, (raw == null) ? nullNode() : rawValueNode(raw));
+    }
+
+    /**
+     * Method that will insert specified numeric value
+     * at specified position in this array.
+     *
+     * @return This array node, to allow chaining
+     *
+     * @since 2.13
+     */
+    public ArrayNode insert(int index, short v) {
+        return _insert(index, numberNode(v));
+    }
+
+    /**
+     * Alternative method that we need to avoid bumping into NPE issues
+     * with auto-unboxing.
+     *
+     * @return This array node, to allow chaining
+     *
+     * @since 2.13
+     */
+    public ArrayNode insert(int index, Short value) {
+        return _insert(index, (value == null) ? nullNode() : numberNode(value.shortValue()));
     }
 
     /**
@@ -644,8 +812,7 @@ public class ArrayNode
      * @return This array node, to allow chaining
      */
     public ArrayNode insert(int index, int v) {
-        _insert(index, numberNode(v));
-        return this;
+        return _insert(index, numberNode(v));
     }
 
     /**
@@ -654,13 +821,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode insert(int index, Integer value) {
-        if (value == null) {
-            insertNull(index);
-        } else {
-            _insert(index, numberNode(value.intValue()));
-        }
-        return this;
+    public ArrayNode insert(int index, Integer v) {
+        return _insert(index, (v == null) ? nullNode() : numberNode(v.intValue()));
     }
 
     /**
@@ -679,11 +841,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode insert(int index, Long value) {
-        if (value == null) {
-            return insertNull(index);
-        }
-        return _insert(index, numberNode(value.longValue()));
+    public ArrayNode insert(int index, Long v) {
+        return _insert(index, (v == null) ? nullNode() : numberNode(v.longValue()));
     }
 
     /**
@@ -702,11 +861,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode insert(int index, Float value) {
-        if (value == null) {
-            return insertNull(index);
-        }
-        return _insert(index, numberNode(value.floatValue()));
+    public ArrayNode insert(int index, Float v) {
+        return _insert(index, (v == null) ? nullNode() : numberNode(v.floatValue()));
     }
 
     /**
@@ -725,11 +881,8 @@ public class ArrayNode
      *
      * @return This array node, to allow chaining
      */
-    public ArrayNode insert(int index, Double value) {
-        if (value == null) {
-            return insertNull(index);
-        }
-        return _insert(index, numberNode(value.doubleValue()));
+    public ArrayNode insert(int index, Double v) {
+        return _insert(index, (v == null) ? nullNode() : numberNode(v.doubleValue()));
     }
 
     /**
@@ -739,10 +892,7 @@ public class ArrayNode
      * @return This array node, to allow chaining
      */
     public ArrayNode insert(int index, BigDecimal v) {
-        if (v == null) {
-            return insertNull(index);
-        }
-        return _insert(index, numberNode(v));
+        return _insert(index, (v == null) ? nullNode() : numberNode(v));
     }
 
     /**
@@ -754,12 +904,9 @@ public class ArrayNode
      * @since 2.9
      */
     public ArrayNode insert(int index, BigInteger v) {
-        if (v == null) {
-            return insertNull(index);
-        }
-        return _insert(index, numberNode(v));
+        return _insert(index, (v == null) ? nullNode() : numberNode(v));
     }
-    
+
     /**
      * Method that will insert specified String
      * at specified position in this array.
@@ -767,10 +914,7 @@ public class ArrayNode
      * @return This array node, to allow chaining
      */
     public ArrayNode insert(int index, String v) {
-        if (v == null) {
-            return insertNull(index);
-        }
-        return _insert(index, textNode(v));
+        return _insert(index, (v == null) ? nullNode() : textNode(v));
     }
 
     /**
@@ -810,6 +954,206 @@ public class ArrayNode
         return _insert(index, binaryNode(v));
     }
 
+    /**
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode setNull(int index) {
+        return _set(index, nullNode());
+    }
+
+    /**
+     * @return This array node, to allow chaining
+     *
+     * @since 2.13
+     */
+    public ArrayNode setPOJO(int index, Object pojo) {
+        return _set(index, (pojo == null) ? nullNode() : pojoNode(pojo));
+    }
+
+    /**
+     * @return This array node, to allow chaining
+     *
+     * @since 2.13
+     */
+    public ArrayNode setRawValue(int index, RawValue raw) {
+        return _set(index, (raw == null) ? nullNode() : rawValueNode(raw));
+    }
+
+    /**
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, short v) {
+        return _set(index, numberNode(v));
+    }
+
+    /**
+     * Alternative method that we need to avoid bumping into NPE issues
+     * with auto-unboxing.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, Short v) {
+        return _set(index, (v == null) ? nullNode() : numberNode(v.shortValue()));
+    }
+
+    /**
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, int v) {
+        return _set(index, numberNode(v));
+    }
+
+    /**
+     * Alternative method that we need to avoid bumping into NPE issues
+     * with auto-unboxing.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, Integer v) {
+        return _set(index, (v == null) ? nullNode() : numberNode(v.intValue()));
+    }
+
+    /**
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, long v) {
+        return _set(index, numberNode(v));
+    }
+
+    /**
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, Long v) {
+        return _set(index, (v == null) ? nullNode() : numberNode(v.longValue()));
+    }
+
+    /**
+     * Method for setting value of a field to specified numeric value.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, float v) {
+        return _set(index, numberNode(v));
+    }
+
+    /**
+     * Alternative method that we need to avoid bumping into NPE issues
+     * with auto-unboxing.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, Float v) {
+        return _set(index, (v == null) ? nullNode() : numberNode(v.floatValue()));
+    }
+
+    /**
+     * Method for setting value of a field to specified numeric value.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, double v) {
+        return _set(index, numberNode(v));
+    }
+
+    /**
+     * Alternative method that we need to avoid bumping into NPE issues
+     * with auto-unboxing.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, Double v) {
+        return _set(index, (v == null) ? nullNode() : numberNode(v.doubleValue()));
+    }
+
+    /**
+     * Method for setting value of a field to specified numeric value.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, BigDecimal v) {
+        return _set(index, (v == null) ? nullNode() : numberNode(v));
+    }
+
+    /**
+     * Method for setting value of a field to specified numeric value.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, BigInteger v) {
+        return _set(index, (v == null) ? nullNode() : numberNode(v));
+    }
+
+    /**
+     * Method for setting value of a field to specified String value.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, String v) {
+        return _set(index, (v == null) ? nullNode() : textNode(v));
+    }
+
+    /**
+     * Method for setting value of a field to specified String value.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, boolean v) {
+        return _set(index, booleanNode(v));
+    }
+
+    /**
+     * Alternative method that we need to avoid bumping into NPE issues
+     * with auto-unboxing.
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, Boolean v) {
+        return _set(index, (v == null) ? nullNode() : booleanNode(v.booleanValue()));
+    }
+
+    /**
+     * Method for setting value of a field to specified binary value
+     *
+     * @return This node (to allow chaining)
+     *
+     * @since 2.13
+     */
+    public ArrayNode set(int index, byte[] v) {
+        return _set(index, (v == null) ? nullNode() : binaryNode(v));
+    }
+
     /*
     /**********************************************************
     /* Standard methods
@@ -839,27 +1183,20 @@ public class ArrayNode
         return _children.hashCode();
     }
 
-    @Override
-    public String toString()
-    {
-        StringBuilder sb = new StringBuilder(16 + (size() << 4));
-        sb.append('[');
-        for (int i = 0, len = _children.size(); i < len; ++i) {
-            if (i > 0) {
-                sb.append(',');
-            }
-            sb.append(_children.get(i).toString());
-        }
-        sb.append(']');
-        return sb.toString();
-    }
-
     /*
     /**********************************************************
     /* Internal methods (overridable)
     /**********************************************************
      */
 
+    protected ArrayNode _set(int index, JsonNode node) {
+        if (index < 0 || index >= _children.size()) {
+            throw new IndexOutOfBoundsException("Illegal index "+ index +", array size "+size());
+        }
+        _children.set(index, node);
+        return this;
+    }
+
     protected ArrayNode _add(JsonNode node) {
         _children.add(node);
         return this;
