@@ -29,6 +29,7 @@ import com.fasterxml.jackson.databind.util.BeanUtil;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 import com.fasterxml.jackson.databind.util.Converter;
 import com.fasterxml.jackson.databind.util.IgnorePropertiesUtil;
+import com.fasterxml.jackson.databind.util.NativeImageUtil;
 
 /**
  * Factory class that can provide serializers for any regular Java beans
@@ -416,6 +417,10 @@ public class BeanSerializerFactory
         }
 
         // Any properties to suppress?
+
+        // 10-Dec-2021, tatu: [databind#3305] Some JDK types need special help
+        //    (initially, `CharSequence` with its `isEmpty()` default impl)
+        props = filterUnwantedJDKProperties(config, beanDesc, props);
         props = filterBeanProperties(config, beanDesc, props);
 
         // Need to allow reordering of properties to serialize
@@ -472,7 +477,10 @@ public class BeanSerializerFactory
         }
         if (ser == null) { // Means that no properties were found
             // 21-Aug-2020, tatu: Empty Records should be fine tho
-            if (type.isRecordType()) {
+            // 18-Mar-2022, yawkat: [databind#3417] Record will also appear empty when missing
+            // reflection info. needsReflectionConfiguration will check that a constructor is present,
+            // else we fall back to the empty bean error msg
+            if (type.isRecordType() && !NativeImageUtil.needsReflectionConfiguration(type.getRawClass())) {
                 return builder.createDummy();
             }
 
@@ -636,7 +644,7 @@ ClassUtil.getTypeDescription(beanDesc.getType()), ClassUtil.name(propName)));
     /* Overridable non-public methods for manipulating bean properties
     /**********************************************************
      */
-    
+
     /**
      * Overridable method that can filter out properties. Default implementation
      * checks annotations class may have.
@@ -672,6 +680,37 @@ ClassUtil.getTypeDescription(beanDesc.getType()), ClassUtil.name(propName)));
         return props;
     }
 
+    /**
+     * Overridable method used to filter out specifically problematic JDK provided
+     * properties.
+     *<p>
+     * See issue <a href="https://github.com/FasterXML/jackson-databind/issues/3305">
+     * databind-3305</a> for details.
+     *
+     * @since 2.13.1
+     */
+    protected List<BeanPropertyWriter> filterUnwantedJDKProperties(SerializationConfig config,
+            BeanDescription beanDesc, List<BeanPropertyWriter> props)
+    {
+        // First, only consider something that implements `CharSequence`
+        if (beanDesc.getType().isTypeOrSubTypeOf(CharSequence.class)) {
+            // And only has a single property from "isEmpty()" default method
+            if (props.size() == 1) {
+                BeanPropertyWriter prop = props.get(0);
+                // And only remove property induced by `isEmpty()` method declared
+                // in `CharSequence` (default implementation)
+                // (could in theory relax this limit, probably but... should be fine)
+                AnnotatedMember m = prop.getMember();
+                if ((m instanceof AnnotatedMethod)
+                        && "isEmpty".equals(m.getName())
+                        && m.getDeclaringClass() == CharSequence.class) {
+                    props.remove(0);
+                }
+            }
+        }
+        return props;
+    }
+
     /**
      * Method called to handle view information for constructed serializer,
      * based on bean property writers.
