@@ -141,8 +141,7 @@ public class StdArraySerializers
                 serializeContents(value, g, provider);
                 return;
             }
-            g.writeStartArray(len);
-            g.setCurrentValue(value);
+            g.writeStartArray(value, len);
             serializeContents(value, g, provider);
             g.writeEndArray();
         }
@@ -219,13 +218,11 @@ public class StdArraySerializers
                 serializeContents(value, g, provider);
                 return;
             }
-            g.writeStartArray(len);
-            g.setCurrentValue(value);
+            g.writeStartArray(value, len);
             serializeContents(value, g, provider);
             g.writeEndArray();
         }
-        
-        @SuppressWarnings("cast")
+
         @Override
         public void serializeContents(short[] value, JsonGenerator g, SerializerProvider provider)
             throws IOException
@@ -274,8 +271,7 @@ public class StdArraySerializers
         {
             // [JACKSON-289] allows serializing as 'sparse' char array too:
             if (provider.isEnabled(SerializationFeature.WRITE_CHAR_ARRAYS_AS_JSON_ARRAYS)) {
-                g.writeStartArray(value.length);
-                g.setCurrentValue(value);
+                g.writeStartArray(value, value.length);
                 _writeArrayContents(g, value);
                 g.writeEndArray();
             } else {
@@ -389,7 +385,6 @@ public class StdArraySerializers
                 return;
             }
             // 11-May-2016, tatu: As per [core#277] we have efficient `writeArray(...)` available
-            g.setCurrentValue(value);
             g.writeArray(value, 0, value.length);
         }
 
@@ -462,7 +457,6 @@ public class StdArraySerializers
                 return;
             }
             // 11-May-2016, tatu: As per [core#277] we have efficient `writeArray(...)` available
-            g.setCurrentValue(value);
             g.writeArray(value, 0, value.length);
         }
         
@@ -539,8 +533,7 @@ public class StdArraySerializers
                 serializeContents(value, g, provider);
                 return;
             }
-            g.writeStartArray(len);
-            g.setCurrentValue(value);
+            g.writeStartArray(value, len);
             serializeContents(value, g, provider);
             g.writeEndArray();
         }
@@ -626,7 +619,6 @@ public class StdArraySerializers
                 serializeContents(value, g, provider);
                 return;
             }
-            g.setCurrentValue(value);
             // 11-May-2016, tatu: As per [core#277] we have efficient `writeArray(...)` available
             g.writeArray(value, 0, value.length);
         }
