@@ -7,6 +7,7 @@ import java.util.TreeMap;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.base.ParserMinimalBase;
+import com.fasterxml.jackson.core.io.NumberInput;
 import com.fasterxml.jackson.core.json.JsonWriteContext;
 import com.fasterxml.jackson.core.util.ByteArrayBuilder;
 import com.fasterxml.jackson.core.util.JacksonFeatureSet;
@@ -19,15 +20,10 @@ import com.fasterxml.jackson.databind.*;
  * ones; but not significantly less efficient for larger), highly efficient
  * for linear iteration and appending. Implemented as segmented/chunked
  * linked list of tokens; only modifications are via appends.
- *<p>
- * Note that before version 2.0, this class was located in the "core"
- * bundle, not data-binding; but since it was only used by data binding,
- * was moved here to reduce size of core package
  */
 public class TokenBuffer
-/* Won't use JsonGeneratorBase, to minimize overhead for validity
- * checking
- */
+// Won't use JsonGeneratorBase, to minimize overhead for validity
+// checking
     extends JsonGenerator
 {
     protected final static int DEFAULT_GENERATOR_FEATURES = JsonGenerator.Feature.collectDefaults();
@@ -196,7 +192,10 @@ public class TokenBuffer
      *</pre>
      *
      * @since 2.9
+     *
+     * @deprecated Since 2.13: use {@link DeserializationContext#bufferAsCopyOfValue} instead.
      */
+    @Deprecated // since 2.13
     public static TokenBuffer asCopyOfValue(JsonParser p) throws IOException {
         TokenBuffer b = new TokenBuffer(p);
         b.copyCurrentStructure(p);
@@ -299,6 +298,17 @@ public class TokenBuffer
         return _first.type(0);
     }
 
+    /**
+     * Accessor for checking whether this buffer has one or more tokens
+     * or not.
+     *
+     * @return True if this buffer instance has no tokens
+     *
+     * @since 2.13
+     */
+    public boolean isEmpty() {
+        return (_appendAt == 0) && (_first == _last);
+    }
     /*
     /**********************************************************
     /* Other custom methods not needed for implementing interfaces
@@ -435,9 +445,9 @@ public class TokenBuffer
                     } else if (n instanceof String) {
                         gen.writeNumber((String) n);
                     } else {
-                        throw new JsonGenerationException(String.format(
+                        _reportError(String.format(
                                 "Unrecognized value type for VALUE_NUMBER_FLOAT: %s, cannot serialize",
-                                n.getClass().getName()), gen);
+                                n.getClass().getName()));
                     }
                 }
                 break;
@@ -482,10 +492,9 @@ public class TokenBuffer
             copyCurrentStructure(p);
             return this;
         }
-        /* 28-Oct-2014, tatu: As per [databind#592], need to support a special case of starting from
-         *    FIELD_NAME, which is taken to mean that we are missing START_OBJECT, but need
-         *    to assume one did exist.
-         */
+        // 28-Oct-2014, tatu: As per [databind#592], need to support a special case of starting from
+        //    FIELD_NAME, which is taken to mean that we are missing START_OBJECT, but need
+        //    to assume one did exist.
         JsonToken t;
         writeStartObject();
         do {
@@ -595,6 +604,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
         return _generatorFeatures;
     }
 
+    // Note: cannot be removed until deprecated method removed from base class
     @Override
     @Deprecated
     public JsonGenerator setFeatureMask(int mask) {
@@ -929,10 +939,10 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             return;
         }
         if (_objectCodec == null) {
-            /* 28-May-2014, tatu: Tricky choice here; if no codec, should we
-             *   err out, or just embed? For now, do latter.
-             */
-//          throw new JsonMappingException("No ObjectCodec configured for TokenBuffer, writeObject() called");
+            // 28-May-2014, tatu: Tricky choice here; if no codec, should we
+            //   err out, or just embed? For now, do latter.
+
+//          throw new XxxException("No ObjectCodec configured for TokenBuffer, writeObject() called");
             _appendValue(JsonToken.VALUE_EMBEDDED_OBJECT, value);
         } else {
             _objectCodec.writeValue(this, value);
@@ -1488,7 +1498,10 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
                 boolean hasNativeTypeIds, boolean hasNativeObjectIds,
                 JsonStreamContext parentContext)
         {
-            super(0);
+            // 25-Jun-2022, tatu: Ideally would pass parser flags along (as
+            //    per [databund#3528]) but for now make sure not to clear the flags
+            //    but let defaults be used
+            super();
             _segment = firstSeg;
             _segmentPtr = -1; // not yet read
             _codec = codec;
@@ -1835,9 +1848,9 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             if (value instanceof String) {
                 String str = (String) value;
                 if (str.indexOf('.') >= 0) {
-                    return Double.parseDouble(str);
+                    return NumberInput.parseDouble(str, isEnabled(StreamReadFeature.USE_FAST_DOUBLE_PARSER));
                 }
-                return Long.parseLong(str);
+                return NumberInput.parseLong(str);
             }
             if (value == null) {
                 return null;
@@ -1935,7 +1948,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
 
         @Override
         @SuppressWarnings("resource")
-        public byte[] getBinaryValue(Base64Variant b64variant) throws IOException, JsonParseException
+        public byte[] getBinaryValue(Base64Variant b64variant) throws IOException
         {
             // First: maybe we some special types?
             if (_currToken == JsonToken.VALUE_EMBEDDED_OBJECT) {
@@ -2010,7 +2023,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             return _segment.get(_segmentPtr);
         }
 
-        protected final void _checkIsNumber() throws JsonParseException
+        protected final void _checkIsNumber() throws JacksonException
         {
             if (_currToken == null || !_currToken.isNumeric()) {
                 throw _constructError("Current token ("+_currToken+") not numeric, cannot use numeric value accessors");
@@ -2018,7 +2031,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
         }
 
         @Override
-        protected void _handleEOF() throws JsonParseException {
+        protected void _handleEOF() {
             _throwInternal();
         }
     }
