@@ -204,6 +204,45 @@ public class ObjectMapperTest extends BaseMapTest
         assertNotNull(result);
     }
 
+    public void testCopyWith() throws JsonProcessingException {
+        ObjectMapper mapper = new ObjectMapper();
+        //configuring some settings to non-defaults
+        mapper.configure(DeserializationFeature.FAIL_ON_NULL_FOR_PRIMITIVES, true);
+        mapper.configure(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL, true);
+        mapper.configure(SerializationFeature.INDENT_OUTPUT, true);
+        mapper.configure(SerializationFeature.FAIL_ON_SELF_REFERENCES, true);
+        JsonFactory newFactory = JsonFactory.builder()
+            .configure(JsonFactory.Feature.USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING, false)
+            .build();
+        ObjectMapper copiedMapper = mapper.copyWith(newFactory);
+        String json = "{ \"color\" : \"Black\", \"free\" : \"true\", \"pages\" : \"204.04\" }";
+        JsonNode readResult = copiedMapper.readTree(json);
+        // validate functionality
+        assertEquals("Black", readResult.get("color").asText());
+        assertEquals(true, readResult.get("free").asBoolean());
+        assertEquals(204, readResult.get("pages").asInt());
+        String readResultAsString = "{\n  \"color\" : \"Black\",\n  \"free\" : \"true\",\n  \"pages\" : \"204.04\"\n}";
+        assertEquals(readResultAsString, mapper.writeValueAsString(readResult));
+
+        // validate properties
+        Boolean mapperConfig1 = mapper._deserializationConfig.isEnabled(DeserializationFeature.FAIL_ON_NULL_FOR_PRIMITIVES);
+        Boolean copiedMapperConfig1 = copiedMapper._deserializationConfig.isEnabled(DeserializationFeature.FAIL_ON_NULL_FOR_PRIMITIVES);
+        Boolean mapperConfig2 = mapper._deserializationConfig.isEnabled(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL);
+        Boolean copiedMapperConfig2 = copiedMapper._deserializationConfig.isEnabled(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL);
+        Boolean mapperConfig3 = mapper._serializationConfig.isEnabled(SerializationFeature.INDENT_OUTPUT);
+        Boolean copiedMapperConfig3 = copiedMapper._serializationConfig.isEnabled(SerializationFeature.INDENT_OUTPUT);
+        Boolean mapperConfig4 = mapper._serializationConfig.isEnabled(SerializationFeature.FAIL_ON_SELF_REFERENCES);
+        Boolean copiedMapperConfig4 = copiedMapper._serializationConfig.isEnabled(SerializationFeature.FAIL_ON_SELF_REFERENCES);
+        assertNotSame(mapper.getFactory(), copiedMapper.getFactory());
+        assertSame(mapperConfig1, copiedMapperConfig1);
+        assertSame(mapperConfig2, copiedMapperConfig2);
+        assertSame(mapperConfig3, copiedMapperConfig3);
+        assertSame(mapperConfig4, copiedMapperConfig4);
+        assertNotSame(mapper.getFactory().isEnabled(JsonFactory.Feature.USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING),
+            copiedMapper.getFactory().isEnabled(JsonFactory.Feature.USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING)
+        );
+    }
+
     public void testFailedCopy() throws Exception
     {
         NoCopyMapper src = new NoCopyMapper();
