@@ -146,7 +146,7 @@ public class AnySetterTest
             props.put(name, value);
         }
     }
-    
+
     static class JsonAnySetterOnMap {
         public int id;
 
@@ -163,7 +163,7 @@ public class AnySetterTest
         public int id;
 
         @JsonAnySetter
-        protected HashMap<String, String> other;
+        protected Map<String, String> other;
 
         @JsonAnyGetter
         public Map<String, String> any() {
@@ -171,6 +171,14 @@ public class AnySetterTest
         }
     }
 
+    @SuppressWarnings("serial")
+    static class CustomMap extends LinkedHashMap<String, String> { }
+
+    static class JsonAnySetterOnCustomNullMap {
+        @JsonAnySetter
+        public CustomMap other;
+    }
+
     static class MyGeneric<T>
     {
         private String staticallyMappedProperty;
@@ -344,20 +352,33 @@ public class AnySetterTest
         assertTrue(ob instanceof Impl);
         assertEquals("xyz", ((Impl) ob).value);
     }
-    
-	public void testJsonAnySetterOnMap() throws Exception {
+
+    public void testJsonAnySetterOnMap() throws Exception {
 		JsonAnySetterOnMap result = MAPPER.readValue("{\"id\":2,\"name\":\"Joe\", \"city\":\"New Jersey\"}",
 		        JsonAnySetterOnMap.class);
 		assertEquals(2, result.id);
 		assertEquals("Joe", result.other.get("name"));
 		assertEquals("New Jersey", result.other.get("city"));
-	}
+    }
 
-	public void testJsonAnySetterOnNullMap() throws Exception {
-		JsonAnySetterOnNullMap result = MAPPER.readValue("{\"id\":2,\"name\":\"Joe\", \"city\":\"New Jersey\"}",
-		        JsonAnySetterOnNullMap.class);
-		assertEquals(2, result.id);
-		assertNull(result.other);
+    public void testJsonAnySetterOnNullMap() throws Exception {
+        final String DOC = a2q("{'id':2,'name':'Joe', 'city':'New Jersey'}");
+        JsonAnySetterOnNullMap result = MAPPER.readValue(DOC,
+                JsonAnySetterOnNullMap.class);
+        assertEquals(2, result.id);
+        // 01-Aug-2022, tatu: As per [databind#3559] should "just work"...
+        assertNotNull(result.other);
+        assertEquals("Joe", result.other.get("name"));
+        assertEquals("New Jersey", result.other.get("city"));
+
+        // But not with unknown "special" maps
+        try {
+            MAPPER.readValue(DOC, JsonAnySetterOnCustomNullMap.class);
+            fail("Should not pass");
+        } catch (DatabindException e) {
+            verifyException(e, "Cannot create an instance of");
+            verifyException(e, "for use as \"any-setter\" 'other'");
+        }
     }
 
     final static String UNWRAPPED_JSON_349 = a2q(
