@@ -3,6 +3,7 @@ package com.fasterxml.jackson.databind.deser.creators;
 import com.fasterxml.jackson.annotation.*;
 
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.exc.MismatchedInputException;
 
 public class RequiredCreatorTest extends BaseMapTest
 {
@@ -62,23 +63,23 @@ public class RequiredCreatorTest extends BaseMapTest
         FascistPoint p;
 
         // First: fine if both params passed
-        p = POINT_READER.readValue(aposToQuotes("{'y':2,'x':1}"));
+        p = POINT_READER.readValue(a2q("{'y':2,'x':1}"));
         assertEquals(1, p.x);
         assertEquals(2, p.y);
-        p = POINT_READER.readValue(aposToQuotes("{'x':3,'y':4}"));
+        p = POINT_READER.readValue(a2q("{'x':3,'y':4}"));
         assertEquals(3, p.x);
         assertEquals(4, p.y);
 
         // also fine if 'y' is MIA
-        p = POINT_READER.readValue(aposToQuotes("{'x':3}"));
+        p = POINT_READER.readValue(a2q("{'x':3}"));
         assertEquals(3, p.x);
         assertEquals(0, p.y);
 
         // but not so good if 'x' missing
         try {
-            POINT_READER.readValue(aposToQuotes("{'y':3}"));
+            POINT_READER.readValue(a2q("{'y':3}"));
             fail("Should not pass");
-        } catch (JsonMappingException e) {
+        } catch (MismatchedInputException e) {
             verifyException(e, "Missing required creator property 'x' (index 0)");
         }
     }
@@ -88,16 +89,16 @@ public class RequiredCreatorTest extends BaseMapTest
         FascistPoint p;
 
         // as per above, ok to miss 'y' with default settings:
-        p = POINT_READER.readValue(aposToQuotes("{'x':2}"));
+        p = POINT_READER.readValue(a2q("{'x':2}"));
         assertEquals(2, p.x);
         assertEquals(0, p.y);
 
         // but not if global checks desired
         ObjectReader r = POINT_READER.with(DeserializationFeature.FAIL_ON_MISSING_CREATOR_PROPERTIES);
         try {
-            r.readValue(aposToQuotes("{'x':6}"));
+            r.readValue(a2q("{'x':6}"));
             fail("Should not pass");
-        } catch (JsonMappingException e) {
+        } catch (MismatchedInputException e) {
             verifyException(e, "Missing creator property 'y' (index 1)");
         }
     }
@@ -105,11 +106,11 @@ public class RequiredCreatorTest extends BaseMapTest
     // [databind#2591]
     public void testRequiredViaParameter2591() throws Exception
     {
-        final String input = aposToQuotes("{'status':'OK', 'message':'Sent Successfully!'}");
+        final String input = a2q("{'status':'OK', 'message':'Sent Successfully!'}");
         try {
             /*LoginUserResponse resp =*/ MAPPER.readValue(input, LoginUserResponse.class);
             fail("Shoud not pass");
-        } catch (JsonMappingException e) {
+        } catch (MismatchedInputException e) {
             verifyException(e, "Missing required creator property 'otp'");
         }
     }
