@@ -184,7 +184,7 @@ public class TestCreators3 extends BaseMapTest
         final ObjectMapper mapper = newJsonMapper();
         mapper.setAnnotationIntrospector(new MyParamIntrospector());
 
-        MultiCtor bean = mapper.readValue(aposToQuotes("{'a':'123','b':'foo'}"), MultiCtor.class);
+        MultiCtor bean = mapper.readValue(a2q("{'a':'123','b':'foo'}"), MultiCtor.class);
         assertNotNull(bean);
         assertEquals("123", bean._a);
         assertEquals("foo", bean._b);
@@ -192,7 +192,7 @@ public class TestCreators3 extends BaseMapTest
 
     // [databind#1853]
     public void testSerialization() throws Exception {
-        assertEquals(quote("testProduct"),
+        assertEquals(q("testProduct"),
                 MAPPER.writeValueAsString(new Product1853(false, "testProduct")));
     }
 
@@ -203,7 +203,7 @@ public class TestCreators3 extends BaseMapTest
 
     public void testDeserializationFromString() throws Exception {
         assertEquals("DELEG:testProduct",
-                MAPPER.readValue(quote("testProduct"), Product1853.class).getName());
+                MAPPER.readValue(q("testProduct"), Product1853.class).getName());
     }
 }
 
