@@ -12,6 +12,8 @@ import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
 import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
+import com.fasterxml.jackson.databind.exc.InvalidDefinitionException;
+import com.fasterxml.jackson.databind.exc.MismatchedInputException;
 
 @SuppressWarnings("serial")
 public class MapDeserializationTest
@@ -25,7 +27,7 @@ public class MapDeserializationTest
         extends HashMap<Object,Object>
     {
         // No default ctor, nor @JsonCreators
-        public BrokenMap(boolean dummy) { super(); }
+        public BrokenMap(boolean dummy, boolean dummy2) { super(); }
     }
 
     @JsonDeserialize(using=CustomMapDeserializer.class)
@@ -88,7 +90,7 @@ public class MapDeserializationTest
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = new ObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
     public void testBigUntypedMap() throws Exception
     {
@@ -104,6 +106,9 @@ public class MapDeserializationTest
         }
         String json = MAPPER.writeValueAsString(map);
         Object bound = MAPPER.readValue(json, Object.class);
+
+        assertTrue(bound instanceof Map<?,?>);
+        assertEquals(map.size(), ((Map<?,?>) bound).size());
         assertEquals(map, bound);
     }
     
@@ -191,7 +196,7 @@ public class MapDeserializationTest
     {
         ObjectMapper m = new ObjectMapper();
         m.configure(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT, true);
-        Map<?,?> result = m.readValue(quote(""), Map.class);
+        Map<?,?> result = m.readValue(q(""), Map.class);
         assertNull(result);
     }
 
@@ -449,7 +454,7 @@ public class MapDeserializationTest
     public void testKeyWithCreator() throws Exception
     {
         // first, key should deserialize normally:
-        KeyType key = MAPPER.readValue(quote("abc"), KeyType.class);
+        KeyType key = MAPPER.readValue(q("abc"), KeyType.class);
         assertEquals("abc", key.value);
 
         Map<KeyType,Integer> map = MAPPER.readValue("{\"foo\":3}", new TypeReference<Map<KeyType,Integer>>() {} );
@@ -459,7 +464,7 @@ public class MapDeserializationTest
     }
 
     public void testClassKeyMap() throws Exception {
-        ClassStringMap map = MAPPER.readValue(aposToQuotes("{'java.lang.String':'foo'}"),
+        ClassStringMap map = MAPPER.readValue(a2q("{'java.lang.String':'foo'}"),
                 ClassStringMap.class);
         assertNotNull(map);
         assertEquals(1, map.size());
@@ -467,7 +472,7 @@ public class MapDeserializationTest
     }
 
     public void testcharSequenceKeyMap() throws Exception {
-        String JSON = aposToQuotes("{'a':'b'}");
+        String JSON = a2q("{'a':'b'}");
         Map<CharSequence,String> result = MAPPER.readValue(JSON, new TypeReference<Map<CharSequence,String>>() { });
         assertNotNull(result);
         assertEquals(1, result.size());
@@ -486,7 +491,7 @@ public class MapDeserializationTest
      */
     public void testMapWithDeserializer() throws Exception
     {
-        CustomMap result = MAPPER.readValue(quote("xyz"), CustomMap.class);
+        CustomMap result = MAPPER.readValue(q("xyz"), CustomMap.class);
         assertEquals(1, result.size());
         assertEquals("xyz", result.get("x"));
     }
@@ -503,7 +508,7 @@ public class MapDeserializationTest
             Object result = MAPPER.readValue("[ 1, 2 ]", 
                     new TypeReference<Map<String,String>>() { });
             fail("Expected an exception, but got result value: "+result);
-        } catch (JsonMappingException jex) {
+        } catch (MismatchedInputException jex) {
             verifyException(jex, "START_ARRAY");
         }
     }
@@ -514,7 +519,7 @@ public class MapDeserializationTest
             BrokenMap result = MAPPER.readValue("{ \"a\" : 3 }", BrokenMap.class);
             // should never get here; assert added to remove compiler warning
             assertNull(result);
-        } catch (JsonMappingException e) {
+        } catch (InvalidDefinitionException e) {
             // instead, should get this exception:
             verifyException(e, "no default constructor found");
         }
