@@ -3,11 +3,9 @@ package com.fasterxml.jackson.databind.jsontype;
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
 
 import com.fasterxml.jackson.databind.BaseMapTest;
-import com.fasterxml.jackson.databind.JsonMappingException;
 import com.fasterxml.jackson.databind.MapperFeature;
 import com.fasterxml.jackson.databind.ObjectMapper;
-
-import java.io.IOException;
+import com.fasterxml.jackson.databind.exc.InvalidTypeIdException;
 
 public class TestBaseTypeAsDefault extends BaseMapTest
 {
@@ -15,16 +13,13 @@ public class TestBaseTypeAsDefault extends BaseMapTest
     static class Parent {
     }
 
-
     static class Child extends Parent {
     }
 
-
     @JsonTypeInfo(use = JsonTypeInfo.Id.CLASS, property = "@class", defaultImpl = ChildOfChild.class)
     static abstract class AbstractParentWithDefault {
     }
 
-
     static class ChildOfAbstract extends AbstractParentWithDefault {
     }
 
@@ -44,48 +39,48 @@ public class TestBaseTypeAsDefault extends BaseMapTest
     protected ObjectMapper MAPPER_WITHOUT_BASE = jsonMapperBuilder()
             .disable(MapperFeature.USE_BASE_TYPE_AS_DEFAULT_IMPL)
             .build();
-    
-    public void testPositiveForParent() throws IOException {
+
+    public void testPositiveForParent() throws Exception {
         Object o = MAPPER_WITH_BASE.readerFor(Parent.class).readValue("{}");
         assertEquals(o.getClass(), Parent.class);
     }
 
-    public void testPositiveForChild() throws IOException {
+    public void testPositiveForChild() throws Exception {
         Object o = MAPPER_WITH_BASE.readerFor(Child.class).readValue("{}");
         assertEquals(o.getClass(), Child.class);
     }
 
-    public void testNegativeForParent() throws IOException {
+    public void testNegativeForParent() throws Exception {
         try {
             /*Object o =*/ MAPPER_WITHOUT_BASE.readerFor(Parent.class).readValue("{}");
             fail("Should not pass");
-        } catch (JsonMappingException ex) {
-            assertTrue(ex.getMessage().contains("missing type id property '@class'"));
+        } catch (InvalidTypeIdException e) {
+            assertTrue(e.getMessage().contains("missing type id property '@class'"));
         }
     }
 
-    public void testNegativeForChild() throws IOException {
+    public void testNegativeForChild() throws Exception {
         try {
             /*Object o =*/ MAPPER_WITHOUT_BASE.readerFor(Child.class).readValue("{}");
             fail("Should not pass");
-        } catch (JsonMappingException ex) {
+        } catch (InvalidTypeIdException ex) {
             assertTrue(ex.getMessage().contains("missing type id property '@class'"));
         }
     }
 
-    public void testConversionForAbstractWithDefault() throws IOException {
+    public void testConversionForAbstractWithDefault() throws Exception {
         // should pass shouldn't it?
         Object o = MAPPER_WITH_BASE.readerFor(AbstractParentWithDefault.class).readValue("{}");
         assertEquals(o.getClass(), ChildOfChild.class);
     }
 
-    public void testPositiveWithTypeSpecification() throws IOException {
+    public void testPositiveWithTypeSpecification() throws Exception {
         Object o = MAPPER_WITH_BASE.readerFor(Parent.class)
                 .readValue("{\"@class\":\""+Child.class.getName()+"\"}");
         assertEquals(o.getClass(), Child.class);
     }
 
-    public void testPositiveWithManualDefault() throws IOException {
+    public void testPositiveWithManualDefault() throws Exception {
         Object o = MAPPER_WITH_BASE.readerFor(ChildOfAbstract.class).readValue("{}");
 
         assertEquals(o.getClass(), ChildOfChild.class);
