@@ -174,9 +174,8 @@ public class TestDefaultForObject
         try {
             m.readValue(serial, AbstractBean[].class);
             fail("Should have failed");
-        } catch (JsonMappingException e) {
-            // let's use whatever is currently thrown exception... may change tho
-            verifyException(e, "cannot construct");
+        } catch (InvalidDefinitionException e) {
+            verifyException(e, "cannot construct instance of");
         }
         // and then that we will succeed with default type info
         m = JsonMapper.builder()
@@ -417,14 +416,14 @@ public class TestDefaultForObject
                 .activateDefaultTyping(NoCheckSubTypeValidator.instance,
                         ObjectMapper.DefaultTyping.NON_FINAL)
                 .build();
-        assertEquals(aposToQuotes("{'name':'abc'}"),
+        assertEquals(a2q("{'name':'abc'}"),
                 mapper.writeValueAsString(new FinalStringBean("abc")));
 
         mapper = JsonMapper.builder()
                 .activateDefaultTyping(NoCheckSubTypeValidator.instance,
                         ObjectMapper.DefaultTyping.EVERYTHING)
                 .build();
-        assertEquals(aposToQuotes("['"+FinalStringBean.class.getName()+"',{'name':'abc'}]"),
+        assertEquals(a2q("['"+FinalStringBean.class.getName()+"',{'name':'abc'}]"),
                 mapper.writeValueAsString(new FinalStringBean("abc")));
     }
 
