@@ -5,6 +5,7 @@ import java.util.Comparator;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.io.SerializedString;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.testutil.NoCheckSubTypeValidator;
 import com.fasterxml.jackson.databind.util.RawValue;
 
 /**
@@ -27,6 +28,11 @@ public class TestJsonNode extends NodeTestBase
         assertEquals("false", f.asText());
         assertEquals(JsonToken.VALUE_FALSE, f.asToken());
 
+        assertFalse(f.isNumber());
+        assertFalse(f.canConvertToInt()); 
+        assertFalse(f.canConvertToLong()); 
+        assertFalse(f.canConvertToExactIntegral()); 
+
         // and ditto for true
         BooleanNode t = BooleanNode.getTrue();
         assertNotNull(t);
@@ -68,6 +74,11 @@ public class TestJsonNode extends NodeTestBase
         byte[] data = new byte[3];
         data[1] = (byte) 3;
         BinaryNode n = BinaryNode.valueOf(data, 1, 1);
+        assertFalse(n.isNumber());
+        assertFalse(n.canConvertToInt()); 
+        assertFalse(n.canConvertToLong()); 
+        assertFalse(n.canConvertToExactIntegral()); 
+
         data[2] = (byte) 3;
         BinaryNode n2 = BinaryNode.valueOf(data, 2, 1);
         assertTrue(n.equals(n2));
@@ -83,8 +94,8 @@ public class TestJsonNode extends NodeTestBase
         assertStandardEquals(n);
         assertEquals(n, new POJONode("x"));
         assertEquals("x", n.asText());
-        // not sure if this is what it'll remain as but:
-        assertEquals("x", n.toString());
+        // 10-Dec-2018, tatu: With 2.10, should serialize same as via ObjectMapper/ObjectWriter
+        assertEquals("\"x\"", n.toString());
 
         assertEquals(new POJONode(null), new POJONode(null));
 
@@ -177,7 +188,7 @@ public class TestJsonNode extends NodeTestBase
     public void testArrayWithDefaultTyping() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper()
-            .enableDefaultTyping();
+            .activateDefaultTyping(NoCheckSubTypeValidator.instance);
 
         JsonNode array = mapper.readTree("[ 1, 2 ]");
         assertTrue(array.isArray());
