@@ -230,7 +230,7 @@ public class AnyGetterTest extends BaseMapTest
     public void testAnyDisabling() throws Exception
     {
         String json = MAPPER.writeValueAsString(new NotEvenAnyBean());
-        assertEquals(aposToQuotes("{'value':42}"), json);
+        assertEquals(a2q("{'value':42}"), json);
     }
 
     // Trying to repro [databind#577]
@@ -238,7 +238,7 @@ public class AnyGetterTest extends BaseMapTest
     {
         MapAsAny input = new MapAsAny();
         input.add("bar", null);
-        assertEquals(aposToQuotes("{'bar':null}"),
+        assertEquals(a2q("{'bar':null}"),
                 MAPPER.writeValueAsString(input));
     }
 
@@ -304,11 +304,11 @@ public class AnyGetterTest extends BaseMapTest
     // [databind#2592]
     public void testAnyGetterConfigIncludeNonEmpty() throws Exception
     {
-        ObjectMapper mapper = new ObjectMapper();
-        mapper.configOverride(Map.class).setInclude(JsonInclude.Value.construct(
-                JsonInclude.Include.USE_DEFAULTS,
-                JsonInclude.Include.NON_EMPTY
-        ));
+        final ObjectMapper mapper = jsonMapperBuilder()
+                .withConfigOverride(Map.class, incl -> incl.setInclude(
+                    JsonInclude.Value.construct(JsonInclude.Include.USE_DEFAULTS,
+                    JsonInclude.Include.NON_EMPTY)))
+                .build();
         Bean2592NoAnnotations input = new Bean2592NoAnnotations();
         input.add("non-empty", "property");
         input.add("empty", "");
