@@ -4,14 +4,12 @@ import java.io.IOException;
 
 import com.fasterxml.jackson.annotation.JsonProperty;
 import com.fasterxml.jackson.databind.BaseMapTest;
-import com.fasterxml.jackson.databind.type.TypeFactory;
 
 
 // https://github.com/FasterXML/jackson-databind/issues/1647
-public class TestTypeFactoryWithRecursiveTypes extends BaseMapTest {
-
-    static interface IFace<T> {
-    }
+public class TestTypeFactoryWithRecursiveTypes extends BaseMapTest
+{
+    static interface IFace<T> { }
 
     static class Base implements IFace<Sub> {
         @JsonProperty int base = 1;
